package com.crunii.micro.service.appsecond.readListener;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.excel.util.ListUtils;
import com.alibaba.fastjson.JSON;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.QczdIndustryField;
import com.crunii.micro.domain.entity.main.QczdTeamStay;
import com.crunii.micro.service.appsecond.api.dto.data.ExcelMistakeData;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpacePassTeamCUExportDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpacePassTeamCUReqDTO;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.BusinessStateEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.DemandEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.SexEnum;
import com.crunii.micro.service.appsecond.enums.YouthSpaceEnums.CommEnum;
import com.crunii.micro.service.appsecond.service.impl.SpacePassTeamServiceImpl;
import java.util.*;
import java.util.stream.Collectors;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;

/**
 * @Author thw
 * @date 2023/12/14
 * @description
 */
@Slf4j
public class ImportTeamReadListener extends AnalysisEventListener<SpacePassTeamCUExportDTO> {

  /**
   * 每隔5条存储数据库，实际使用中可以100条，然后清理list ，方便内存回收
   */
  private static final int BATCH_COUNT = 600;
  /**
   * 缓存的数据(业务规定：单个文件数据量不允许超过500条。)
   */
  private List<QczdTeamStay> cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);

  /**
   * 团队行业领域实例数据
   */
  private List<QczdIndustryField> industryFields = ListUtils.newArrayList();

  private SpacePassTeamServiceImpl service;
  private Worker worker;

  /**
   * 表头属性名
   */
  private List<String> includeFieldList;

  /**
   * 错误的数据
   */
  List<SpacePassTeamCUExportDTO> errContentList;

  private int countNum = 0;


  public ImportTeamReadListener(SpacePassTeamServiceImpl service, List<String> includeFieldList,
      Worker worker, List<SpacePassTeamCUExportDTO> errContentList) {
    this.service = service;
    this.includeFieldList = includeFieldList;
    this.worker = worker;
    this.errContentList = errContentList;
  }


  @Override
  public void invokeHead(Map<Integer, CellData> headMap, AnalysisContext context) {
    for (Integer key : headMap.keySet()) {
      CellData data = headMap.get(key);
      // 由于只存在一行表头
      if (!includeFieldList.contains(data.getStringValue())) {
        throw new BusinessException("导入模板错误!");
      }
    }
  }


  @Override
  public void invoke(SpacePassTeamCUExportDTO dto, AnalysisContext context) {
    log.info("解析到第" + countNum++ + "条数据:{}", JSON.toJSONString(dto));
    // 参数校验
    checkExportDateContent(dto, countNum);

    // 校验数据是否重复（表格和数据库）
    List<QczdTeamStay> existTeams = service.qczdTeamStayRepo.findByApplicantNameAndPhone(
        dto.getApplicantName(), dto.getPhone());
    if (ObjectUtil.isNotEmpty(existTeams)) {
//      dto.setErrMessage("第" + countNum + "数据参数校验失败: {" + "申请人姓名与联系人电话:[" + dto.getApplicantName() + "-" + dto.getPhone()
//          + "]在数据库中已存在! }");
      // 将存在错误的对象添加到错误队列中
      errContentList.add(dto);
    }

    if (ObjectUtil.isNotEmpty(cachedDataList)) {
      Iterator<QczdTeamStay> teamStayIterator = cachedDataList.iterator();
      while (teamStayIterator.hasNext()) {
        QczdTeamStay qczdTeamStay = teamStayIterator.next();
        if (ObjectUtil.isNotEmpty(dto.getApplicantName()) && ObjectUtil.isNotEmpty(dto.getPhone())
            && dto.getApplicantName().equals(qczdTeamStay.getApplicantName()) && dto.getPhone()
            .equals(qczdTeamStay.getPhone())) {
//          dto.setErrMessage("第" + countNum + "数据参数校验失败: {" +
//              "申请人姓名与联系人电话:[" + dto.getApplicantName() + "-" + dto.getPhone()
//                  + "]在表中重复! }");
          // 将存在错误的DTO对象添加到错误队列中
          errContentList.add(dto);

          // 将在关联表行业领域缓存队列中的的重复数据删除
          SpacePassTeamCUExportDTO exportDTO = new SpacePassTeamCUExportDTO();
          BeanUtils.copyProperties(qczdTeamStay, exportDTO);
          // 处理需要转换的内容
          exportDTO.setBusinessLicenseName(
              BusinessStateEnum.getBusinessStateEnum(qczdTeamStay.getBusinessLicense()).getDesc());
          exportDTO.setWorkNeedName(DemandEnum.getDemandEnum(qczdTeamStay.getWorkNeed()).getDesc());
          exportDTO.setOfficeNeedName(
              DemandEnum.getDemandEnum(qczdTeamStay.getOfficeNeed()).getDesc());

          List<QczdIndustryField> errFieldList = new ArrayList<>();
          Iterator<QczdIndustryField> fieldIterator = industryFields.iterator();
          while (fieldIterator.hasNext()) {
            QczdIndustryField field = fieldIterator.next();
            if (field.getTeamId().equals(qczdTeamStay.getId())) {
              errFieldList.add(field);
              fieldIterator.remove();
            }
          }
          // 数据反包装回DTO
          if (ObjectUtil.isNotEmpty(errFieldList)) {
            List<String> fieldList = errFieldList.stream().map(QczdIndustryField::getIndustryField)
                .collect(Collectors.toList());
//            exportDTO.setIndustryField(fieldList);
          }
          // 将重复数据team实例从cache队列中移除并添加到错误队列
          teamStayIterator.remove();
          errContentList.add(exportDTO);
        }
      }
    }

    QczdTeamStay qczdTeamStay = new QczdTeamStay();
    BeanUtils.copyProperties(dto, qczdTeamStay);
    // 特殊属性处理
    qczdTeamStay.setBusinessLicense(
        BusinessStateEnum.getBusinessStateEnumDesc(dto.getBusinessLicenseName()).getNum());
    qczdTeamStay.setWorkNeed(DemandEnum.getDemandEnumDesc(dto.getWorkNeedName()).getNum());
    qczdTeamStay.setOfficeNeed(CommEnum.getCommEnumDesc(dto.getOfficeNeedName()).getNum());
    // 设置新增时团队实例必要属性
//    service.createTeamNeedSetMethod(dto.getIdCode(), qczdTeamStay, worker,Boolean.FALSE);
//    service.createIndustryFieldContent(industryFields, dto.getIndustryField(), qczdTeamStay,
//        Boolean.TRUE);
    cachedDataList.add(qczdTeamStay);
    // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
    if (cachedDataList.size() >= BATCH_COUNT) {
      saveTeamData();
      // 存储完成清理 list
      cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
      industryFields = ListUtils.newArrayList();
    }
  }

  /**
   * 参数校验
   */
  public void checkExportDateContent(SpacePassTeamCUExportDTO bo, int countNum) {
    try {
      if (ObjectUtil.isEmpty(bo.getBusinessLicenseName())) {
        throw new BusinessException("有营业执照必填.");
      }
      if (ObjectUtil.isEmpty(BusinessStateEnum.getBusinessStateEnumDesc(bo.getBusinessLicenseName()))) {
        throw new BusinessException("营业执照必填(类型:无、有)!");
      }

      if (ObjectUtil.isNotEmpty(bo.getBusinessLicenseName()) && BusinessStateEnum.YES_BUSINESS.getDesc()
          .equals(bo.getBusinessLicenseName())) {
        if (ObjectUtil.isEmpty(bo.getCompanyName()) || bo.getCompanyName().length() > 50) {
          throw new BusinessException("有营业执照时,企业名称必填且字符长度不能超过50字符.");
        }
      }

      if (ObjectUtil.isEmpty(DemandEnum.getDemandEnumDesc(bo.getWorkNeedName()))) {
        throw new BusinessException("工位需求必填(类型:否、是)!");
      }

      if (ObjectUtil.isEmpty(DemandEnum.getDemandEnumDesc(bo.getOfficeNeedName()))) {
        throw new BusinessException("办公室需求必填(类型:否、是)!");
      }

      if (ObjectUtil.isEmpty(SexEnum.getSexEnumDesc(bo.getSexName()))) {
        throw new BusinessException("性别必填(男、女)!");
      }

      if (ObjectUtil.isEmpty(bo.getNationName())) {
        throw new BusinessException("民族必填");
      }

      if (ObjectUtil.isEmpty(bo.getFormalSchoolingName())) {
        throw new BusinessException("学历必填");
      }

      if (ObjectUtil.isEmpty(bo.getPoliticalStatusName())) {
        throw new BusinessException("政治面貌必填");
      }

      // 共同参数校验
      SpacePassTeamCUReqDTO spacePassTeamCUReqDTO = new SpacePassTeamCUReqDTO();
      BeanUtils.copyProperties(bo, spacePassTeamCUReqDTO);

      List<ExcelMistakeData> mistakeData = new ArrayList<>();
//      service.checkParamsData(spacePassTeamCUReqDTO,mistakeData);
    } catch (Exception e) {
      log.error("第" + countNum + "数据参数校验失败:{}", e.getMessage());
//      bo.setErrMessage("第" + countNum + "数据参数校验失败: {" + e.getMessage() + " }");
      // 将存在错误的对象添加到错误队列中
      errContentList.add(bo);
    }
  }


  /**
   * 所有数据解析完成了 都会来调用
   * @param context
   */
  @Override
  public void doAfterAllAnalysed(AnalysisContext context) {
    // 这里也要保存数据，确保最后遗留的数据也存储到数据库
    saveTeamData();
    log.info("所有数据解析完成！");
  }

  /**
   * 加上存储数据库
   */
  private void saveTeamData() {
    log.info("团队{}条数据，开始存储数据库！", cachedDataList.size());
    service.qczdTeamStayRepo.saveAll(cachedDataList);
    service.qczdIndustryFieldRepo.saveAll(industryFields);
    log.info("存储数据库成功！");
  }
}
