package com.crunii.micro.service.appsecond.service.impl;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.*;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.*;
import com.crunii.micro.service.appsecond.config.YouthHotConfigProperty;
import com.crunii.micro.service.appsecond.jdbc.AppMediumDao;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import com.crunii.micro.domain.entity.main.AppMedium;
import com.crunii.micro.domain.repo.main.AppMediumRepo;
import com.crunii.micro.service.appsecond.service.IAppMediumService;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class AppMediumServiceImpl implements IAppMediumService {

    private final AppMediumRepo baseRepo;
    private final AppMediumDao appMediumDao;
    private final YouthHotConfigProperty youthHotConfigProperty;

    private final Map<Long,AtomicInteger> attentionMediumMap = new HashMap<>(16);


    @Override
    public String createMedium(MediumCreateDto dto, Worker worker) {
        if(Objects.isNull(worker.getOrgId())){
            throw new BusinessException("该用户无组织信息，无法关联");
        }
        AppMedium one;
        // 不为空 则做更新处理
        int loginId = worker.getLoginId();
        Date now = new Date();
        if(Objects.nonNull(dto.getMediumId())){
            one = baseRepo.getOne(dto.getMediumId());
            if(Objects.isNull(one)){
                throw new BusinessException("该账号异常");
            }
            if(!one.getAssociatedUserId().equals(dto.getAssociatedUserId())){
                // 验证用户是否绑定过其他媒体账号
                validatedOtherMedium(dto.getAssociatedUserId());
            }
            if(!one.getOrgId().equals(dto.getOrgId()) || !one.getIdName().equals(dto.getIdName())){
                // 验证改组织下是否有这个媒体账号 org+name
                validatedIsCreateMedium(dto.getOrgId(),dto.getIdName());
            }
        }else {
            one = new AppMedium();
            one.setCreateBy(loginId);
            one.setCreateTime(now);
            // 验证用户是否绑定过其他媒体账号
            validatedOtherMedium(dto.getAssociatedUserId());
            // 验证改组织下是否有这个媒体账号 org+name
            validatedIsCreateMedium(dto.getOrgId(),dto.getIdName());
        }
        BeanUtils.copyProperties(dto,one);
        one.setUpdateBy(loginId);
        one.setUpdateTime(now);
        baseRepo.saveAndFlush(one);
        return "操作成功";
    }

    /**
     * 验证当前组织是否已创建该账号名称
     */
    private void validatedIsCreateMedium(Long orgId, String idName) {
        Integer count = appMediumDao.findIsCreateMedium(orgId,idName);
        if(Objects.nonNull(count)){
            throw new BusinessException("该组织已有该媒体账号");
        }
    }

    /**
     * 校验改账号是否关联其他媒体
     */
    private void validatedOtherMedium(Integer associatedUserId) {
        Integer count = appMediumDao.findIsOtherMedium(associatedUserId);
        if(Objects.nonNull(count)){
            throw new BusinessException("该账号下已绑定媒体账号");
        }
    }

    @Override
    public PageRsp<MediumVo> pageMedium(PageReq<MediumDto> req, Worker worker) {
        String orgId = worker.getOrgId();
        boolean superAdmin = isSuperAdmin(worker.getLoginId());
        if(!StringUtils.hasText(orgId) && !superAdmin){
            return new PageRsp<>();
        }
        PageRsp<MediumVo> mediumVoPageRsp = appMediumDao.pageMedium(req, orgId, superAdmin);
        List<MediumVo> result = mediumVoPageRsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return mediumVoPageRsp;
        }
        List<Long> orgIds = result.stream().map(MediumVo::getOrgId).distinct().collect(Collectors.toList());
        // 通过组织id查询当前审核员
        List<AuditOrgUser> users = appMediumDao.findAuditOrgUser(orgIds,youthHotConfigProperty.getAuditRoleId());
        if(CollectionUtils.isEmpty(users)){
            return mediumVoPageRsp;
        }
        Map<Long, List<AuditOrgUser>> auditOrgUserMap = users.stream().collect(Collectors.groupingBy(AuditOrgUser::getOrgId));
        for (MediumVo mediumVo : result) {
            List<AuditOrgUser> auditOrgUser = auditOrgUserMap.get(mediumVo.getOrgId());
            if(!CollectionUtils.isEmpty(auditOrgUser)){
                // 用户
                String userNames = auditOrgUser.stream().map(AuditOrgUser::getAuditUserName).collect(Collectors.joining(","));
                // 手机号
                String phones = auditOrgUser.stream().map(AuditOrgUser::getAuditUserPhone).collect(Collectors.joining(","));
                mediumVo.setAuditUserName(userNames);
                mediumVo.setAuditUserPhone(phones);
            }
        }
        return mediumVoPageRsp;
    }

    /**
     * 判断是否超级管理员
     */
    private boolean isSuperAdmin(int loginId) {
        List<Integer> superLoginIds = CommonConstants.SUPER_LOGIN_IDS;
        return superLoginIds.contains(loginId);
    }

    @Override
    public String attentionMedium(AttentionMediumDto dto) {
        AtomicInteger atomicInteger = attentionMediumMap.get(dto.getMediumId());
        if(Objects.isNull(atomicInteger)){
            atomicInteger = new AtomicInteger();
            attentionMediumMap.put(dto.getMediumId(),atomicInteger);
        }
        if(dto.isIncrease()){
            atomicInteger.incrementAndGet();
        }else {
            atomicInteger.decrementAndGet();
        }
        return "保存成功";
    }

    @Override
    public void flushAttentionMedium() {
        if(CollectionUtils.isEmpty(attentionMediumMap)){
            return;
        }
        Map<Long,AtomicInteger> copyMap = new HashMap<>(attentionMediumMap.size());
        copyMap.putAll(attentionMediumMap);
        attentionMediumMap.clear();
        Set<Long> set = copyMap.keySet();
        for (Long aLong : set) {
            AppMedium appMedium = baseRepo.getOne(aLong);
            if(Objects.isNull(appMedium)){
                continue;
            }
            Integer attention = appMedium.getAttention();
            attention += copyMap.get(aLong).get();
            if(attention <0){
                attention = 0;
            }
            appMediumDao.updateAttention(attention,aLong);
        }
    }


    @Override
    public String updateEnableMedium(EnableMediumDto dto, Worker worker) {
        appMediumDao.updateEnableMedium(dto,worker);
        return "更新成功";
    }

    @Override
    public String delMedium(Long mediumId, Worker worker) {
        AppMedium one = baseRepo.getOne(mediumId);
        if(Objects.isNull(one)){
            return "删除成功";
        }
        one.setDelFlag(1);
        one.setUpdateTime(new Date());
        one.setUpdateBy(worker.getLoginId());
        baseRepo.saveAndFlush(one);
        return "删除成功";
    }

    @Override
    public PageRsp<MediumHotVo> pageMediumHot(PageReq<MediumHotDto> req, Worker worker) {
        if(!StringUtils.hasText(worker.getOrgId())){
            return new PageRsp<>();
        }
        Long queryOrgId = req.getQuery().getOrgId();
        // 如果组织id为空 默认查询当前用户组织
        if(Objects.isNull(queryOrgId)){
            queryOrgId = Long.parseLong(worker.getOrgId());
        }
        boolean superAdmin = isSuperAdmin(worker.getLoginId());
        if(superAdmin){
            queryOrgId =  appMediumDao.findTopOrg().getOrgId();
        }
        // 验证是否有权限
        Integer count = appMediumDao.validatedWorkerFindOrg(worker.getOrgId(),queryOrgId);

        if(Objects.isNull(count) && !superAdmin){
            throw new BusinessException("无权限查看改组织");
        }
        return appMediumDao.pageMediumHot(req,queryOrgId);
    }

    @Override
    public PageRsp<MediumContentHotVo> pageMediumContentHot(PageReq<MediumContentHotDto> req, Worker worker) {
        return appMediumDao.pageMediumContentHot(req);
    }

    @Override
    public String exportExcel(MediumHotExcelDto dto, Worker worker) {
        if(!StringUtils.hasText(worker.getOrgId())){
            throw new BusinessException("当前用户无组织");
        }
        Integer orgTyp = 1;
        if(orgTyp.equals(dto.getType())){
            exportOrg(dto.getId());
        }else {
            exportMedium(dto.getId());
        }
        return "导出成功";
    }

    @Override
    public MediumOrgTree findMediumTree(Worker worker) {
        if(!StringUtils.hasText(worker.getOrgId())){
            return null;
        }
        boolean superAdmin = isSuperAdmin(worker.getLoginId());
        MediumOrgTree mediumOrgTree = appMediumDao.findTopOrg();
        if(mediumOrgTree.getOrgId().equals(Long.parseLong(worker.getOrgId())) || superAdmin){
            List<MediumOrgTree> allCity = appMediumDao.findAllCityAttention();
            if(CollectionUtils.isEmpty(allCity)){
                return null;
            }
            allCity.forEach(x->x.setLeaf(true));
            mediumOrgTree.setChildren(allCity);
        }else {
            // 不为空 则为重庆市下级
            mediumOrgTree = appMediumDao.findIsCityOrg(Long.parseLong(worker.getOrgId()));
            if(Objects.isNull(mediumOrgTree)){
                return null;
            }
            Integer attention = appMediumDao.findMyCityAttention(mediumOrgTree.getOrgId());
            // 为空 证明所属组织下无组织媒体
            if(Objects.isNull(attention)){
                return null;
            }
            mediumOrgTree.setAttention(attention);
            mediumOrgTree.setLeaf(true);
        }
        return mediumOrgTree;
    }

    @Override
    public List<OrgAuthTreeVo> findAuthTree(OrgAuthTreeDto dto, Worker worker) {
        List<OrgAuthTreeVo> list = new ArrayList<>();
        String orgId = worker.getOrgId();
        if(!StringUtils.hasText(orgId) && !CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())){
            return list;
        }
        OrgAuthTreeVo tree =  appMediumDao.findCqOrg(null);
        if(Objects.isNull(tree)){
            return list;
        }
        if(CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())){
            orgId = tree.getOrgId()+"";
        }
        if(!orgId.equals(tree.getOrgId()+"")){
            OrgAuthTreeVo cqOrg = appMediumDao.findCqOrg(Long.parseLong(orgId));
            cqOrg.setIsLeaf(true);
            list.add(cqOrg);
            return list;
        }
        if(dto.getParentOrgId() == 0){
            list.add(tree);
            return list;
        }
        if(dto.getParentOrgId().equals(tree.getOrgId())){
            return appMediumDao.findCqListOrgIds(tree.getOrgId());
        }
        return list;
    }

    private void exportMedium(Long mediumId) {
        List<MediumContentHotVo> list = appMediumDao.findMediumContent(mediumId);
        try {
            ExcelUtil.writeSingleExcel("媒体账号分析表",null,list,MediumContentHotVo.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }
    }

    /**
     * 导出组织信息
     */
    private void exportOrg(Long id) {
       List<MediumHotVo> list =  appMediumDao.findOrgInfo(id);
        try {
            ExcelUtil.writeSingleExcel("组织账号分析表",null,list,MediumHotVo.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }
    }

}
