package com.crunii.micro.service.appsecond.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.PicInfo;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.AttentionMediumDto;
import com.crunii.micro.service.appsecond.api.dto.appmultimodules.AppMultiModulesDto;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppMultiModulesHotResVo;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppMultiModulesHotVo;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppUserViewVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthContextVo;
import com.crunii.micro.service.appsecond.enums.AppSearchTypeEnum;
import com.crunii.micro.service.appsecond.jdbc.AppMultiModulesDao;
import com.crunii.micro.service.appsecond.jdbc.AppYouthHotDao;
import com.crunii.micro.service.appsecond.service.AppMultiModulesService;
import com.crunii.micro.service.appsecond.service.AppYouthHotService;
import com.crunii.micro.service.appsecond.service.IAppMediumService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.crunii.micro.domain.util.CurrWorkerUtil.getCurrWorker;
import static com.crunii.micro.service.appsecond.api.constants.AppSecondConstants.WATER_FALL_DEFAULT_HEIGHT;
import static com.crunii.micro.service.appsecond.api.constants.AppSecondConstants.WATER_FALL_DEFAULT_WIDTH;

/**
 * youth-cq.
 * 热搜/热榜/点赞/收藏/关注/浏览记录
 * @author leixb
 * @since 2023-12-07 19:30
 **/
@RequiredArgsConstructor
@Service
@Slf4j
public class AppMultiModulesServiceImpl implements AppMultiModulesService {
    
    private final AppMultiModulesDao appMultiModulesDao;
    
    private final AppUserCollectRepo appUserCollectRepo;
    
    private final AppContentManageRepo appContentManageRepo;
    
    private final AppUserLikeRepo appUserLikeRepo;
    
    private final AppUserFollowRepo appUserFollowRepo;
    
    private final AppUserViewRepo appUserViewRepo;
    
    private final IAppMediumService iAppMediumService;
    
    private final AppYouthHotDao appYouthHotDao;
    
    /** 避免循环依赖 */
    @Resource(name = "appYouthHotServiceImpl")
    private AppYouthHotService appYouthHotService;
    
    @Value("${upload.url}")
    private String urlPrefix;
    
    /**
     * 点赞/收藏/关注及取消
     * @param worker worker
     * @param req    req
     * @return String
     * @author leixb
     * @since 2023-12-09 16:45
     */
    @Override
    public String doAction(Worker worker, AppMultiModulesDto req) {
        // 内容校验
        checkContentManage(req);
        // worker = getCurrWorker();
        switch (req.getType()) {
            case 1:
                return doCollect(worker, req);
            case 2:
                return doLike(worker, req);
            case 3:
                return doFollow(worker, req);
            default:
                throw new BusinessException("类型错误！");
        }
    }
    
    /**
     * 查询热搜榜/热度榜
     * @param worker worker
     * @return AppMultiModulesHotResVo
     * @author leixb
     * @since 2023-12-09 17:24
     */
    @Override
    public AppMultiModulesHotResVo findHot(Worker worker) {
        // worker = getCurrWorker();
        List<AppMultiModulesHotVo> resultList= appMultiModulesDao.findHot();
        AppMultiModulesHotResVo resVo = new AppMultiModulesHotResVo();
        if (CollectionUtils.isEmpty(resultList)) {
            resVo.setHotSearchList(new ArrayList<>());
            resVo.setHotList(new ArrayList<>());
            return resVo;
        }
        // HotSearch=1 热搜榜 HotList=1 热度榜
        List<YouthContextVo> hotSearchList = resultList.stream()
            .filter(e -> 1 == e.getHotSearch())
            .sorted(Comparator.comparing(AppMultiModulesHotVo::getSearchOrdinalValue))
            .collect(Collectors.toList());
        List<YouthContextVo> hotList = resultList.stream()
            .filter(e -> 1 == e.getHotList())
            .sorted(Comparator.comparing(AppMultiModulesHotVo::getListOrdinalValue))
            .collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(hotSearchList)) {
            appYouthHotService.handleParams(hotSearchList, worker == null ? null : worker.getLoginId());
        }
        if (!CollectionUtils.isEmpty(hotList)) {
            appYouthHotService.handleParams(hotList, worker == null ? null : worker.getLoginId());
        }
        resVo.setHotSearchList(hotSearchList);
        resVo.setHotList(hotList);
        return resVo;
    }
    
    /**
     * 新增浏览记录
     * @param worker    worker
     * @param contentId contentId
     * @author leixb
     * @since 2023-12-12 15:52
     */
    @Override
    public void createView(Worker worker, Long contentId) {
        // worker = getCurrWorker();
        if (null == worker) {
            throw new BusinessException("未获取到用户信息！");
        }
        if (null == contentId) {
            throw new BusinessException("内容ID不能为空！");
        }
        List<AppUserView> appUserViewList = appUserViewRepo.findByContentIdAndUserId(contentId, (long) worker.getLoginId());
        AppUserView appUserView;
        if (null == appUserViewList || appUserViewList.size() == 0) {
            appUserView = new AppUserView();
            appUserView.setContentId(contentId);
            appUserView.setUserId((long) worker.getLoginId());
            appUserView.setCreateTime(new Date());
        } else {
            appUserView = appUserViewList.get(0);
            // 0-有效，1-无效
            if (0 == appUserView.getDelFlag()) {
                appUserView.setUpdateTime(new Date());
            } else {
                appUserView.setDelFlag(0);
                appUserView.setUpdateTime(new Date());
            }
        }
        appUserViewRepo.save(appUserView);
    }
    
    /**
     * 浏览记录分页查询
     * @author leixb
     * @since 2023-12-13 16:32
     * @param worker worker
     * @param req req
     * @return PageRsp<AppUserViewVo>
     */
    @Override
    public PageRsp<AppUserViewVo> pageView(Worker worker, PageReq<Void> req) {
        // worker = getCurrWorker();
        // List<AppUserViewVo> result = pageResult.getResult();
        // List<List<AppUserViewVo>> resVo = new ArrayList<>();
        // // 从result中筛选出今天，昨天，一周内的浏览记录
        // List<AppUserViewVo> todayList = result.stream().filter(e -> 1 == e.getViewType()).collect(Collectors.toList());
        // List<AppUserViewVo> yesterdayList = result.stream().filter(e -> 2 == e.getViewType()).collect(Collectors.toList());
        // List<AppUserViewVo> weekList = result.stream().filter(e -> 3 == e.getViewType()).collect(Collectors.toList());
        PageRsp<AppUserViewVo> pageResult = appMultiModulesDao.pageView(req, (long) worker.getLoginId());
        List<AppUserViewVo> result = pageResult.getResult();
        if (CollectionUtils.isEmpty(result)) {
            return pageResult;
        }
        handleResult(result);
        return pageResult;
    }
    
    /**
     * 处理结果
     * @author leixb
     * @since 2023-12-16 15:34
     * @param result result
     */
    private void handleResult(List<AppUserViewVo> result) {
        for (AppUserViewVo vo : result) {
            String picKey = vo.getPicKey();
            if (!StringUtils.hasText(picKey)) {
                continue;
            }
            String fileId = vo.getFileId();
            PicInfo picInfo;
            // url处理 查询封面图（取第一个）
            String[] split = picKey.split(",");
            if(split.length == 0){
                continue;
            }
            if(StringUtils.hasText(fileId)){
                picInfo = new PicInfo();
                String picUrl;
                if(picKey.contains("http")){
                    picUrl = picKey;
                }else {
                    picUrl = urlPrefix+picKey;
                    picInfo = appYouthHotDao.findPicInfoByFileKey(picKey);
                }
                if(Objects.isNull(picInfo)){
                    picInfo = new PicInfo();
                    picInfo.setWidth(WATER_FALL_DEFAULT_WIDTH);
                    picInfo.setHeight(WATER_FALL_DEFAULT_HEIGHT);
                }
                picInfo.setPicUrl(picUrl);
            } else {
                picInfo = appYouthHotDao.findPicInfoByFileKey(split[0]);
                if(Objects.nonNull(picInfo)){
                    picInfo.setPicUrl(urlPrefix+picInfo.getPicUrl());
                }else {
                    picInfo = new PicInfo();
                    picInfo.setPicUrl(urlPrefix+picInfo.getPicUrl());
                    picInfo.setWidth(WATER_FALL_DEFAULT_WIDTH);
                    picInfo.setHeight(WATER_FALL_DEFAULT_HEIGHT);
                }
            }
            vo.setUrls(picInfo);
        }
    }
    
    /**
     * 一键清空浏览记录列表
     * @param worker worker
     * @return String
     * @author leixb
     * @since 2023-12-13 17:36
     */
    @Override
    public String doClearView(Worker worker) {
        if (null == worker) {
            throw new BusinessException("未获取到用户信息！");
        }
        Integer count = appUserViewRepo.updateByUserId((long) worker.getLoginId());
        if (count > 0) {
            return "清空成功！";
        } else {
            throw new BusinessException("清空失败！");
        }
    }
    
    /**
     * 查询浏览记录数/收藏数
     * @param worker worker
     * @param type   type
     * @return String
     * @author leixb
     * @since 2023-12-13 19:44
     */
    @Override
    public Integer findNum(Worker worker, Integer type) {
        if (null == worker) {
            throw new BusinessException("未获取到用户信息！");
        }
        Long loginId = (long) worker.getLoginId();
        if (1 == type) {
            return appUserViewRepo.countByUserId(loginId);
        } else if (2 == type) {
            return appUserCollectRepo.countByUserId(loginId);
        } else {
            throw new BusinessException("类型错误！");
        }
    }
    
    /**
     * 收藏列表分页查询
     * @param worker worker
     * @param req    req
     * @return PageRsp<AppUserViewVo>
     */
    @Override
    public PageRsp<AppUserViewVo> pageCollect(Worker worker, PageReq<Void> req) {
        // worker = getCurrWorker();
        PageRsp<AppUserViewVo> pageResult = appMultiModulesDao.pageCollect(req, (long) worker.getLoginId());
        List<AppUserViewVo> result = pageResult.getResult();
        if (CollectionUtils.isEmpty(result)) {
            return pageResult;
        }
        handleResult(result);
        return pageResult;
    }
    
    /**
     * 取消收藏
     *
     * @param worker worker
     * @param ids    ids
     * @return String
     */
    @Override
    public String doCancelCollect(Worker worker, List<String> ids) {
        // worker = getCurrWorker();
        if (null == worker) {
            throw new BusinessException("未获取到用户信息！");
        }
        if (null == ids || ids.size() == 0) {
            throw new BusinessException("请选择要取消收藏的内容！");
        }
        // 将ids转Long类型集合
        List<Long> idList = ids.stream().map(Long::valueOf).collect(Collectors.toList());
        Integer num = appUserCollectRepo.updateDelFlagByUserIdAndContentIds((long) worker.getLoginId(), idList);
        if (null != num && num > 0) {
            appContentManageRepo.updateCollectCountByIds(-1, idList);
            return "取消收藏成功！";
        } else {
           throw new BusinessException("取消收藏失败！");
        }
    }

    /**
     * app-应用-热搜
     * @param worker
     * @return
     */
    @Override
    public List<String> findAppHot(Worker worker) {
        return appYouthHotDao.queryApplyHotSearch(AppSearchTypeEnum.SYSTEM.getCode());
    }

    /**
     * 收藏/取消收藏
     * @author leixb
     * @since 2023-12-08 11:10
     * @param worker worker
     * @param req req
     * @return String
     */
    private String doCollect(Worker worker, AppMultiModulesDto req) {
        // worker = getCurrWorker();
        Long publicId = Long.valueOf(req.getPublicId());
        Long loginId = (long) worker.getLoginId();
        int collectCount;
        String msg;
        List<AppUserCollect> appUserCollectList = appUserCollectRepo.findByContentIdAndUserId(publicId, loginId);
        AppUserCollect appUserCollect;
        if (req.getIncrease()) {
            if (null != appUserCollectList && appUserCollectList.size() > 0) {
                appUserCollect = appUserCollectList.get(0);
                if (0 == appUserCollect.getDelFlag()) {
                    throw new BusinessException("内容已收藏！");
                }
                appUserCollect.setDelFlag(0);
                appUserCollect.setUpdateTime(new Date());
            } else {
                appUserCollect = new AppUserCollect();
                appUserCollect.setContentId(publicId);
                appUserCollect.setUserId(loginId);
                appUserCollect.setCreateTime(new Date());
            }
            collectCount = 1;
            msg = "收藏成功！";
        } else {
            if (null != appUserCollectList && appUserCollectList.size() > 0) {
                appUserCollect = appUserCollectList.get(0);
                if (1 == appUserCollect.getDelFlag()) {
                    throw new BusinessException("内容已取消收藏！");
                }
                appUserCollect.setDelFlag(1);
                appUserCollect.setUpdateTime(new Date());
            } else {
                throw new BusinessException("收藏出错！");
            }
            collectCount = -1;
            msg = "取消收藏成功！";
        }
        appUserCollectRepo.save(appUserCollect);
        appContentManageRepo.updateCollectCountById(collectCount, publicId);
        return msg;
    }
    
    /**
     * 点赞/取消点赞
     * @param worker worker
     * @param req    req
     * @author leixb
     * @since 2023-12-08 11:10
     */
    private String doLike(Worker worker, AppMultiModulesDto req) {
        Long publicId = Long.valueOf(req.getPublicId());
        Long loginId = (long) worker.getLoginId();
        int likeCount;
        String msg;
        List<AppUserLike> appUserLikeList = appUserLikeRepo.findByContentIdAndUserId(publicId, loginId);
        AppUserLike appUserLike;
        if (req.getIncrease()) {
            if (null != appUserLikeList && appUserLikeList.size() > 0) {
                appUserLike = appUserLikeList.get(0);
                if (0 == appUserLike.getDelFlag()) {
                    throw new BusinessException("内容已点赞！");
                }
                appUserLike.setDelFlag(0);
                appUserLike.setUpdateTime(new Date());
            } else {
                appUserLike = new AppUserLike();
                appUserLike.setContentId(publicId);
                appUserLike.setUserId(loginId);
                appUserLike.setCreateTime(new Date());
            }
            likeCount = 1;
            msg = "点赞成功！";
        } else {
            if (null != appUserLikeList && appUserLikeList.size() > 0) {
                appUserLike = appUserLikeList.get(0);
                if (1 == appUserLike.getDelFlag()) {
                    throw new BusinessException("内容已取消点赞！");
                }
                appUserLike.setDelFlag(1);
                appUserLike.setUpdateTime(new Date());
            } else {
                throw new BusinessException("点赞出错！");
            }
            likeCount = -1;
            msg = "取消点赞成功！";
        }
        appUserLikeRepo.save(appUserLike);
        appContentManageRepo.updateLikeCountById(likeCount, publicId);
        return msg;
    }
    
    /**
     * 关注/取消关注
     * @param worker worker
     * @param req    req
     * @return String
     * @author leixb
     * @since 2023-12-08 11:10
     */
    private String doFollow(Worker worker, AppMultiModulesDto req) {
        Long publicId = Long.valueOf(req.getPublicId());
        Long loginId = (long) worker.getLoginId();
        String msg;
        List<AppUserFollow> appUserFollowList = appUserFollowRepo.findByAccountIdAndUserId(publicId, loginId);
        AppUserFollow appUserFollow;
        if (req.getIncrease()) {
            if (null != appUserFollowList && appUserFollowList.size() > 0) {
                appUserFollow = appUserFollowList.get(0);
                if (0 == appUserFollow.getDelFlag()) {
                    throw new BusinessException("账号已关注！");
                }
                appUserFollow.setDelFlag(0);
                appUserFollow.setUpdateTime(new Date());
            } else {
                appUserFollow = new AppUserFollow();
                appUserFollow.setAccountId(publicId);
                appUserFollow.setUserId(loginId);
                appUserFollow.setCreateTime(new Date());
            }
            msg = "关注成功！";
        } else {
            if (null != appUserFollowList && appUserFollowList.size() > 0) {
                appUserFollow = appUserFollowList.get(0);
                if (1 == appUserFollow.getDelFlag()) {
                    throw new BusinessException("账号已取消关注！");
                }
                appUserFollow.setDelFlag(1);
                appUserFollow.setUpdateTime(new Date());
            } else {
                throw new BusinessException("关注出错！");
            }
            msg = "取消关注成功！";
        }
        appUserFollowRepo.save(appUserFollow);
        iAppMediumService.attentionMedium(new AttentionMediumDto(publicId, req.getIncrease()));
        return msg;
    }
    
    /**
     * 内容校验
     * @author leixb
     * @since 2023-12-09 15:19
     * @param req req
     */
    private void checkContentManage(AppMultiModulesDto req) {
        // 1-收藏 2-点赞 3-关注
        if (3 != req.getType()) {
            Long publicId = Long.valueOf(req.getPublicId());
            AppContentManage appContentManage = appContentManageRepo.findById(publicId).orElse(null);
            if (null == appContentManage) {
                throw new BusinessException("该内容不存在，请刷新页面！");
            }
            if ( -1L == appContentManage.getContentManageId()) {
                throw new BusinessException("该内容已删除，请刷新页面！");
            }
        }
    }
}
