package com.crunii.micro.service.label.jdbc;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.label.constants.LabelConstants;
import com.crunii.micro.service.label.dto.category.dto.MoveLabelDto;
import com.crunii.micro.service.label.dto.category.dto.SearchAsynchronousTreeDto;
import com.crunii.micro.service.label.dto.category.vo.CategorySearchVo;
import com.crunii.micro.service.label.dto.category.vo.SearchAsynchronousTreeVo;
import com.crunii.micro.service.label.dto.category.vo.SearchSynchronousTreeVo;
import com.crunii.micro.service.label.dto.category.vo.TotalVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-05-30
 */
@Repository
public class CategoryManagementDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_SEARCH_SQL = "select category_id,category_name, level, category_desc, category_parent_id, " +
            " leaf,service_type from tsm_category_management where state = ? and category_parent_id = ? ";

    public static final String PAGE_SEARCH_COUNT_SQL = "select count(*) from tsm_category_management where state = ?" +
            " and category_parent_id = ? ";

    public static final String SEARCH_ASYNCHRONOUS_TREE = "select a.category_id,a.category_name, a.level, a.category_desc, " +
            "a.category_parent_id, a.leaf, b.category_name as category_parent_name,a.service_type from (select category_id,category_name," +
            "level, category_desc, category_parent_id, leaf, service_type from tsm_category_management where state = ? " +
            "and category_parent_id = ?";

    public static final String SEARCH_SYNCHRONOUS_TREE = "select category_id,category_name, level, category_desc, category_parent_id, " +
            " leaf from tsm_category_management where state = ? and category_parent_id = ? ";

    public PageRsp<CategorySearchVo> pageSearch(PageReq<Object> req) {
        StringBuilder sb = new StringBuilder(PAGE_SEARCH_SQL);
        List<Object> param = new ArrayList<>();
        param.add(CommonConstants.STATE_VALID);
        param.add(Long.valueOf(LabelConstants.ROOT_PARENT_ID));
        int total = jt.queryForInt(PAGE_SEARCH_COUNT_SQL, param.toArray());
        List<CategorySearchVo> categorySearchVoList;
        String dbType = jt.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sb, req);
            categorySearchVoList = jt.queryForBeans(CategorySearchVo.class, sb.toString(), param.toArray());
        } else {
            categorySearchVoList = new ArrayList<>();
        }
        PageRsp<CategorySearchVo> pageRsp = new PageRsp<>();
        pageRsp.setPageNo(req.getPageNo());
        pageRsp.setPageSize(req.getPageSize());
        pageRsp.setResult(categorySearchVoList);
        pageRsp.setTotalNum(total);
        // 为手机计算是否由下一页
        pageRsp.next();
        return pageRsp;
    }

    public List<SearchAsynchronousTreeVo> searchAsynchronousTree(SearchAsynchronousTreeDto dto) {
        StringBuilder sql = new StringBuilder(SEARCH_ASYNCHRONOUS_TREE);
        List<Object> param = new ArrayList<>();
        param.add(CommonConstants.STATE_VALID);
        param.add(dto.getCategoryId());
        if (StringUtils.hasText(dto.getServiceType())) {
            sql.append(" and service_type = ? ");
            param.add(dto.getServiceType());
        }
        sql.append(") a LEFT JOIN tsm_category_management b ON a.category_parent_id = b.category_id");
        return jt.queryForBeans(SearchAsynchronousTreeVo.class, sql.toString(), param.toArray());
    }

    public List<SearchSynchronousTreeVo> searchSynchronousTree(String categoryParentId) {
        // 先查询根结点(1级分类)的数据列表
        List<SearchSynchronousTreeVo> searchList = jt.queryForBeans(SearchSynchronousTreeVo.class, SEARCH_SYNCHRONOUS_TREE,
                CommonConstants.STATE_VALID, categoryParentId);
        // 递归选出所有子级类别
        for (SearchSynchronousTreeVo vo : searchList) {
            vo.setChildren(searchSynchronousTree(vo.getCategoryId()));
        }
        return searchList;
    }

    public void deleteCategory(String categoryParentId, String loginId) {
        String sql = "update tsm_category_management set state = ?, last_worker = ?, update_time = ? where category_id = ? ";
        jt.update(sql, CommonConstants.STATE_INVALID, loginId, new Date(), Long.valueOf(categoryParentId));
    }

    public void moveLabel(MoveLabelDto dto, String loginId) {
        String sql = "update tsm_category_management set category_parent_id = ?, last_worker = ?, update_time = ? where category_id = ?";
        jt.update(sql, dto.getTargetCategoryId(), loginId, new Date(), dto.getCategoryId());
    }

    public void updateLeaf(Integer leaf, Long categoryId) {
        String sql = "update tsm_category_management set leaf = ? where category_id = ?";
        jt.update(sql, leaf, categoryId);
    }

    public TotalVo selectCountByCategoryParentId(Long categoryParentId) {
        String sql = "select count(1) as total from tsm_category_management where category_parent_id = ? and state = ? ";
        return jt.queryForBean(TotalVo.class, sql, categoryParentId, CommonConstants.STATE_VALID);
    }

    public TotalVo checkRepeatCategoryName(String categoryName) {
        String sql = "select count(1) as total from tsm_category_management where category_name = ? and state = ? ";
        return jt.queryForBean(TotalVo.class, sql, categoryName, CommonConstants.STATE_VALID);
    }

    public void deleteStateZero() {
        String sql = "delete from tsm_category_management where state = ?";
        jt.update(sql, CommonConstants.STATE_INVALID);
    }
}
