package com.crunii.micro.service.label.service.impl;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmCategoryManagement;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.domain.repo.main.TsmCategoryManagementRepo;
import com.crunii.micro.domain.repo.main.TsmPropertyValueRepo;
import com.crunii.micro.service.label.constants.LabelConstants;
import com.crunii.micro.service.label.dto.category.dto.*;
import com.crunii.micro.service.label.dto.category.vo.CategorySearchVo;
import com.crunii.micro.service.label.dto.category.vo.SearchAsynchronousTreeVo;
import com.crunii.micro.service.label.dto.category.vo.SearchSynchronousTreeVo;
import com.crunii.micro.service.label.dto.category.vo.TotalVo;
import com.crunii.micro.service.label.jdbc.CategoryManagementDao;
import com.crunii.micro.service.label.jdbc.LabelManagementDao;
import com.crunii.micro.service.label.service.AiconsumeCacheService;
import com.crunii.micro.service.label.service.CategoryManagementService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-05-30
 */
@Service
public class CategoryManagementServiceImpl implements CategoryManagementService {

    @Resource
    private CategoryManagementDao categoryManagementDao;

    @Resource
    private LabelManagementDao labelManagementDao;

    @Resource
    private TsmCategoryManagementRepo tsmCategoryManagementRepo;

    @Resource
    private TsmPropertyValueRepo tsmPropertyValueRepo;

    @Resource
    private AiconsumeCacheService aiconsumeCacheService;

    @Override
    @InjectProperty
    public Result<PageRsp<CategorySearchVo>> pageSearch(PageReq<Object> req) {
        return Result.ofSuccess(categoryManagementDao.pageSearch(req));
    }

    @Override
    public Result<Object> addCategory(AddCategoryDto dto, Worker worker) {
        validatedCategoryNameAndDescribeLength(dto.getCategoryName(), dto.getCategoryDesc());
        return commonAddCategory(dto.getCategoryName(), dto.getServiceType(), dto.getCategoryDesc(), dto.getCategoryParentId(),
                dto.getLevel(), worker.getLoginId());
    }

    @Override
    public Result<Object> addBelow(AddBelowDto dto, Worker worker) {
        return commonAddCategory(dto.getCategoryName(), dto.getServiceType(), dto.getCategoryDesc(), dto.getCategoryParentId(),
                dto.getLevel(), worker.getLoginId());
    }

    @Override
    @InjectProperty
    public Result<List<SearchAsynchronousTreeVo>> searchAsynchronousTree(SearchAsynchronousTreeDto dto) {
        List<SearchAsynchronousTreeVo> asynchronousTreeVos = categoryManagementDao.searchAsynchronousTree(dto);
        for (SearchAsynchronousTreeVo vo : asynchronousTreeVos) {
            TsmPropertyValue value = tsmPropertyValueRepo.findTsmPropertyValueByPropCode(LabelConstants.CATEGORY_BOTTOM_LEVEL);
            // 若级别为静态值设置的最底层级别,目前为3级,那么把叶子结点设置为true.
            if (vo.getLevel().equals(Integer.valueOf(value.getValueName()))) {
                vo.setLeaf(true);
            }
        }
        return Result.ofSuccess(asynchronousTreeVos);
    }

    @Override
    public Result<List<SearchSynchronousTreeVo>> searchSynchronousTree() {
        // 通过redis缓存查询类别同步树
        List<SearchSynchronousTreeVo> redisTreeList = aiconsumeCacheService.
                getSynchronousTree(LabelConstants.REDIS_SYNCHRONOUS_TREE);
        if (Objects.nonNull(redisTreeList)) {
            return Result.ofSuccess(redisTreeList);
        }
        // 执行到这一步,说明缓存里面没有数据或者已过期,因此这里查询数据库后,再添加进缓存里面
        // dao层使用了递归来处理,这里直接传值根结点的父级id
        aiconsumeCacheService.putSynchronousTree(LabelConstants.REDIS_SYNCHRONOUS_TREE,
                categoryManagementDao.searchSynchronousTree(LabelConstants.ROOT_PARENT_ID));
        return Result.ofSuccess(categoryManagementDao.searchSynchronousTree(LabelConstants.ROOT_PARENT_ID));
    }

    @Override
    public Result<Object> moveLabel(MoveLabelDto dto, Worker worker) {
        // 先查询该原本类别的信息,主要查询父级id
        TsmCategoryManagement originCategory = tsmCategoryManagementRepo.getOne(Long.valueOf(dto.getCategoryId()));
        // 查询转移目标分类信息
        TsmCategoryManagement targetCategory = tsmCategoryManagementRepo.getOne(Long.valueOf(dto.getTargetCategoryId()));
        // 原分类被转移后仍然要为同级,例如要转移的类别是2级类别,被转移后仍然要为2级类别,那么目前类别必须为1级类别,因为仅支持同级转移.
        // 校验,将原分类级别减去1级与目标分类进行对比匹配,如不符合,需要返回页面
        if ((originCategory.getLevel() - 1) != (targetCategory.getLevel())) {
            return Result.ofFail("仅支持类别同级转移,请重新选择目标分类");
        }

        categoryManagementDao.moveLabel(dto, String.valueOf(worker.getLoginId()));
        // 这里通过类别原来的父级id的数量,查看上级分类是否还有其他下级分类
        TotalVo total = categoryManagementDao.selectCountByCategoryParentId(originCategory.getCategoryParentId());
        if (total.getTotal() == 0) {
            // 列表为0,说明上级类别没有其他下级分类,因此可以执行下列方法
            // 类别转移到新的分类后,原类别就暂时没有了下级类别,因此叶子结点变更为1是
            categoryManagementDao.updateLeaf(LabelConstants.LEAF, originCategory.getCategoryParentId());
        }
        // 因为类别到新的分类了,因此该类别新的上级分类不为叶子结点了,因此需要将叶子结点变更为0否
        categoryManagementDao.updateLeaf(LabelConstants.NOT_LEAF, Long.valueOf(dto.getTargetCategoryId()));
        // 每次更新修改都需要清空查询同步树的缓存
        aiconsumeCacheService.removeSynchronousTree(LabelConstants.REDIS_SYNCHRONOUS_TREE);
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<Object> updateCategory(UpdateCategoryDto dto, Worker worker) {
        validatedCategoryNameAndDescribeLength(dto.getCategoryName(), dto.getCategoryDesc());
        TsmCategoryManagement tsmCategoryManagement = tsmCategoryManagementRepo.getOne(Long.valueOf(dto.getCategoryId()));
        if (StringUtils.isNotBlank(dto.getCategoryName())) {
            //修改功能,如果数据库里该类别的名称与传入的名称相同,说明此时用户并没有对已有类别名称进行修改,那么可以不用做后续校验
            if (!tsmCategoryManagement.getCategoryName().equals(dto.getCategoryName())) {
                // 执行这一步,说明用户已修改了类别名称,需要做校验
                // 校验:类别的名称，不可重复
                if (checkCategoryName(dto.getCategoryName())) {
                    // 如果大于0,说明数据里面已有该标签名称,那么直接返回信息提示页面
                    return Result.ofFail("存在重复的类别名称,请重新填写");
                }
            }
            tsmCategoryManagement.setCategoryName(dto.getCategoryName());
        }
        if (StringUtils.isNotBlank(dto.getCategoryDesc())) {
            tsmCategoryManagement.setCategoryDesc(dto.getCategoryDesc());
        }
        if (StringUtils.isNotBlank(dto.getCategoryParentId())) {
            // 这里通过类别原来的父级id的数量,查看上级分类是否还有其他下级分类
            TotalVo total = categoryManagementDao.selectCountByCategoryParentId(tsmCategoryManagement.getCategoryParentId());
            if (total.getTotal() == 0) {
                // 这里如果更改了上级分类,那么原来的上级分类变为叶子结点,叶子结点leaf字段变更为1是
                categoryManagementDao.updateLeaf(LabelConstants.LEAF, tsmCategoryManagement.getCategoryParentId());
            }
            tsmCategoryManagement.setCategoryParentId(Long.valueOf(dto.getCategoryParentId()));
            // 因为类别到新的分类了,因此该类别新的上级分类不为叶子结点了,因此需要将叶子结点变更为0否
            categoryManagementDao.updateLeaf(LabelConstants.NOT_LEAF, Long.valueOf(dto.getCategoryParentId()));
        }
        if (Objects.nonNull(dto.getLevel())) {
            tsmCategoryManagement.setLevel(dto.getLevel());
        }
        if (StringUtils.isNotBlank(dto.getServiceType())) {
            tsmCategoryManagement.setServiceType(dto.getServiceType());
        }
        tsmCategoryManagement.setLastWorker(String.valueOf(worker.getLoginId()));
        tsmCategoryManagement.setUpdateTime(new Date());
        tsmCategoryManagementRepo.saveAndFlush(tsmCategoryManagement);
        // 每次更新修改都需要清空查询同步树的缓存
        aiconsumeCacheService.removeSynchronousTree(LabelConstants.REDIS_SYNCHRONOUS_TREE);
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<Boolean> deleteConfirm(String categoryId) {
        // 删除确认需要作两次判断,1.查看要删除的类别是否有子类别,2.查看要删除的类别是否绑定标签
        // 只有要删除类别没有子类别和未绑定标签才能删除,因此返回true提示页面表示可删除
        // 该查询是为了查看传过来的类别是否有子类别,若有子类别需要作提示回去.
        TotalVo categoryTotal = categoryManagementDao.selectCountByCategoryParentId(Long.valueOf(categoryId));
        if (categoryTotal.getTotal() > 0) {
            // 有子类别,返回false,页面会先弹窗提示,表示不能删除
            return Result.ofSuccess(false);
        }
        // 通过类别id查询标签里面的数量
        TotalVo labelTotal = labelManagementDao.selectCountLabelByCategoryId(Long.valueOf(categoryId));
        if (labelTotal.getTotal() > 0) {
            // 关联标签, 返回false, 页面会弹窗提示,表示不能删除
            return Result.ofSuccess(false);
        }
        // 以上判断若都不符合,表示该类别可删除,返回true
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> deleteCategory(String categoryId, Worker worker) {
        // 删除类别定为将类别状态设置为0
        categoryManagementDao.deleteCategory(categoryId, String.valueOf(worker.getLoginId()));
        TsmCategoryManagement tsmCategoryManagement = tsmCategoryManagementRepo.getOne(Long.valueOf(categoryId));
        // 这里通过类别原来的父级id的数量,查看上级分类是否还有其他下级分类
        TotalVo total = categoryManagementDao.selectCountByCategoryParentId(tsmCategoryManagement.getCategoryParentId());
        if (total.getTotal() == 0) {
            // 说明删除类别后,此时的上级类别没有其他下级类别.因此上级类别暂定为叶子结点,leaf要改为1是
            categoryManagementDao.updateLeaf(LabelConstants.LEAF, tsmCategoryManagement.getCategoryParentId());
        }
        // 每次更新修改都需要清空查询同步树的缓存
        aiconsumeCacheService.removeSynchronousTree(LabelConstants.REDIS_SYNCHRONOUS_TREE);
        return Result.ofSuccess("操作成功!");
    }

    public void saveCategoryManagement(String categoryName, String categoryDesc, String categoryParentId, Integer level,
                                       String loginId, String serviceType) {
        TsmCategoryManagement tsmCategoryManagement = new TsmCategoryManagement();
        tsmCategoryManagement.setCategoryName(categoryName);
        tsmCategoryManagement.setCategoryDesc(categoryDesc);
        tsmCategoryManagement.setCategoryParentId(Long.valueOf(categoryParentId));
        tsmCategoryManagement.setLevel(level);
        tsmCategoryManagement.setCreateTime(new Date());
        tsmCategoryManagement.setUpdateTime(new Date());
        tsmCategoryManagement.setState(CommonConstants.STATE_VALID);
        // 这里新增数据后,会是叶子结点, 因此赋值为1
        tsmCategoryManagement.setLeaf(LabelConstants.LEAF);
        tsmCategoryManagement.setLastWorker(loginId);
        tsmCategoryManagement.setServiceType(serviceType);
        tsmCategoryManagementRepo.saveAndFlush(tsmCategoryManagement);
        // 这里判断:如果父级ID为0,说明目前新增分类是1级类别.如果不为0,说明新增分类可能是2级或者3级,需要对上级分类的leaf字段做出状态更改
        if (!LabelConstants.ZERO_STRING.equals(categoryParentId)) {
            categoryManagementDao.updateLeaf(LabelConstants.NOT_LEAF, Long.valueOf(categoryParentId));
        }
    }

    public boolean checkCategoryName(String categoryName) {
        TotalVo total = categoryManagementDao.checkRepeatCategoryName(categoryName);
        // 大于0返回true,否则返回false
        return total.getTotal() > 0;
    }

    private void validatedCategoryNameAndDescribeLength(String categoryName, String categoryDesc) {
        int length = StrUtil.length(categoryName);
        if (length > LabelConstants.CATEGORY_MAX_LENGTH) {
            throw new BusinessException("分类名称不能超过" + LabelConstants.CATEGORY_MAX_LENGTH + "字");
        }

        int labelDescLength = StrUtil.length(categoryDesc);
        if (labelDescLength > LabelConstants.CATEGORY_DESCRIBE_MAX_LENGTH) {
            throw new BusinessException("分类描述不能超过" + LabelConstants.CATEGORY_DESCRIBE_MAX_LENGTH + "字");
        }
    }

    public Result<Object> commonAddCategory(String categoryName, String serviceType, String categoryDesc, String categoryParentId,
                                            Integer level, int loginId) {
        // 校验:类别的名称，不可重复
        if (checkCategoryName(categoryName)) {
            // 如果大于0,说明数据里面已有该标签名称,那么直接返回信息提示页面
            return Result.ofFail("存在重复的类别名称,请重新填写");
        }
        saveCategoryManagement(categoryName, categoryDesc, categoryParentId, level, String.valueOf(loginId), serviceType);

        // 每次更新修改都需要清空查询同步树的缓存
        aiconsumeCacheService.removeSynchronousTree(LabelConstants.REDIS_SYNCHRONOUS_TREE);
        return Result.ofSuccess("操作成功!");
    }
}
