package com.crunii.micro.service.label.service.impl;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmCategoryManagement;
import com.crunii.micro.domain.entity.main.TsmLabelManagement;
import com.crunii.micro.domain.repo.main.TsmLabelManagementRepo;
import com.crunii.micro.feign.client.ElsticService;
import com.crunii.micro.feign.client.FileManagerService;
import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.label.constants.LabelConstants;
import com.crunii.micro.service.label.dto.category.vo.TotalVo;
import com.crunii.micro.service.label.dto.label.dto.*;
import com.crunii.micro.service.label.dto.label.vo.LabelPageSearchVo;
import com.crunii.micro.service.label.dto.label.vo.LabelRecentlyCountVo;
import com.crunii.micro.service.label.jdbc.LabelManagementDao;
import com.crunii.micro.service.label.service.AiconsumeCacheService;
import com.crunii.micro.service.label.service.LabelManagementService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;

/**
 * @author baoxy
 * @date 2023-05-30
 */
@Service
public class LabelManagementServiceImpl implements LabelManagementService {

    @Resource
    private LabelManagementDao labelManagementDao;

    @Resource
    private TsmLabelManagementRepo tsmLabelManagementRepo;

    @Resource
    private ElsticService elsticService;

    @Resource
    private AiconsumeCacheService aiconsumeCacheService;

    @Resource
    private FileManagerService fileManagerService;

    @Override
    @InjectProperty
    public Result<PageRsp<LabelPageSearchVo>> pageSearch(PageReq<LabelPageSearchDto> req) {
        PageRsp<LabelPageSearchVo> pageRsp = labelManagementDao.pageSearch(req);
        List<LabelPageSearchVo> labelPageSearchVoList = pageRsp.getResult();
        // 需要赋值每个标签类别的名称:以“-“连接各层类别展示  例如L1-L2-L3 L1表示1级分类,L2表示2级分类,L3表示3级分类
        // 2023-06-06改动:与前段沟通后,类别名称数据类型改为list列表,顺序以L1,L2,L3的顺序返回
        for (LabelPageSearchVo vo : labelPageSearchVoList) {
            List<String> categoryName = linkCategoryName(vo.getCategoryId());
            vo.setCategoryName(categoryName);
        }
        pageRsp.setResult(labelPageSearchVoList);
        return Result.ofSuccess(pageRsp);
    }


    @Override
    public Result<Object> addLabel(AddLabelDto dto, Worker worker) {
        validatedLabelNameAndDescribeLength(dto.getLabelName(), dto.getLabelDesc());
        // 校验:标签的名称，不可重复
        if (checkLabelName(dto.getLabelName())) {
            return Result.ofFail("存在重复的标签名称,请重新填写");
        }
        TsmLabelManagement tsmLabelManagement = new TsmLabelManagement();
        tsmLabelManagement.setCategoryId(Long.valueOf(dto.getCategoryId()));
        tsmLabelManagement.setLabelName(dto.getLabelName());
        tsmLabelManagement.setLabelDesc(dto.getLabelDesc());
        tsmLabelManagement.setCreateTime(new Date());
        tsmLabelManagement.setUpdateTime(new Date());
        tsmLabelManagement.setState(CommonConstants.STATE_VALID);
        // 新增的标签引用次数设置为0
        tsmLabelManagement.setUseCount(LabelConstants.ZERO_INTEGER);
        tsmLabelManagement.setLastWorker(String.valueOf(worker.getLoginId()));
        tsmLabelManagement.setServiceType(dto.getServiceType());
        Long labelId = tsmLabelManagementRepo.saveAndFlush(tsmLabelManagement).getLabelId();
        // 将标签数据保存到ES
        LabelDto labelDto = new LabelDto();
        labelDto.setLabelId(labelId.toString());
        labelDto.setLabelName(dto.getLabelName());
        elsticService.saveLabel(labelDto);
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<Object> updateLabel(UpdateLabelDto dto, Worker worker) {
        validatedLabelNameAndDescribeLength(dto.getLabelName(), dto.getLabelDesc());
        TsmLabelManagement tsmLabelManagement = tsmLabelManagementRepo.getOne(Long.valueOf(dto.getLabelId()));
        if (StringUtils.isNotBlank(dto.getLabelName())) {
            //修改功能,如果数据库里该标签的名称与传入的名称相同,说明此时用户并没有对已有标签名称进行修改,那么可以不用做后续校验
            if (!tsmLabelManagement.getLabelName().equals(dto.getLabelName())) {
                // 执行这一步,说明用户已修改了类别名称,需要做校验
                // 校验:标签的名称，不可重复
                if (checkLabelName(dto.getLabelName())) {
                    return Result.ofFail("存在重复的标签名称,请重新填写");
                }
            }
            tsmLabelManagement.setLabelName(dto.getLabelName());
        }
        tsmLabelManagement.setLabelDesc(dto.getLabelDesc());
        if (StringUtils.isNotBlank(dto.getCategoryId())) {
            tsmLabelManagement.setCategoryId(Long.valueOf(dto.getCategoryId()));
        }
        tsmLabelManagement.setServiceType(dto.getServiceType());
        tsmLabelManagement.setUpdateTime(new Date());
        tsmLabelManagement.setLastWorker(String.valueOf(worker.getLoginId()));
        tsmLabelManagementRepo.saveAndFlush(tsmLabelManagement);
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<Object> deleteLabel(DeleteLabelDto dto, Worker worker) {
        labelManagementDao.deleteLabel(dto, String.valueOf(worker.getLoginId()));
        CompletableFuture.runAsync(() -> dto.getLabelIdList().forEach(e -> {
            // 删除ES中的标签数据
            elsticService.delLabel(e);
            // 删除标签绑定的图片
            fileManagerService.delPicByLabelId(Long.valueOf(e));
        }));
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<List<LabelDto>> hotLabel() {
        // 先从热门标签48小时的缓存列表取出数据
        List<LabelDto> fortyEightHotLabelList = aiconsumeCacheService.getFortyEightHotLabel(LabelConstants.REDIS_HOT_LABEL_FORTY_EIGHT);
        if (Objects.nonNull(fortyEightHotLabelList)) {
            // 若列表不为null，说明有数据
            if (fortyEightHotLabelList.size() < LabelConstants.TEN_INT) {
                // 若该列表数据不足10条,需要从热门标签长期的缓存列表里取出数据补齐
                List<LabelDto> longTimeHotLabelList = aiconsumeCacheService.getLongTimeHotLabel(LabelConstants.REDIS_HOT_LABEL_LONG_TIME);
                if (Objects.nonNull(longTimeHotLabelList)) {
                    fortyEightHotLabelList.forEach(e -> longTimeHotLabelList.forEach(f -> {
                        // 取出不同标签id的数据,如果有相同数据不要存入, 并且列表已有10条数据后,直接返回数据即可
                        if (!e.getLabelId().equals(f.getLabelId()) && fortyEightHotLabelList.size() == LabelConstants.TEN_INT) {
                            fortyEightHotLabelList.add(f);
                        }
                    }));
                    return Result.ofSuccess(fortyEightHotLabelList);
                }
                // 若热门标签长期的缓存列表里为null,则直接返回48小时缓存列表
                return Result.ofSuccess(fortyEightHotLabelList);
            }
            // 执行到这一步,说明列表数据已有10条,所以直接返回数据
            return Result.ofSuccess(fortyEightHotLabelList);
        }
        // 执行到这一步,说明48小时缓存列表为null,则使用长期缓存列表
        List<LabelDto> longTimeHotLabelList = aiconsumeCacheService.getLongTimeHotLabel(LabelConstants.REDIS_HOT_LABEL_LONG_TIME);
        if (Objects.nonNull(longTimeHotLabelList)) {
            // 不为null,则直接返回数据,并且更新48小时缓存列表
            aiconsumeCacheService.putFortyEightHotLabel(LabelConstants.REDIS_HOT_LABEL_FORTY_EIGHT, longTimeHotLabelList);
            return Result.ofSuccess(longTimeHotLabelList);
        }
        // 若以上条件都没满足,则直接通过查询数据库来返回
        return Result.ofSuccess(labelManagementDao.hotLabel());
    }

    @Override
    @InjectProperty
    public Result<PageRsp<LabelPageSearchVo>> pageSearchByDrawYouth(PageReq<LabelPageSearchDto> req) {
        PageRsp<LabelPageSearchVo> pageRsp = labelManagementDao.pageSearchByDrawYouth(req);
        List<LabelPageSearchVo> labelPageSearchVoList = pageRsp.getResult();
        // 需要赋值每个标签类别的名称:以“-“连接各层类别展示  例如L1-L2-L3 L1表示1级分类,L2表示2级分类,L3表示3级分类
        // 2023-06-06改动:与前段沟通后,类别名称数据类型改为list列表,顺序以L1,L2,L3的顺序返回
        for (LabelPageSearchVo vo : labelPageSearchVoList) {
            List<String> categoryName = linkCategoryName(vo.getCategoryId());
            vo.setCategoryName(categoryName);
        }
        pageRsp.setResult(labelPageSearchVoList);
        return Result.ofSuccess(pageRsp);
    }

    @Override
    public Result<List<LabelRecentlyCountVo>> labelRecentlyCount(LabelRecentlyCountDto dto) {
        return Result.ofSuccess(labelManagementDao.labelRecentlyCount(dto));
    }

    @Override
    public Result<List<LabelRecentlyCountVo>> labelPublicRecentlyCount(LabelRecentlyCountDto dto) {
        return Result.ofSuccess(labelManagementDao.labelPublicRecentlyCount(dto));
    }

    public List<String> linkCategoryName(String categoryId) {
        // 类别结果列表,以L1,L2,L3的顺序排序
        List<String> result = new ArrayList<>();
        TsmCategoryManagement categoryName = labelManagementDao.linkCategoryName(Long.valueOf(categoryId));
        // 判断当前类别是否为1级类别
        if (LabelConstants.ONE_LEVEL.equals(categoryName.getLevel())) {
            // 1级类别只需要存一个值即可
            result.add(categoryName.getCategoryName());
            return result;
        }
        // 判断当前类别是否为2级类别
        if (LabelConstants.TWO_LEVEL.equals(categoryName.getLevel())) {
            // 若为2级类别, 需要再查询1级类别后存储
            TsmCategoryManagement oneLevel = labelManagementDao.linkCategoryName(categoryName.getCategoryParentId());
            // 类别名称存储,以L1,L2的顺序排序
            result.add(oneLevel.getCategoryName());
            result.add(categoryName.getCategoryName());
            return result;
        }
        // 判断当前类别是否为3级类别
        if (LabelConstants.THREE_LEVEL.equals(categoryName.getLevel())) {
            // 若为3级类别, 需要再查询2级类别，最后查询1级类别,并且存储
            TsmCategoryManagement twoLevel = labelManagementDao.linkCategoryName(categoryName.getCategoryParentId());
            TsmCategoryManagement oneLevel = labelManagementDao.linkCategoryName(twoLevel.getCategoryParentId());
            // 类别名称组装, 以L1,L2,L3的顺序排序
            result.add(oneLevel.getCategoryName());
            result.add(twoLevel.getCategoryName());
            result.add(categoryName.getCategoryName());
            return result;
        }
        return new ArrayList<>();
    }

    public boolean checkLabelName(String labelName) {
        TotalVo total = labelManagementDao.checkRepeatLabelName(labelName);
        // 大于0返回true,否则返回false
        return total.getTotal() > 0;
    }

    private void validatedLabelNameAndDescribeLength(String labelName, String labelDesc) {
        int length = StrUtil.length(labelName);
        if (length > LabelConstants.LABEL_MAX_LENGTH) {
            throw new BusinessException("标签名称不能超过" + LabelConstants.LABEL_MAX_LENGTH + "字");
        }

        int labelDescLength = StrUtil.length(labelDesc);
        if (labelDescLength > LabelConstants.DESCRIBE_MAX_LENGTH) {
            throw new BusinessException("描述不能超过" + LabelConstants.DESCRIBE_MAX_LENGTH + "字");
        }
    }
}
