package com.crunii.micro.service.label.task;

import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.label.constants.LabelConstants;
import com.crunii.micro.service.label.dto.label.vo.TotalUseCountVo;
import com.crunii.micro.service.label.jdbc.CategoryManagementDao;
import com.crunii.micro.service.label.jdbc.LabelManagementDao;
import com.crunii.micro.service.label.service.AiconsumeCacheService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * @author baoxy
 * @date 2023-06-05
 */
@Slf4j
@Component
@EnableScheduling
public class DataUpdateTask {

    @Resource
    private AiconsumeCacheService aiconsumeCacheService;

    @Resource
    private LabelManagementDao labelManagementDao;

    @Resource
    private CategoryManagementDao categoryManagementDao;

    @Scheduled(cron = "0 0 0/2 * * ?")
    public void labelUseCount() {
        // 该任务每两个小时更新一次tsm_label_management的引用次数
        log.info("开始执行更新标签引用次数的任务, 当前时间:" + LocalDateTime.now());
        // 先获取redis所有key
        Set<String> keys = aiconsumeCacheService.labelUseCountKey();
        keys.forEach(e -> {
            // 获取redis该键值(标签id)对应的值(使用次数),并更新进数据库里
            Integer useCount = aiconsumeCacheService.getLabelUseCount(e);
            log.info("标签id:{}, 引用次数:{}", e, useCount);
            labelManagementDao.updateUseCount(e, useCount);
        });
        aiconsumeCacheService.removeLongTimeHotLabel(LabelConstants.REDIS_HOT_LABEL_LONG_TIME);
        // 存入redis的热门标签列表
        List<LabelDto> labelDtoList = labelManagementDao.hotLabel();
        aiconsumeCacheService.putLongTimeHotLabel(LabelConstants.REDIS_HOT_LABEL_LONG_TIME, labelDtoList);
    }

    @Scheduled(cron = "0 0 1 * * ?")
    public void fileLabelUseCount() {
        // 该任务每天凌晨1点执行,主要调用ycq_file_label_relation获取标签的使用次数,目的是为了保证数据准确性
        log.info("开始执行从文件与标签关联表表里更新标签引用次数的任务, 当前时间:" + LocalDateTime.now());
        List<TotalUseCountVo> totalUseCountVoList = labelManagementDao.fileLabelCount();
        log.info("执行任务的文件关联标签的数据:" + totalUseCountVoList);
        // 存入redis的热门标签列表
        List<LabelDto> labelDtoList = new ArrayList<>();
        totalUseCountVoList.forEach(e -> {
            log.info("标签id:{}, 引用次数:{}", e.getLabelId(), e.getTotal());
            aiconsumeCacheService.putLabelUseCount(e.getLabelId(), e.getTotal());
            labelManagementDao.updateUseCount(e.getLabelId(), e.getTotal());
            // 列表有10条数据后就不需要再存入数据
            if (labelDtoList.size() < LabelConstants.TEN_INT) {
                LabelDto labelDto = new LabelDto();
                labelDto.setLabelId(e.getLabelId());
                labelDto.setLabelName(e.getLabelName());
                labelDtoList.add(labelDto);
            }
        });
        aiconsumeCacheService.removeLongTimeHotLabel(LabelConstants.REDIS_HOT_LABEL_LONG_TIME);
        // 存入redis的热门标签列表
        aiconsumeCacheService.putLongTimeHotLabel(LabelConstants.REDIS_HOT_LABEL_LONG_TIME, labelDtoList);
    }

    @Scheduled(cron = "0 0 2 1 1/3 ?")
    public void recycleTask() {
        // 该方法为回收站机制,每隔3个月清理一次数据库状态值state为0的数据
        log.info("开始执行回收站任务,当前时间:" + LocalDateTime.now());
        // 清理类别管理表状态值为0的数据
        categoryManagementDao.deleteStateZero();
        // 清理标签管理状态值为0的数据
        labelManagementDao.deleteStateZero();
    }
}
