package com.crunii.micro.service.league.service.impl;

import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.league.constants.LeagueConstants;
import com.crunii.micro.service.league.dto.*;
import com.crunii.micro.service.league.jdbc.YcqLeagueDao;
import com.crunii.micro.service.league.service.LeagueService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangchen create 2023/5/31 17:31
 */
@Service
@Slf4j
public class LeagueServiceImpl implements LeagueService {

    private final YcqLeagueDao ycqLeagueDao;

    public LeagueServiceImpl(YcqLeagueDao ycqLeagueDao) {
        this.ycqLeagueDao = ycqLeagueDao;
    }

    @Override
    public List<YcqLeagueTreeVo> getOrgChildren(String leagueCode) {
        List<YcqLeagueTreeVo> ycqLeagueVos = ycqLeagueDao.getOrgChildren(leagueCode);
        paddingLeaf(ycqLeagueVos);
        return ycqLeagueVos;
    }

    @Override
    public PageRsp<YcqLeagueVo> getOrgTabulation(PageReq<YcqLeagueDto> req) {
        return ycqLeagueDao.getOrgTabulation(req);
    }

    @Override
    public PageRsp<AgeDataVo> getLmAgeBigData(PageReq<PageLeagueAgeDto> req) {
        throwNotLeagueCodeException(req.getQuery().getLeagueCode());
        PageRsp<AgeDataVo> pageRsp = ycqLeagueDao.getLmAgeBigData(req);
        List<AgeDataVo> ageDataVos = pageRsp.getResult();
        if(CollectionUtils.isEmpty(ageDataVos)) {
            return pageRsp;
        }
        handleAgeData(ageDataVos);
        return pageRsp;
    }

    @Override
    public LeagueEduVo getLmEduBigData(String leagueCode) {
        throwNotLeagueCodeException(leagueCode);
        LeagueEduVo leagueEduVo = new LeagueEduVo();
        List<LeagueEduTabulation> leagueEduTabulations = ycqLeagueDao.getLmEduBigData(leagueCode);
        if(CollectionUtils.isEmpty(leagueEduTabulations)){
            return leagueEduVo;
        }
        DecimalFormat df =new DecimalFormat();
        df.setMaximumFractionDigits(1);
        df.setMinimumFractionDigits(1);
        int sum = leagueEduTabulations.stream().mapToInt(LeagueEduTabulation::getLmCount).sum();
        String lmProportion = "0.0";
        for (LeagueEduTabulation leagueEduTabulation : leagueEduTabulations) {
            if(sum == 0){
                leagueEduTabulation.setLmProportion(lmProportion);
            }else {
                int lmCount = leagueEduTabulation.getLmCount();
                leagueEduTabulation.setLmProportion(df.format(lmCount *100.00 / sum));
            }
        }
        leagueEduVo.setLeagueEduTabulations(leagueEduTabulations);
        leagueEduVo.setLmSumCount(sum);
        return leagueEduVo;
    }

    @Override
    @InjectProperty
    public PageRsp<MemberNationality>  getLmNationalityBigData(PageReq<PageNationalityDto> req) {
        throwNotLeagueCodeException(req.getQuery().getLeagueCode());
        PageRsp<MemberNationality> rsp = ycqLeagueDao.getLmNationalityBigData(req);
        handleLmNationality(rsp.getResult());
        return rsp;
    }

    @Override
    public LeagueOrgDetailVo getOrgDetail(String leagueCode) {
        throwNotLeagueCodeException(leagueCode);
        return ycqLeagueDao.getOrgDetail(leagueCode);
    }

    @Override
    public LeagueSuperOrgDetailVo getSuperiorOrgDetail(String leagueCode) {
        throwNotLeagueCodeException(leagueCode);
        return ycqLeagueDao.getSuperiorOrgDetail(leagueCode);
    }


    @Override
    public PageRsp<YcqLeagueTreeVo> pageLeague(PageReq<PageLeague> req) {
        PageRsp<YcqLeagueTreeVo> rsp = ycqLeagueDao.pageLeague(req);
        List<YcqLeagueTreeVo> result = rsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return rsp;
        }
        for (YcqLeagueTreeVo ycqLeagueTreeVo : result) {
            String leagueCode = ycqLeagueTreeVo.getLeagueCode();
            List<YcqLeagueTreeVo> list = ycqLeagueDao.findParentLeagueCode(leagueCode);
            List<YcqLeagueTreeVo> parentLeague = new LinkedList<>();
            if(CollectionUtils.isEmpty(list)){
                continue;
            }
            String parentLeagueCode = ycqLeagueTreeVo.getParentLeagueCode();
            Map<String,YcqLeagueTreeVo> mapLeague = new HashMap<>(list.size()*2);
            for (YcqLeagueTreeVo leagueTreeVo : list) {
                mapLeague.put(leagueTreeVo.getLeagueCode(),leagueTreeVo);
            }
            for (int i = 0; i < mapLeague.size(); i++) {
                YcqLeagueTreeVo ycqLeagueTreeVo1 = mapLeague.get(parentLeagueCode);
                parentLeague.add(ycqLeagueTreeVo1);
                if(Objects.nonNull(ycqLeagueTreeVo1) && StringUtils.hasText(ycqLeagueTreeVo1.getParentLeagueCode())){
                    parentLeagueCode = ycqLeagueTreeVo1.getParentLeagueCode();
                }else {
                    break;
                }
            }
            ycqLeagueTreeVo.setParentLeague(parentLeague);
        }
        return rsp;
    }

    @Override
    @InjectProperty
    public NationalityDataVo getLmNationalityBigDataEcharts(String leagueCode) {
        throwNotLeagueCodeException(leagueCode);
        List<MemberNationality> memberNationalities = ycqLeagueDao.getLmNationalityBigDataEcharts(leagueCode);
        return handleLmNationalityEcharts(memberNationalities);
    }

    @Override
    public List<YcqLeagueTreeVo> getPublicOrgChildren(String leagueCode, Worker worker) {
        List<YcqLeagueTreeVo> ycqLeagueVos;
        if(CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())){
            ycqLeagueVos = this.getOrgChildren(leagueCode);
        }else {
            ycqLeagueVos = ycqLeagueDao.getPublicOrgChildren(leagueCode,worker.getOrgId());
        }
        paddingLeaf(ycqLeagueVos);
        return ycqLeagueVos;
    }

    @Override
    public List<AgeDataVo> getLmAgeBigDataEcharts(String leagueCode) {
        throwNotLeagueCodeException(leagueCode);
        List<AgeDataVo> ageDataVos = ycqLeagueDao.getLmAgeBigDataEcharts(leagueCode);
        if(CollectionUtils.isEmpty(ageDataVos)) {
            return ageDataVos;
        }
        handleAgeData(ageDataVos);
        return ageDataVos;
    }

    @Override
    public NationalityDataVo getLmNationalityCount(String leagueCode) {
        throwNotLeagueCodeException(leagueCode);
        NationalityDataVo nationalityDataVo = ycqLeagueDao.findNationalityCount(leagueCode);
        handleLmNationalityCount(nationalityDataVo);
        return nationalityDataVo;
    }

    /**
     * 组织团员数量
     */
    private void handleLmNationalityCount(NationalityDataVo nationalityDataVo) {
        DecimalFormat df =new DecimalFormat();
        df.setMaximumFractionDigits(1);
        df.setMinimumFractionDigits(1);
        String hanLmProportion = "0.0";
        String minorityLmProportion = "0.0";
        if(Objects.isNull(nationalityDataVo.getLmSumCount())){
            nationalityDataVo.setLmSumCount(0);
        }
        if(Objects.isNull(nationalityDataVo.getHanLmCount())){
            nationalityDataVo.setHanLmCount(0);
        }
        if(Objects.isNull(nationalityDataVo.getMinorityLmCount())){
            nationalityDataVo.setMinorityLmCount(0);
        }
        if(nationalityDataVo.getLmSumCount() != 0){
            // 少数名族
            minorityLmProportion = df.format(nationalityDataVo.getMinorityLmCount() *100.00 / nationalityDataVo.getLmSumCount());
            // 汉族
            hanLmProportion = df.format(nationalityDataVo.getHanLmCount() *100.00 / nationalityDataVo.getLmSumCount());
        }
        nationalityDataVo.setHanLmProportion(hanLmProportion);
        nationalityDataVo.setMinorityLmProportion(minorityLmProportion);
    }

    /**
     * 统一处理组织编码不能为空
     */
    private void throwNotLeagueCodeException(String leagueCode){
        if(!StringUtils.hasText(leagueCode)){
            throw new BusinessException("组织编码不能为空");
        }
    }

    /**
     * 处理团员大数据-名族分布
     * @param memberNationalities 组织列表
     */
    private void handleLmNationality(List<MemberNationality> memberNationalities) {
        int lmSumCount = memberNationalities.stream().mapToInt(MemberNationality::getMemberNum).sum();
        if(lmSumCount == 0){
            return;
        }
        DecimalFormat df =new DecimalFormat();
        df.setMaximumFractionDigits(1);
        df.setMinimumFractionDigits(1);
        // 处理列表团员百分比
        for (MemberNationality memberNationality : memberNationalities) {
            String nationality = df.format(memberNationality.getMemberNum() *100.00 / lmSumCount);
            memberNationality.setLmProportion(nationality);
        }
    }

    /**
     * 处理团员大数据-名族分布
     * @param memberNationalities 组织列表
     * @return data
     */
    private NationalityDataVo handleLmNationalityEcharts(List<MemberNationality> memberNationalities) {
        int lmSumCount = memberNationalities.stream().mapToInt(MemberNationality::getMemberNum).sum();
        NationalityDataVo nationalityDataVo = new NationalityDataVo();
        if(lmSumCount == 0){
            nationalityDataVo.setMemberNationalities(memberNationalities);
            return nationalityDataVo;
        }
        Integer hanLmCount = 0;
        String hanLmProportion = "0.0";
        DecimalFormat df =new DecimalFormat();
        df.setMaximumFractionDigits(1);
        df.setMinimumFractionDigits(1);
        // 处理列表团员百分比
        for (MemberNationality memberNationality : memberNationalities) {
            String nationality = df.format(memberNationality.getMemberNum() *100.00 / lmSumCount);
            memberNationality.setLmProportion(nationality);
            if(LeagueConstants.HAN_LEAGUE_CODE.equals(memberNationality.getNationId())){
                hanLmCount = memberNationality.getMemberNum();
                hanLmProportion = memberNationality.getLmProportion();
            }
        }
        // 少数民族处理
        int minorityLmCount = lmSumCount - hanLmCount;
        String minorityLmProportion = df.format(minorityLmCount *100.00 / lmSumCount);
        // 赋值返回
        nationalityDataVo.setLmSumCount(lmSumCount);
        nationalityDataVo.setHanLmCount(hanLmCount);
        nationalityDataVo.setHanLmProportion(hanLmProportion);
        nationalityDataVo.setMinorityLmCount(minorityLmCount);
        nationalityDataVo.setMinorityLmProportion(minorityLmProportion);
        nationalityDataVo.setMemberNationalities(memberNationalities);
        return nationalityDataVo;
    }

    /**
     * 处理年龄分布
     */
    private void handleAgeData(List<AgeDataVo> ageDataVos) {
        ageDataVos.forEach(ageDataVo -> {
            Long lmCount = ageDataVo.getLmCount();
            // 防止男女团员年龄总数为0
            if(lmCount == 0){
                ageDataVo.setFemaleProportion("0.0");
                ageDataVo.setMaleProportion("0.0");
                return;
            }
            DecimalFormat df =new DecimalFormat();
            df.setMaximumFractionDigits(1);
            df.setMinimumFractionDigits(1);
            String femaleAge = df.format(ageDataVo.getFemaleLm() *100.00 / lmCount);
            String maleAge = df.format(ageDataVo.getMaleLm() *100.00 / lmCount);
            ageDataVo.setFemaleProportion(femaleAge);
            ageDataVo.setMaleProportion(maleAge);
            ageDataVo.setLmCount(lmCount);
        });
    }


    /**
     * 填充属性
     * @param result YcqLeagueVo
     */
    private void paddingLeaf(List<YcqLeagueTreeVo> result) {
        List<String> collect = result.stream().map(YcqLeagueTreeVo::getLeagueCode).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(collect)){
            return;
        }
        List<YcqLeagueCountDto> count = ycqLeagueDao.findIsHaveChidren(collect);
        List<String> haveLeagueCode = count.stream().map(YcqLeagueCountDto::getLeagueCode).collect(Collectors.toList());
        for (YcqLeagueTreeVo ycqLeagueVo : result) {
            if(!haveLeagueCode.contains(ycqLeagueVo.getLeagueCode())){
                ycqLeagueVo.setLeaf(true);
            }
        }
    }

    public static void main(String[] args) {
        String  a= "{\"position\":\"委员\",\"nature\":\"专职团干\",\"time\":\"2021年10月-至今\",\"leagueName\":\"重庆市重庆三峡学院外国语学院英语2021级2班团支部\"}";
        HonorVo honorVo = JSONUtil.toBean(a, HonorVo.class);
        System.out.println(honorVo);
    }

}
