package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SmsTemplateHandler;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.message.SendMessageTaskReqDTO;
import com.crunii.micro.domain.entity.main.YcqMessageTemplate;
import com.crunii.micro.domain.repo.main.YcqMessageTemplateRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.MessageService;
import com.crunii.micro.domain.dto.message.SaveMessageTaskReqBo;
import com.crunii.micro.service.match.enums.MessageReplaceEnum;
import com.crunii.micro.service.match.service.MatchMessageCommonService;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.TreeMap;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

/**
 * @Author thw
 * @date 2023/11/27
 * @description
 */
@Slf4j
@Service
public class MatchMessageCommonServiceImpl implements MatchMessageCommonService {

  @Autowired
  private MessageService messageService;

  @Resource
  private YcqMessageTemplateRepo ycqMessageTemplateRepo;


  /** 推送消息到发送表中 */
  @Override
  public Result saveMessage(SaveMessageTaskReqBo taskReqDto){

    if (ObjectUtil.isEmpty(taskReqDto) || ObjectUtil.isEmpty(taskReqDto.getSystemName()) || ObjectUtil.isEmpty(taskReqDto.getSceneName())
    || ObjectUtil.isEmpty(taskReqDto.getTitle()) || ObjectUtil.isEmpty(taskReqDto.getValueList()) || ObjectUtil.isEmpty(taskReqDto.getScopeType())){
      throw new BusinessException("必传参数为空!");
    }

    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    YcqMessageTemplate messageTemplate = getMessageTemplateContent(taskReqDto.getSystemName(), taskReqDto.getSceneName(), taskReqDto.getTitle(),
        (byte) 0);
    // 按顺序添加模板值,返回最终封装好的消息内容
    String messageFinal = getFinalMessage(messageTemplate.getMessageTemplate(),taskReqDto.getValueList());

    SendMessageTaskReqDTO sendMessageTaskReqDTO = new SendMessageTaskReqDTO();
    // 发送范围
    sendMessageTaskReqDTO.setScopeType(taskReqDto.getScopeType());
    // 范围数据(逗号拼接ID)
    sendMessageTaskReqDTO.setScopeData(taskReqDto.getScopeData());
    // 推送方式
    sendMessageTaskReqDTO.setSendWay(messageTemplate.getPushType());
    // 推送终端
    sendMessageTaskReqDTO.setMessageType(messageTemplate.getPushEndpoint());
    // 消息发送内容
    sendMessageTaskReqDTO.setMessageContent(messageFinal);
    // 消息来源
    sendMessageTaskReqDTO.setDataSource(messageTemplate.getSystemName());
    // 消息分类
    sendMessageTaskReqDTO.setMessageClassify(messageTemplate.getSceneName());
    // 模板ID
    sendMessageTaskReqDTO.setTemplateId(messageTemplate.getTemplateId());
    // 发送人id
    sendMessageTaskReqDTO.setSendLoginId((long) currWorker.getLoginId());

    sendMessageTaskReqDTO.setFillData(SmsTemplateHandler.fillTemplateAndStoreInDatabase(messageTemplate.getMessageTemplate(),taskReqDto.getValueList()));

    messageService.saveMessageTask(sendMessageTaskReqDTO);
    log.info(messageFinal);

    return Result.ofSuccess();
  }



  /***
   * @Description:  得到最终的消息内容
   * @Author: thw
   * @Date: 2023/11/27 11:28
   * @Params:
   * @param messageTemplate   获取到的消息模板
   * @param valueList         按顺序设置需要替换模板值的List
   * @return: java.lang.String
   **/
  private String getFinalMessage(String messageTemplate, List<String> valueList) {
    Assert.notEmpty(valueList,"参数值列表不能为空!");
    List<String> replaceTempleNameList = getReplaceTempleNameList(messageTemplate);
    if (ObjectUtil.isNotEmpty(valueList) && valueList.size() != replaceTempleNameList.size()){
      throw new BusinessException("模板替换参数与传入值个数不匹配!");
    }
    HashMap<String, String> finalMap = new HashMap<>();
    if (ObjectUtil.isNotEmpty(replaceTempleNameList)){
      for (int i = 0; i < replaceTempleNameList.size(); i++) {
        finalMap.put(replaceTempleNameList.get(i),valueList.get(i));
      }
    }
    String messageFinal = messageTemplate;
    if (ObjectUtil.isNotEmpty(finalMap)){
      for (Entry<String, String> entry : finalMap.entrySet()) {
        messageFinal = messageFinal.replace(entry.getKey(), entry.getValue());
      }
    }
    return messageFinal;
  }


  /***
   * @Description:  按顺序获取需要替换的模板字段，后续需要设置模板对应的值
   * @Author: thw
   * @Date: 2023/11/27 12:28
   * @Params:
   * @param messageTemplate   消息模板
   * @return: java.util.List<java.lang.String>
   **/
  private List<String> getReplaceTempleNameList(String messageTemplate) {
    List<String> allName = MessageReplaceEnum.getAllName();
    TreeMap<Integer, String> needReplaceNameMap = new TreeMap<>();
    if (ObjectUtil.isNotEmpty(allName)){
      for (String name : allName) {
        int index = messageTemplate.indexOf(name);
        if (index > 0){
          needReplaceNameMap.put(index,name);
        }
      }
    }
    List<String> valueList = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(needReplaceNameMap)){
      for (Entry<Integer, String> entry : needReplaceNameMap.entrySet()) {
        valueList.add(entry.getValue());
      }
    }
    return valueList;
  }

  /***
   * @Description:  获取消息模板对象
   * @Author: thw
   * @Date: 2023/11/27 12:29
   * @Params:
   * @param systemName  系统名称（应用业务系统）
   * @param sceneName   场景名称（业务场景名称）
   * @param title       消息标题
   * @param delFlag     数据是否有效（0有效）
   * @return: java.lang.String
   **/
  public YcqMessageTemplate getMessageTemplateContent(String systemName, String sceneName, String title,
      Byte delFlag) {
    YcqMessageTemplate template = ycqMessageTemplateRepo
        .findBySystemNameAndSceneNameAndTitleAndDelFlag(systemName,sceneName,title,delFlag);
    if (ObjectUtil.isEmpty(template)){
      throw new BusinessException("根据业务系统名称,业务场景名称与消息标题未找到相应数据。");
    }
    return template;
  }


}
