package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.AESUtil;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.RSAUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.platform.components.Locker;
import com.crunii.micro.service.match.api.bo.MatchReviewerPageReqBo;
import com.crunii.micro.service.match.api.bo.MatchReviewerReqBo;
import com.crunii.micro.service.match.api.bo.MatchReviewerUpBo;
import com.crunii.micro.service.match.api.vo.MatchReviewerPageVO;
import com.crunii.micro.service.match.enums.MatchInsertRoleEnum.RoleType;
import com.crunii.micro.service.match.enums.RoundEnums.LineEnum;
import com.crunii.micro.service.match.enums.RoundEnums.RoundType;
import com.crunii.micro.service.match.enums.RoundEnums.ScoreMode;
import com.crunii.micro.service.match.jdbc.MatchReviewerManageDao;
import com.crunii.micro.service.match.service.MatchReviewerManageService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.Transactional;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

/**
 * @Author thw
 * @date 2023/11/20
 * @description 评审员相关
 */
@Service
public class MatchReviewerManageServiceImpl implements MatchReviewerManageService {

  @Autowired
  private MatchReviewerManageDao matchReviewerManageDao;

  @Autowired
  private MatchOrganizerAccountServiceImpl matchOrganizerAccountService;

  @Autowired
  private MthRoundInstanceRepo mthRoundInstanceRepo;

  @Autowired
  private MthMatchItemRepo mthMatchItemRepo;

  @Autowired
  private TsmWorkerRepo tsmWorkerRepo;

  @Autowired
  private MthRoundReviewConfRepo mthRoundReviewConfRepo;

  @Autowired
  private MthRoundReviewerRepo mthRoundReviewerRepo;

  @Autowired
  private Locker locker;

  ////生成评审员临时账号锁参数
  private final static String MATCH_REVIEWER = "LOCK:MATCH:REVIEWER";

  private String ROLE_NAME = RoleType.REVIEWER_ROLE.getName();

  // 线上评审
  private String ONLINE = LineEnum.ONLINE.getCode();
  // 场次状态开启
  private String ROUND_STATE = "0";
  // 是否按赛项生成账号（1:是,0:否）
  private String IS_MATCH_ITEM = "1";
  private String IS_NOT_MATCH_ITEM = "0";

  @Override
  public PageRsp<MatchReviewerPageVO> pageReviewerAccounts(PageReq<MatchReviewerPageReqBo> bo) {

    // 默认场次id必传
    if (ObjectUtil.isNotEmpty(bo) && ObjectUtil.isEmpty(bo.getQuery().getRoundId())) {
      throw new BusinessException("场次id必传!");
    }
    Map<String, Object> props = new HashMap<>();
    props.put("roundId", bo.getQuery().getRoundId());
    props.put("delFlag", 0);
    List<MthRoundReviewConf> reviewConfs = mthRoundReviewConfRepo.findByProps(props);
    if (CollectionUtils.isEmpty(reviewConfs)){
      throw new BusinessException("未通过场次id找到评审员配置信息!");
    }
    if (reviewConfs.size() > 1){
      throw new BusinessException("未通过场次id找到多条评审员配置信息，数据有误!");
    }
    MthRoundReviewConf conf = reviewConfs.get(0);
    PageRsp<MatchReviewerPageVO> pageRsp = matchReviewerManageDao.pageReviewerAccounts(bo);
    if (ObjectUtil.isNotEmpty(pageRsp.getResult())){
      for (MatchReviewerPageVO vo : pageRsp.getResult()) {
        // 评审员临时账号
        vo.setMobile(RSAUtil.encrypt(vo.getMobile()));
        // 密码解密后加密
        vo.setReviewerPwd(RSAUtil.encrypt(AESUtil.decrypt(vo.getReviewerPwd())));
        vo.setFlag(String.valueOf(conf.getMatchItemFlag()));
        vo.setReviewCount(conf.getReviewCount());
      }
    }
    return pageRsp;
  }

  @Override
  @Transactional(rollbackOn = Exception.class)
  public Result saveReviewerAccount(MatchReviewerReqBo bo) {

    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    if (ObjectUtil.isEmpty(bo) || ObjectUtil.isEmpty(bo.getRoundId())) {
      throw new BusinessException("必填必传参数未传!");
    }

    locker.lock(MATCH_REVIEWER, bo.getRoundId().toString());

    try {

      // 查询场次实例表通过场次id
      Optional<MthRoundInstance> instanceOptional = mthRoundInstanceRepo.findById(bo.getRoundId());
      if (!instanceOptional.isPresent()) {
        throw new BusinessException("通过场次id为未到相关数据!");
      }
      MthRoundInstance instance = instanceOptional.get();

      // 需要分辨的评分方式
      List<String> scoreModeExclude = new ArrayList<>();
      scoreModeExclude.add(ScoreMode.PASS.getCode());
      scoreModeExclude.add(ScoreMode.GRAD.getCode());

      if (!ROUND_STATE.equals(String.valueOf(instance.getRoundState())) || !ONLINE.equals(instance.getAuditMode())) {
        throw new BusinessException("场次开启且线上评审才能操作生成评审员账号。");
      }

      // 查询是否已经生成评审员账号
      HashMap<String, Object> prop = Maps.newHashMap();
      prop.put("roundId",bo.getRoundId());
      prop.put("delFlag",0);
      List<MthRoundReviewer> roundReviewers = mthRoundReviewerRepo.findByProps(prop);
      if (CollectionUtils.isNotEmpty(roundReviewers)){
        throw new BusinessException("该场次已经存在评审员，不允许重复生成！");
      }else {

        // 判断评分方式设置为ABC、通过不通过，评审次数默认为1
        int scoreCount = 1;
        if (!scoreModeExclude.contains(instance.getSocreMode())) {
          scoreCount = bo.getReviewCount();
        }

        int totalCount = 0;
        // 是否按赛项生成账号
        if (IS_MATCH_ITEM.equals(bo.getFlag())) {
          Map<String, Object> props = new HashMap<>();
          props.put("matchId", instance.getMatchId());
          props.put("delFlag", 0);
          // 通过比赛id查询所属赛项类别
          List<MthMatchItem> matchItems = mthMatchItemRepo.findByProps(props);
          if (CollectionUtils.isEmpty(matchItems)) {
            throw new BusinessException("根据比赛id未找到相关赛项信息");
          }
          // 按场次或赛项需要评审的次数
          totalCount = matchItems.size() * scoreCount;
          // 创建评审员临时账号并为账号添加角色
          List<TsmWorker> tsmWorkers = createUserAndAddRole(totalCount, ROLE_NAME);
          // 创建-评审员配置信息
          createReviewConfig(currWorker, instance.getId(), scoreCount, Integer.valueOf(IS_MATCH_ITEM));
          // 创建评审员通过赛项分类时
          createRoundReviewerByItem(currWorker, instance, scoreCount, matchItems, tsmWorkers,
              Boolean.TRUE);
        } else {
          // 按场次或赛项需要评审的次数
          totalCount = scoreCount;
          // 创建评审员临时账号并为账号添加角色
          List<TsmWorker> tsmWorkers = createUserAndAddRole(totalCount, ROLE_NAME);
          // 创建-评审员配置信息
          createReviewConfig(currWorker, instance.getId(), scoreCount,
              Integer.valueOf(IS_NOT_MATCH_ITEM));
          // 创建评审员
          createRoundReviewer(currWorker, instance, scoreCount, tsmWorkers, Boolean.FALSE);
        }

      }
    }finally {
      locker.unlock();
    }
    return Result.ofSuccess();
  }

  @Override
  public Result updateReviewerAccount(List<MatchReviewerUpBo> bo) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    if (ObjectUtil.isEmpty(bo)) {
      throw new BusinessException("必传参数为空!");
    }
    Set<Long> ids = bo.stream().map(MatchReviewerUpBo::getId).collect(Collectors.toSet());
    List<MthRoundReviewer> roundReviewerList = mthRoundReviewerRepo.findAllById(ids);
    if (CollectionUtils.isNotEmpty(roundReviewerList)) {
      for (MthRoundReviewer reviewer : roundReviewerList) {
        for (MatchReviewerUpBo upBo : bo) {
          if (reviewer.getId().equals(upBo.getId())) {
            // 添加备注
            reviewer.setRemarks(upBo.getRemarks());
            reviewer.setUpdateId((long) currWorker.getLoginId());
            reviewer.setUpdateBy(currWorker.getWorkerName());
            reviewer.setUpdateTime(new Date());
          }
        }
      }
    }
    mthRoundReviewerRepo.saveAll(roundReviewerList);
    return Result.ofSuccess();
  }

  @Override
  public void exportReviewerAccount(HttpServletResponse response, MatchReviewerPageReqBo bo) {
    // 默认场次id必传
    if (ObjectUtil.isEmpty(bo) || ObjectUtil.isEmpty(bo.getRoundId())) {
      throw new BusinessException("场次id必传!");
    }
    PageReq<MatchReviewerPageReqBo> pageReq = new PageReq<>();
    pageReq.setQuery(bo);
    pageReq.setPageSize(1000000);
    List<MatchReviewerPageVO> result = matchReviewerManageDao.pageReviewerAccounts(pageReq).getResult();

    if (ObjectUtil.isNotEmpty(result)){
      for (MatchReviewerPageVO vo : result) {
        // 评审员临时账号
//        vo.setMobile(RSAUtil.encrypt(vo.getMobile()));
        // 密码解密后加密
        vo.setReviewerPwd(AESUtil.decrypt(vo.getReviewerPwd()));
      }
    }

    // 查询场次的评审员配置表得到是否按照赛项分类
    HashMap<String, Object> props = new HashMap<>();
    props.put("roundId", bo.getRoundId());
    props.put("delFlag", 0);
    List<MthRoundReviewConf> mthRoundReviewConfs = mthRoundReviewConfRepo.findByProps(props);
    if (CollectionUtils.isEmpty(mthRoundReviewConfs)) {
      throw new BusinessException("未通过场次id找到评审员配置信息。");
    }
    if (mthRoundReviewConfs.size() > 1) {
      throw new BusinessException("通过场次id找到多条评审员配置信息，数据有误！");
    }
    MthRoundReviewConf conf = mthRoundReviewConfs.get(0);

    // 封装表头
    String[] headContent = {"赛项", "账号", "密码", "备注"};
    List<List<String>> headTable = new ArrayList<>();
    List<String> headTemp;
    if (IS_MATCH_ITEM.equals(String.valueOf(conf.getMatchItemFlag()))) {
      for (int i = 0; i < headContent.length; i++) {
        headTemp = new ArrayList<>();
        headTemp.add(headContent[i]);
        headTable.add(headTemp);
      }
    }else {
      for (int i = 1; i < headContent.length; i++) {
        headTemp = new ArrayList<>();
        headTemp.add(headContent[i]);
        headTable.add(headTemp);
      }
    }

    List<List<Object>> tableContent = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(result)){
      // 封装数据内容
      if (IS_MATCH_ITEM.equals(String.valueOf(conf.getMatchItemFlag()))){
        for (MatchReviewerPageVO vo : result) {
          tableContent.add(Arrays.asList(vo.getItemName(),vo.getMobile(),vo.getReviewerPwd(),vo.getRemarks()));
        }
      }else {
        for (MatchReviewerPageVO vo : result) {
          tableContent.add(Arrays.asList(vo.getMobile(),vo.getReviewerPwd(),vo.getRemarks()));
        }
      }
    }


    try{

      String fileName = "评审员账号导出" + DateUtil.format(new Date(), DateUtil.defaultDT1Pattern) + ExcelTypeEnum.XLSX.getValue();
      response.setContentType("application/vnd.ms-excel;charset=utf-8");
      // 文件名称
      fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name());
      response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
          "attachment;filename=" + fileName);
      ServletOutputStream out = response.getOutputStream();
      ExcelWriter excelWriter = EasyExcel.write(out).build();
      // 判断否为导出模板
      if (ObjectUtil.isNotEmpty(bo.getExportFlag()) && bo.getExportFlag()){
        excelWriter.write(Lists.newArrayList(), EasyExcel.writerSheet("评审员账号").head(headTable).build());
      }else {
        excelWriter.write(tableContent, EasyExcel.writerSheet("评审员账号").head(headTable).build());
      }
      excelWriter.finish();
    }catch (Exception e){
      throw new BusinessException("导出失败!");
    }
  }

  /**
   * 创建评审员临时账号
   */
  private List<TsmWorker> createUserAndAddRole(int totalCount, String roleName) {
    // 创建评审员临时账号
    List<TsmWorker> tsmWorkers = matchOrganizerAccountService.doAddTsmWorkers(totalCount, "评审员账号");
    Set<Integer> workerIds = tsmWorkers.stream().map(e -> e.getLoginId())
        .collect(Collectors.toSet());
    TsmRole role = matchOrganizerAccountService.getTsmRoles(roleName);
    // 评审员添加角色
    matchOrganizerAccountService.insertWorkerRole(workerIds, role.getRoleId());
    return tsmWorkers;
  }

  /***
   * @Description: 创建评审员，按赛项分类时
   * @Author: thw
   * @Date: 2023/11/21 11:27
   * @Params:
   * @param currWorker    当前用户
   * @param instance      场次实例
   * @param scoreCount    判断评分方式设置为ABC、通过不通过，评审次数默认为1，其它则为参数值
   * @param matchItems    赛项对象列表
   * @param tsmWorkers    新增评审员临时账号
   * @param label         label 为true，则说明是按照赛项分类新增
   * @return: void
   **/
  private void createRoundReviewerByItem(Worker currWorker, MthRoundInstance instance,
      int scoreCount,
      List<MthMatchItem> matchItems, List<TsmWorker> tsmWorkers, Boolean label) {
    ArrayList<MthRoundReviewer> mthRoundReviewers = new ArrayList<>();
    int countControl = 0;
    for (int i = 0; i < matchItems.size(); i++) {
      MthMatchItem mthMatchItem = matchItems.get(i);
      for (int j = 0; j < scoreCount; j++) {
        TsmWorker tsmWorker = tsmWorkers.get(countControl);
        // 创建评审员
        setRoundReviewerObjectInfo(currWorker, instance, label, mthRoundReviewers, mthMatchItem,
            tsmWorker);
        countControl++;
      }
    }
    mthRoundReviewerRepo.saveAllAndFlush(mthRoundReviewers);

  }


  /***
   * @Description: 创建评审员, 不通过赛项分类
   * @Author: thw
   * @Date: 2023/11/21 14:44
   * @Params:
   * @param currWorker  当前用户
   * @param instance    场次实例
   * @param scoreCount  评审次数
   * @param tsmWorkers  评审员账号信息
   * @param label       是否为赛项分类
   * @return: void
   **/
  private void createRoundReviewer(Worker currWorker, MthRoundInstance instance, int scoreCount,
      List<TsmWorker> tsmWorkers, Boolean label) {
    List<MthRoundReviewer> mthRoundReviewers = new ArrayList<>();
    for (int j = 0; j < scoreCount; j++) {
      TsmWorker tsmWorker = tsmWorkers.get(j);
      // 创建评审员
      setRoundReviewerObjectInfo(currWorker, instance, label, mthRoundReviewers, null,
          tsmWorker);
    }
    mthRoundReviewerRepo.saveAllAndFlush(mthRoundReviewers);

  }

  /***
   * @Description: 填充评审员对象信息
   * @Author: thw
   * @Date: 2023/11/21 14:24
   * @Params:
   * @param currWorker 当前用户
   * @param instance   场次实例
   * @param label      是否根据赛项分类
   * @param mthRoundReviewers 评审员添加集合
   * @param mthMatchItem  赛项对象
   * @param tsmWorker     评审员对象
   * @return: void
   **/
  private void setRoundReviewerObjectInfo(Worker currWorker, MthRoundInstance instance,
      Boolean label, List<MthRoundReviewer> mthRoundReviewers, MthMatchItem mthMatchItem,
      TsmWorker tsmWorker) {
    MthRoundReviewer mthRoundReviewer = new MthRoundReviewer();
    mthRoundReviewer.setRoundId(instance.getId());
    mthRoundReviewer.setReviewerId((long) tsmWorker.getLoginId());
    mthRoundReviewer.setDelFlag(0);
    if (label) {
      // 设置赛项信息
      mthRoundReviewer.setItemId(mthMatchItem.getId());
      mthRoundReviewer.setItemName(mthMatchItem.getItemName());
    }
    mthRoundReviewer.setCreateId((long) currWorker.getLoginId());
    mthRoundReviewer.setCreateBy(currWorker.getWorkerName());
    mthRoundReviewer.setCreateTime(new Date());
    mthRoundReviewers.add(mthRoundReviewer);
  }


  /***
   * @Description: 创建-评审员配置信息
   * @Author: thw
   * @Date: 2023/11/21 11:29
   * @Params:
   * @param currWorker  当前用户
   * @param roundId     场次id
   * @param reviewCount 每个赛项或整个场次需要评审的次数
   * @param itemFlag     按照赛事(0:否，1：是)
   * @return: void
   **/
  private void createReviewConfig(Worker currWorker, Long roundId, int
      reviewCount, Integer itemFlag) {
    MthRoundReviewConf mthRoundReviewConf = new MthRoundReviewConf();
    mthRoundReviewConf.setRoundId(roundId);
    mthRoundReviewConf.setMatchItemFlag(itemFlag);
    // 设置每个赛项或整个场次需要评审的次数
    mthRoundReviewConf.setReviewCount(reviewCount);
    // 设置评审方式 review_type 以弃
    // mthRoundReviewConf.setReviewType();
    mthRoundReviewConf.setDelFlag(0);
    mthRoundReviewConf.setCreateId((long) currWorker.getLoginId());
    mthRoundReviewConf.setCreateBy(currWorker.getWorkerName());
    mthRoundReviewConf.setCreateTime(new Date());
    mthRoundReviewConfRepo.saveAndFlush(mthRoundReviewConf);
  }

  /** 通过比赛id查询到属于该比赛的所有评审员 */
  public List<Long> getReviewerAccount(Long matchId){
    List<Long> loginIds = new ArrayList<>();
    List<MthRoundInstance> mthRoundInstances = mthRoundInstanceRepo.findByMatchIdAndRoundType(
        matchId, RoundType.INSTANCE.getCode());
    if (ObjectUtil.isNotEmpty(mthRoundInstances)){
      List<Long> roundIds = mthRoundInstances.stream().filter(e -> ObjectUtil.isNotEmpty(e)).map(MthRoundInstance::getId).distinct()
          .collect(Collectors.toList());
      List<MthRoundReviewer> mthRoundInstanceList = mthRoundReviewerRepo.findAllByRoundIdIn(roundIds);
      if (ObjectUtil.isNotEmpty(mthRoundInstanceList)){
        loginIds = mthRoundInstanceList.stream().filter(e -> ObjectUtil.isNotEmpty(e)).map(MthRoundReviewer::getReviewerId).distinct()
            .collect(Collectors.toList());
      }
    }
    return loginIds;
  }
}
