package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthMatchSign;
import com.crunii.micro.domain.entity.main.MthMatchSignInformation;
import com.crunii.micro.domain.entity.main.MthRoundInstance;
import com.crunii.micro.domain.repo.main.MthMatchSignInformationRepo;
import com.crunii.micro.domain.repo.main.MthMatchSignRepo;
import com.crunii.micro.domain.repo.main.MthRoundInstanceRepo;
import com.crunii.micro.service.match.api.bo.SignInfoVo;
import com.crunii.micro.service.match.api.vo.sign.LocationDataVO;
import com.crunii.micro.service.match.api.vo.sign.MthSignInReqVo;
import com.crunii.micro.service.match.jdbc.MthMatchSignInDao;
import com.crunii.micro.service.match.service.MthMatchSignInService;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;

import static com.crunii.micro.common.constants.CommonConstants.MATCH_SIGN_IDENTIFY;
import static com.crunii.micro.service.match.constants.MatchMessageConstants.*;

/**
 * @author gongsongtao
 * @date 2023/11/21 18:49
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class MthMatchSignInServiceImpl implements MthMatchSignInService {

    private final MthMatchSignRepo mthMatchSignRepo;
    private final MthRoundInstanceRepo mthRoundInstanceRepo;
    private final MthMatchSignInDao mthMatchSignInDao;
    private final MthMatchSignInformationRepo mthMatchSignInformationRepo;

    private static final String CHARSET = "utf-8";
    private static final String FORMAT_NAME = "JPG";
    // 二维码尺寸
    private static final int QRCODE_SIZE = 330;
    // 字体大小
    private static final int FONT_SIZE = 15;
    //二维码底部文字
    private static final String SIGN_IN_TIME = "签到时间：";
    private static final String SIGN_IN_CONDITION = "签到地点：";
    private static final String SIGN_OUT_TIME = "签退时间：";
    private static final String SIGN_OUT_CONDITION = "签退地点：";
    private static final String SYMBOL = "-";

    /**
     * app签到接口
     * @param worker
     * @param mthSignInReqVo
     * @return
     */
    @Override
    public Result<Boolean> doSignIn(Worker worker, MthSignInReqVo mthSignInReqVo) {
        Result<Boolean> result=new Result();
        //校验该二维码
        if (!(mthSignInReqVo.getSignType()==2)){
            verifyQrCode(mthSignInReqVo);
        }
        //校验该用户是否能否签到
        verifyMatch(worker, mthSignInReqVo);
        //计算距离

        //写入签到数据
        if (mthSignInReqVo.getSignType()==1){
            MthMatchSign mthMatchSign=new MthMatchSign();
            mthMatchSign.setSignQrId(mthSignInReqVo.getQrId());
            mthMatchSign.setSignHandle(mthSignInReqVo.getHandle());
            mthMatchSign.setSignType(mthSignInReqVo.getSignType());
            mthMatchSign.setWorkerId(worker.getLoginId());
            mthMatchSign.setSignTime(new Date());
            mthMatchSign.setMthRoundListId(mthSignInReqVo.getRoundInstanceId());
            if (mthSignInReqVo.getHandle()==1){
                result.setMsg(MATCH_SIGN_IN_SUCCEED);
            }
            if (mthSignInReqVo.getHandle()==-1){
                result.setMsg(MATCH_SIGN_OUT_SUCCEED);
            }
            mthMatchSignRepo.save(mthMatchSign);
        }
        result.setData(true);
        return result;
    }

    /**
     * 生成签到二维码
     * @param mthSignInReqVo
     * @return
     */
    @Override
    public void createSignQrCode(HttpServletResponse response,MthSignInReqVo mthSignInReqVo) {
        //校验场次
        if (ObjectUtil.isEmpty(mthSignInReqVo.getRoundInstanceId()) || ObjectUtil.isEmpty(mthSignInReqVo.getHandle())){
            throw new BusinessException(MATCH_SIGN_PARAM_NONENTITY);
        }
        MthRoundInstance match = mthRoundInstanceRepo.getOne(mthSignInReqVo.getRoundInstanceId());
        if (ObjectUtil.isEmpty(match)){
            throw new BusinessException(MATCH_SIGN_ROUND_NONENTITY);
        }
        //是否需要生成签到二维码
        MthMatchSignInformation  information = mthMatchSignInformationRepo.findByInstanceIdAndHandle(mthSignInReqVo.getRoundInstanceId(), mthSignInReqVo.getHandle());
        if (ObjectUtil.isEmpty(information)){
            throw new BusinessException(MATCH_SIGN_INFORMATION_NULL);
        }
        if (information.getSignType()!=1 && mthSignInReqVo.getHandle()==1 && (StringUtils.isEmpty(information.getSignLongitude())||StringUtils.isEmpty(information.getSignLatitude()))){
            throw new BusinessException(MATCH_SIGN_IN_NOT_NEED);
        }
        if (information.getSignType()!=1 && mthSignInReqVo.getHandle()==-1 && (StringUtils.isEmpty(information.getSignLongitude())||StringUtils.isEmpty(information.getSignLatitude()))){
            throw new BusinessException(MATCH_SIGN_OUT_NOT_NEED);
        }

        //二维码包含的信息
        MthSignInReqVo qrCodeMsg=new MthSignInReqVo();
        qrCodeMsg.setRoundInstanceId(mthSignInReqVo.getRoundInstanceId());
        qrCodeMsg.setHandle(mthSignInReqVo.getHandle());
        qrCodeMsg.setSignType(information.getSignType());
        //二维码唯一标识
        String qrIdentifier = UUIDUtil.getUUID();
        System.out.println(qrIdentifier);
        qrCodeMsg.setQrId(qrIdentifier);
        information.setSignQrId(StringUtils.isNotEmpty(information.getSignQrId())?information.getSignQrId()+","+qrIdentifier:qrIdentifier);
        //系统标识
        qrCodeMsg.setIdentify(MATCH_SIGN_IDENTIFY);
        //下方备注文字
        SimpleDateFormat createTime = new SimpleDateFormat("MM-dd HH:mm");
        List<String> comments=new ArrayList<>();
        //签到信息
        if (mthSignInReqVo.getHandle() == 1 && information.getSignType()!=1) {
            //签到时间
            if (ObjectUtil.isNotEmpty(information.getSignStartTime()) && ObjectUtil.isNotEmpty(information.getSignEndTime())) {
                String startTime = createTime.format(information.getSignStartTime());
                String endTime = createTime.format(information.getSignEndTime());
                comments.add(SIGN_IN_TIME + startTime + SYMBOL + endTime);
            }
            //签到地点
            if (StringUtils.isNotEmpty(information.getSignSiteName())){
                comments.add(SIGN_IN_CONDITION+information.getSignSiteName()+" "+(StringUtils.isNotEmpty(information.getSignAddress())?information.getSignAddress():""));
            }
        }
        //签退信息
        if (mthSignInReqVo.getHandle() == -1 && information.getSignType()!=1) {
            //签退时间
            if (ObjectUtil.isNotEmpty(information.getSignStartTime()) && ObjectUtil.isNotEmpty(information.getSignEndTime())) {
                String startTime = createTime.format(information.getSignStartTime());
                String endTime = createTime.format(information.getSignEndTime());
                comments.add(SIGN_OUT_TIME + startTime + SYMBOL + endTime);
            }
            //签退地点
            if (StringUtils.isNotEmpty(information.getSignSiteName())) {
                comments.add(SIGN_OUT_CONDITION + information.getSignSiteName()+" "+(StringUtils.isNotEmpty(information.getSignAddress())?information.getSignAddress():""));
            }
        }
        //生成二维码
        try {
            byte[] qrCode = createQrCode(qrCodeMsg, comments);
            response.setContentType("application/octet-stream");
            String uuid = UUIDUtil.getUUID();
            response.setHeader("Content-Disposition", "attachment;filename=" + uuid + ".png");
            response.addHeader("Content-Length", "" + qrCode.length);
            BufferedOutputStream outputStream = new BufferedOutputStream(response.getOutputStream());            //ServletOutputStream outputStream = response.getOutputStream();
            outputStream.write(qrCode);
            outputStream.flush();
            outputStream.close();
        } catch (IOException | WriterException e) {
            log.info("生成二维码失败");
            e.printStackTrace();
        }
        //保存二维码唯一标识
        mthRoundInstanceRepo.save(match);
    }

    /**
     * web保存签到信息
     * @param signInfoVo
     * @return
     */
    @Override
    public Boolean saveSignInformation(SignInfoVo signInfoVo) {
        //校验签到信息
        if (signInfoVo==null){
            return false;
        }
        validSignInformation(signInfoVo);
        MthMatchSignInformation information;
        //编辑还是新增
        if (signInfoVo.getId() != null) {
            information = mthMatchSignInformationRepo.getOne(signInfoVo.getId());
            //判断签到时间，地点是否改变
            if (!(StringUtils.isNotEmpty(information.getSignQrId())
                    && signInfoVo.getSignSiteName().equals(information.getSignSiteName())
                    && (StringUtils.isNotEmpty(signInfoVo.getSignAddress()) && signInfoVo.getSignAddress().equals(information.getSignAddress()))
                    && information.getSignStartTime().equals(signInfoVo.getSignStartTime())
                    && information.getSignEndTime().equals(signInfoVo.getSignEndTime()))) {
                information.setSignQrId(null);
            }
            //校验开始时间距此刻是否大于24小时
            verifyTime(information,signInfoVo);
            //开始时间不能早于上一次设置的开始时间
            if (ObjectUtil.isNotEmpty(signInfoVo.getSignStartTime())
                    && ObjectUtil.isNotEmpty(information.getSignStartTime())
                    && signInfoVo.getSignStartTime().before(information.getSignStartTime())
                    && !signInfoVo.getSignStartTime().equals(information.getSignStartTime())) {
                throw new BusinessException(signInfoVo.getInformationType() == 1 ? MATCH_SIGN_IN_DATE_ONLY_AFTER : MATCH_SIGN_OUT_DATE_ONLY_AFTER);
            }
        }else {
            information= mthMatchSignInformationRepo.findByInstanceIdAndHandle(signInfoVo.getInstanceId(), signInfoVo.getInformationType());
            if (ObjectUtil.isEmpty(information)){
                information=new MthMatchSignInformation();
            }else {
                //校验开始时间距此刻是否大于24小时
                verifyTime(information,signInfoVo);
            }
        }
        BeanUtil.copyProperties(signInfoVo,information, "id","sign_qr_id");
        mthMatchSignInformationRepo.saveAndFlush(information);
        return true;
    }

    /**
     * 校验开始时间距此刻是否大于24小时
     * @param information
     * @param signInfoVo
     */
    private void verifyTime(MthMatchSignInformation information,SignInfoVo signInfoVo) {
        //校验签到/签退 开始时间距此刻是否大于24小时
        Date date=new Date();
        Calendar monthCalendar = Calendar.getInstance();
        monthCalendar.setTime(date);
        monthCalendar.add(Calendar.HOUR_OF_DAY, 24);
        Date time = monthCalendar.getTime();
        if (!signInfoVo.getSignStartTime().equals(information.getSignStartTime()) && !time.before(information.getSignStartTime()) && signInfoVo.getInformationType()==1){
            throw new BusinessException(MATCH_SIGN_IN_DATE_DEFICIENCY);
        }
        if (!signInfoVo.getSignStartTime().equals(information.getSignStartTime()) && !time.before(information.getSignStartTime()) && signInfoVo.getInformationType()==-1){
            throw new BusinessException(MATCH_SIGN_OUT_DATE_DEFICIENCY);
        }
    }

    /**
     * web查询签到信息
     * @param instanceId
     * @return
     */
    @Override
    public List<SignInfoVo> getSignInformation(Long instanceId) {
        List<SignInfoVo> result=new ArrayList<>();
        MthRoundInstance instance = mthRoundInstanceRepo.getOne(instanceId);
        if (ObjectUtil.isEmpty(instance) || instance.getDelFlag()==1){
            throw new BusinessException(MATCH_SIGN_ROUND_NONENTITY);
        }
        List<MthMatchSignInformation> informations = mthMatchSignInformationRepo.findByInstanceId(instanceId);
        for (MthMatchSignInformation information : informations) {
            SignInfoVo signInfoVo=new SignInfoVo();
            BeanUtil.copyProperties(information,signInfoVo);
            result.add(signInfoVo);
        }
        return result;
    }

    /**
     * 位置签到
     * @param worker
     * @param mthSignInReqVo
     * @return
     */
    @Override
    public Result<Boolean> doSiteSignIn(Worker worker, MthSignInReqVo mthSignInReqVo) {
        Result<Boolean>result=new Result<>();
        verifyMatch(worker, mthSignInReqVo);
        //写入签到信息
        MthMatchSign mthMatchSign=new MthMatchSign();
        mthMatchSign.setSignHandle(mthSignInReqVo.getHandle());
        mthMatchSign.setSignType(mthSignInReqVo.getSignType());
        mthMatchSign.setWorkerId(worker.getLoginId());
        mthMatchSign.setSignTime(new Date());
        mthMatchSign.setMthRoundListId(mthSignInReqVo.getRoundInstanceId());
        if (mthSignInReqVo.getHandle()==1){
            result.setMsg(MATCH_SIGN_IN_SUCCEED);
        }
        if (mthSignInReqVo.getHandle()==-1){
            result.setMsg(MATCH_SIGN_OUT_SUCCEED);
        }
        mthMatchSignRepo.save(mthMatchSign);
        return result;
    }

    /**
     * 计算经纬度
     * @param mthSignInReqVo
     * @return
     */
    @Override
    public Result<LocationDataVO> doCalculateDistance(MthSignInReqVo mthSignInReqVo) {
        return checkIn(mthSignInReqVo.getLatitude(),mthSignInReqVo.getLongitude(),mthSignInReqVo.getMatchLatitude(),mthSignInReqVo.getMatchLongitude()) ;
    }

    /**
     * 生成二维码
     * @param qrCodeMsg
     */
    private byte[] createQrCode(MthSignInReqVo qrCodeMsg,List<String> comments) throws IOException, WriterException {
        Hashtable hints = new Hashtable();
        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.M);
        hints.put(EncodeHintType.CHARACTER_SET, CHARSET);
        hints.put(EncodeHintType.MARGIN, 1);
        BitMatrix bitMatrix = new MultiFormatWriter().encode(JsonUtil.getJsonString(qrCodeMsg), BarcodeFormat.QR_CODE, QRCODE_SIZE, QRCODE_SIZE,
                hints);
        int width = bitMatrix.getWidth();
        int height = bitMatrix.getHeight();
        int tempHeight = height+45;

        BufferedImage image = new BufferedImage(width, tempHeight, BufferedImage.TYPE_INT_RGB);
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y, bitMatrix.get(x, y) ? 0xFF000000 : 0xFFFFFFFF);
            }
        }
        //添加底部文字
        if (comments.size()>0){
            addFontImage(image,comments);
        }
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ImageIO.write(image, FORMAT_NAME, outputStream);
        byte[] bytes = outputStream.toByteArray();
        return bytes;
    }

    /**
     * 底部添加文字
     * @param source
     */
    private void addFontImage(BufferedImage source,List<String> comments) {
        BufferedImage textImage = strToImage(QRCODE_SIZE, 45,comments);
        Graphics2D graph = source.createGraphics();
        //开启文字抗锯齿
        graph.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

        int width = textImage.getWidth(null);
        int height = textImage.getHeight(null);

        Image src = textImage;
        graph.drawImage(src, 0, QRCODE_SIZE, width, height, null);
        graph.dispose();
        return;
    }

    /**
     * 设置文字相关参数并转图片
     * @param width
     * @param height
     * @param comments
     * @return
     */
    private static BufferedImage strToImage(int width, int height,List<String> comments) {
        BufferedImage textImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = (Graphics2D) textImage.getGraphics();
        //开启文字抗锯齿
        g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g2.setBackground(Color.WHITE);
        g2.clearRect(0, 0, width, height);

        g2.setPaint(Color.BLACK);
        FontRenderContext context = g2.getFontRenderContext();
        Font font = new Font("新宋体", Font.BOLD, FONT_SIZE);
        g2.setFont(font);
        //设置底部文字
        int num=0;
        for (String comment : comments) {
            LineMetrics lineMetrics = font.getLineMetrics(comment, context);
            float y = num + lineMetrics.getHeight();
            g2.drawString(comment, 15,  y);
            num+=y;
        }
        return textImage;
    }

    /**
     * 校验二维码
     * @param mthSignInReqVo
     */
    private void verifyQrCode(MthSignInReqVo mthSignInReqVo) {
        //扫描的二维码是否是我们系统生成的签到二维码
        if (StringUtils.isEmpty(mthSignInReqVo.getIdentify()) || !mthSignInReqVo.getIdentify().equals(MATCH_SIGN_IDENTIFY)){
            throw new BusinessException(MATCH_SIGN_QR_CODE_ERROR);
        }
    }

    /**
     * 校验该用户能否签到
     * @param worker 登录用户
     * @param mthSignInReqVo 签到请求参数
     */
    private void verifyMatch(Worker worker, MthSignInReqVo mthSignInReqVo) {
        //校验请求参数
        if (mthSignInReqVo.getSignType()==null||ObjectUtil.isEmpty(mthSignInReqVo.getRoundInstanceId()) || ObjectUtil.isEmpty(mthSignInReqVo.getHandle())){
            log.info("签到参数异常：{}",mthSignInReqVo);
            throw new BusinessException(REQUEST_ERROR);
        }
        //该用户是否参数比赛
        MthRoundInstance newestInstance = mthMatchSignInDao.getNewestInstance(worker.getLoginId(),mthSignInReqVo.getRoundInstanceId());
        if (ObjectUtil.isEmpty(newestInstance) || !mthSignInReqVo.getRoundInstanceId().equals(newestInstance.getId())){
            throw new BusinessException(MATCH_SIGN_NOT_JOIN);
        }
        MthMatchSignInformation information = mthMatchSignInformationRepo.findByInstanceIdAndHandle(mthSignInReqVo.getRoundInstanceId(), mthSignInReqVo.getHandle());
        if (ObjectUtil.isEmpty(information)){
            throw new BusinessException(MATCH_SIGN_INFORMATION_NULL);
        }
        //是否满足签到条件
        Date date = new Date();
        if (mthSignInReqVo.getHandle() == 1) {
            //是否在签到时间段内
            if (date.before(information.getSignStartTime())) {
                throw new BusinessException(MATCH_SIGN_IN_TIME_BEFORE);
            }
            if (date.after(information.getSignEndTime())) {
                throw new BusinessException(MATCH_SIGN_IN_TIME_AFTER);
            }
            if (!information.getSignType().equals(2)) {
                //扫描二维码是否是最新生成的二维码
                if (StringUtils.isEmpty(information.getSignQrId()) || !information.getSignQrId().contains(mthSignInReqVo.getQrId())) {
                    throw new BusinessException(MATCH_SIGN_IN_QR_ERROR);
                }
            }
        }
        //是否满足签退条件
        if (mthSignInReqVo.getHandle() == -1) {
            //是否在签退时间段内
            if (date.before(information.getSignStartTime())) {
                throw new BusinessException(MATCH_SIGN_OUT_TIME_BEFORE);
            }
            if (date.after(information.getSignEndTime())) {
                throw new BusinessException(MATCH_SIGN_OUT_TIME_AFTER);
            }
            if (!information.getSignType().equals(2)){
                //扫描二维码是否是最新生成的二维码
                if (StringUtils.isEmpty(information.getSignQrId()) || !information.getSignQrId().contains(mthSignInReqVo.getQrId())){
                    throw new BusinessException(MATCH_SIGN_OUT_QR_ERROR);
                }
            }
        }
        //是否已签到/签退
        MthMatchSign signRecord = mthMatchSignRepo.findBySignRecord(newestInstance.getId(), worker.getLoginId(),mthSignInReqVo.getHandle());
        if (ObjectUtil.isNotEmpty(signRecord) && mthSignInReqVo.getHandle()==1){
            throw new BusinessException(MATCH_SIGNED_IN);
        }
        if (ObjectUtil.isNotEmpty(signRecord) && mthSignInReqVo.getHandle()==-1){
            throw new BusinessException(MATCH_SIGNED_OUT);
        }
    }

    /**
     * 校验信息
     *
     * @param signInfoVo
     */
    private void validSignInformation(SignInfoVo signInfoVo) {
        if (signInfoVo==null){
            return;
        }

        if (signInfoVo.getInstanceId()!=null){
            MthRoundInstance newestInstance = mthRoundInstanceRepo.getOne(signInfoVo.getInstanceId());
            if (ObjectUtil.isEmpty(newestInstance)||newestInstance.getDelFlag()==1){
                throw new BusinessException(MATCH_SIGN_ROUND_NONENTITY);
            }
        }
        //校验签到信息
        if (signInfoVo.getInformationType()==1) {
            //校验签到时间是否完善
            if ((ObjectUtil.isNotEmpty(signInfoVo.getSignStartTime()) && ObjectUtil.isEmpty(signInfoVo.getSignEndTime()))
                    || (ObjectUtil.isNotEmpty(signInfoVo.getSignEndTime()) && ObjectUtil.isEmpty(signInfoVo.getSignStartTime()))) {
                throw new BusinessException(MATCH_SIGN_IN_TIME_INCOMPLETE);
            }
            //有签到地点就必须有签到时间
            if ((ObjectUtil.isNotEmpty(signInfoVo.getSignLatitude()) || ObjectUtil.isNotEmpty(signInfoVo.getSignLongitude()))
                    && ObjectUtil.isEmpty(signInfoVo.getSignStartTime()) || ObjectUtil.isEmpty(signInfoVo.getSignEndTime())) {
                throw new BusinessException(MATCH_SIGN_IN_TIME_INCOMPLETE);
            }
            //校验签到地点是否完善
            if ((ObjectUtil.isNotEmpty(signInfoVo.getSignLatitude()) && ObjectUtil.isEmpty(signInfoVo.getSignLongitude())) ||
                    (ObjectUtil.isNotEmpty(signInfoVo.getSignLongitude()) && ObjectUtil.isEmpty(signInfoVo.getSignLatitude()))) {
                throw new BusinessException(MATCH_SIGN_IN_SITE_INCOMPLETE);
            }
            //非扫码，有签到时间就必须有签到地点
            if (signInfoVo.getSignType()!=1 && ((ObjectUtil.isNotEmpty(signInfoVo.getSignStartTime()) || ObjectUtil.isNotEmpty(signInfoVo.getSignEndTime()))
                    && (StringUtils.isEmpty(signInfoVo.getSignLatitude()) || StringUtils.isEmpty(signInfoVo.getSignLongitude()))
                    || StringUtils.isEmpty(signInfoVo.getSignSiteName()))) {
                throw new BusinessException(MATCH_SIGN_IN_SITE_INCOMPLETE);
            }
        }
        //校验签退信息
        if (signInfoVo.getInformationType()==-1){
            //校验签退时间是否完善
            if ((ObjectUtil.isNotEmpty(signInfoVo.getSignStartTime()) && ObjectUtil.isEmpty(signInfoVo.getSignEndTime()))
                    || (ObjectUtil.isNotEmpty(signInfoVo.getSignEndTime()) && ObjectUtil.isEmpty(signInfoVo.getSignStartTime()))) {
                throw new BusinessException(MATCH_SIGN_OUT_TIME_INCOMPLETE);
            }
            //有签退地点就必须有签退时间
            if ((ObjectUtil.isNotEmpty(signInfoVo.getSignLatitude()) || ObjectUtil.isNotEmpty(signInfoVo.getSignLongitude()))
                    && ObjectUtil.isEmpty(signInfoVo.getSignStartTime()) || ObjectUtil.isEmpty(signInfoVo.getSignEndTime())) {
                throw new BusinessException(MATCH_SIGN_OUT_TIME_INCOMPLETE);
            }
            //校验签退地点是否完善
            if ((ObjectUtil.isNotEmpty(signInfoVo.getSignLatitude()) && ObjectUtil.isEmpty(signInfoVo.getSignLongitude()))
                    || (ObjectUtil.isNotEmpty(signInfoVo.getSignLongitude()) && ObjectUtil.isEmpty(signInfoVo.getSignLatitude()))) {
                throw new BusinessException(MATCH_SIGN_OUT_SITE_INCOMPLETE);
            }
            //非扫码，有签退时间就必须有签退地点
            if (signInfoVo.getSignType()!=1 && ((ObjectUtil.isNotEmpty(signInfoVo.getSignStartTime()) || ObjectUtil.isNotEmpty(signInfoVo.getSignEndTime()))
                    && (StringUtils.isEmpty(signInfoVo.getSignLongitude()) || StringUtils.isEmpty(signInfoVo.getSignLatitude())||
                    StringUtils.isEmpty(signInfoVo.getSignSiteName())))) {
                throw new BusinessException(MATCH_SIGN_OUT_SITE_INCOMPLETE);
            }
        }
    }

    // 签到方法，根据距离进行签到提示
    public Result<LocationDataVO> checkIn(String lat1, String lon1, String lat2, String lon2) {
        try {
            double latitude1 = Double.parseDouble(lat1);
            double longitude1 = Double.parseDouble(lon1);
            double latitude2 = Double.parseDouble(lat2);
            double longitude2 = Double.parseDouble(lon2);

            double distance = calculateDistance(latitude1, longitude1, latitude2, longitude2);

            if (distance > 100) {
                return Result.ofFail("当前不在签到范围内！",new LocationDataVO().setDistance(Math.round(distance)));
            } else {
                return Result.ofSuccessWithMsg(new LocationDataVO().setDistance(Math.round(distance)),"到达目的地，是否签到！");
            }
        } catch (NumberFormatException e) {
            throw new BusinessException("输入格式错误，请输入正确的经度和纬度！");
        }
    }

    /**
     * 计算距离
     * @param lat1
     * @param lon1
     * @param lat2
     * @param lon2
     * @return
     */
    public double calculateDistance(double lat1, double lon1, double lat2, double lon2) {
        final int R = 6371; // 地球半径（单位：千米）

        double latDistance = Math.toRadians(lat2 - lat1);
        double lonDistance = Math.toRadians(lon2 - lon1);
        double a = Math.sin(latDistance / 2) * Math.sin(latDistance / 2)
                + Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2))
                * Math.sin(lonDistance / 2) * Math.sin(lonDistance / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        double distance = R * c * 1000; // 转换为米

        return distance;
    }
}
