package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SmsTemplateHandler;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.domain.dto.message.SendMessageTaskReqDTO;
import com.crunii.micro.domain.entity.main.YcqMessageTemplate;
import com.crunii.micro.domain.repo.main.YcqMessageTemplateRepo;
import com.crunii.micro.feign.client.MessageService;
import com.crunii.micro.service.match.api.bo.PushMsgTemplateBo;
import com.crunii.micro.service.match.constants.PushMsgConstants;
import com.crunii.micro.service.match.service.PushMsgService;
import java.util.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.common.TemplateParserContext;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.stereotype.Service;

/**
 * @author liuchuan
 * @version 1.0
 * @description:
 * @date 2023/11/30 11:09
 */
@Service
@Slf4j
@RequiredArgsConstructor

public class PushMsgServiceImpl implements PushMsgService {

    private final YcqMessageTemplateRepo ycqMessageTemplateRepo;

    private final MessageService messageService;


    /**
     * @param templateBo 模板相关信息
     * @param userId     推送人员id
     * @param map        模板中占位符替换参数
     * @description: 推送单人单调信息
     * @author liuchuan
     * @date: 2023/11/30 11:10
     */
    @Override
    public void pushMsg(PushMsgTemplateBo templateBo, Long userId, Map<String, Object> map) {

        YcqMessageTemplate msgTemplate = getMsgTemplate(templateBo);
        String content = parserExpression(msgTemplate.getMessageTemplate(), map);
        List<String> paramsList = SmsTemplateHandler.convertMapValuesToStringList(map);
        sendMsg(msgTemplate, content, userId.toString(),paramsList,templateBo.getPathAndJson(),templateBo.getExpandParam());

    }


    /**
     * @param templateBo    模板相关信息
     * @param userIdList    推送人员数组
     * @param paramsMapList 模板中占位符替换参数
     * @description: 推送给多人，没人占位符中替换信息不一致
     * @author liuchuan
     * @date: 2023/11/30 11:11
     */
    @Override
    public void pushMsgs(PushMsgTemplateBo templateBo, List<Long> userIdList, List<Map<String, Object>> paramsMapList) {
        if (userIdList.size()!=paramsMapList.size()){
            throw new BusinessException("消息条数与人员不匹配");
        }
        YcqMessageTemplate msgTemplate = getMsgTemplate(templateBo);
        for (int i=0 ;i<userIdList.size();i++){
            String content = parserExpression(msgTemplate.getMessageTemplate(), paramsMapList.get(i));
            List<String> paramsList = SmsTemplateHandler.convertMapValuesToStringList(paramsMapList.get(i));
            sendMsg(msgTemplate, content,userIdList.get(i).toString(),paramsList,templateBo.getPathAndJson(),templateBo.getExpandParam());
        }

    }

    /**
     * @param templateBo 模板相关信息
     * @param userIdList 推送人员id
     * @param paramsMap  模板中占位符替换参数（所有人的信息都一样）
     * @description: 推送多人相同的信息，
     * @author liuchuan
     * @date: 2023/11/30 11:11
     */
    @Override
    public void pushSameMsgs(PushMsgTemplateBo templateBo, List<Long> userIdList, Map<String, Object> paramsMap) {
        if (CollectionUtil.isEmpty(userIdList)) {
            throw new BusinessException("推送人员id为空!");
        }
        YcqMessageTemplate msgTemplate = getMsgTemplate(templateBo);
        String content = parserExpression(msgTemplate.getMessageTemplate(), paramsMap);
        String userIdStr = StreamUtils.join(userIdList, String::valueOf, StrUtil.COMMA);
        log.info("消息模板:{},发送内容{},发送人：{}",msgTemplate.getTemplateId(), content, userIdStr);
        List<String> paramsList = SmsTemplateHandler.convertMapValuesToStringList(paramsMap);
        sendMsg(msgTemplate, content, userIdStr,paramsList,templateBo.getPathAndJson(),templateBo.getExpandParam());
    }


    private void sendMsg(YcqMessageTemplate messageTemplate, String content, String userIds,List<String> paramsList,String pathAndJson,String expandParam) {
        try {
            SendMessageTaskReqDTO sendMessageTaskReqDTO = new SendMessageTaskReqDTO();
            // 范围数据(逗号拼接ID)
            sendMessageTaskReqDTO.setScopeData(userIds);
            // 消息发送内容
            sendMessageTaskReqDTO.setMessageContent(content);
            // 发送人id  默认系统发送
            sendMessageTaskReqDTO.setSendLoginId(1L);

            // 发送范围
            sendMessageTaskReqDTO.setScopeType(PushMsgConstants.SEND_SCOPE_TYPE);
            // 推送方式
            sendMessageTaskReqDTO.setSendWay(messageTemplate.getPushType());
            // 推送终端
            sendMessageTaskReqDTO.setMessageType(messageTemplate.getPushEndpoint());
            // 消息来源
            sendMessageTaskReqDTO.setDataSource(messageTemplate.getSystemName());
            // 消息分类
            sendMessageTaskReqDTO.setMessageClassify(messageTemplate.getSceneName());
            // 模板ID
            sendMessageTaskReqDTO.setTemplateId(messageTemplate.getTemplateId());
            // 设置路由+对应参数
            sendMessageTaskReqDTO.setPathAndJson(pathAndJson);
            //短信现在必穿模板和填充参数，使用json存储
            sendMessageTaskReqDTO.setFillData(SmsTemplateHandler.fillTemplateAndStoreInDatabase(messageTemplate.getMessageTemplate(),paramsList));

            sendMessageTaskReqDTO.setExpandParam(expandParam);
            Result result = messageService.saveMessageTask(sendMessageTaskReqDTO);
            log.info("消息发送{},result:{}",content,result);
        } catch (Exception e) {
            log.error("消息发送失败:{}", e);
            e.printStackTrace();
        }

    }


    private YcqMessageTemplate getMsgTemplate(PushMsgTemplateBo templateBo) {
        log.info("查询消息模板:{}", templateBo.toString());
        List<YcqMessageTemplate> templateList =new ArrayList<>();
        if (ObjectUtil.isEmpty(templateBo.getSystemName())){
             templateList = ycqMessageTemplateRepo
                    .findBySystemNameAndSceneNameAndTitleAndDelFlagOrderByCreateTimeDesc(PushMsgConstants.SYSTEM_NAME, templateBo.getSceneName(),
                            templateBo.getTitle(), (byte) 0);
        }else {
            templateList = ycqMessageTemplateRepo
                    .findBySystemNameAndSceneNameAndTitleAndDelFlagOrderByCreateTimeDesc(templateBo.getSystemName(), templateBo.getSceneName(),
                            templateBo.getTitle(), (byte) 0);
        }

        if (CollectionUtil.isNotEmpty(templateList)) {
            return templateList.get(0);
        } else {
            throw new BusinessException("没有找到消息模板");
        }

    }

    /**
     * @param
     * @description: 占位符替换
     * @return: 发送信息
     * @author liuchuan
     * @date: 2023/11/30 14:45
     */
    private String parserExpression(String template, Map<String, Object> paramsMap) {
        log.info("消息模：{}", template);
        ExpressionParser parser = new SpelExpressionParser();
        TemplateParserContext parserContext = new TemplateParserContext("${", "}");
        String content = parser.parseExpression(template, parserContext).getValue(paramsMap, String.class);
        log.info("替换后消息:{}", content);
        return content;
    }

    public static void main(String[] args) {

        String template = "您参与的${[competition]}作品还未上传，作品提交截止时间为${[deadline]}，请前往我的赛事尽快提交作品";
        HashMap<String,Object> paramsMap = new HashMap<String,Object>();
        paramsMap.put("competition","绘画大赛");
        paramsMap.put("deadline",new Date());
        log.info("消息模：{}", template);
        ExpressionParser parser = new SpelExpressionParser();
        TemplateParserContext parserContext = new TemplateParserContext("${", "}");
        String content = parser.parseExpression(template, parserContext).getValue(paramsMap, String.class);
        log.info("替换后消息:{}", content);
    }

}
