package com.crunii.micro.service.obs.core.client.s3;

import com.amazonaws.ClientConfiguration;
import com.amazonaws.Protocol;
import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.AWSStaticCredentialsProvider;
import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.client.builder.AwsClientBuilder;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3ClientBuilder;
import com.amazonaws.services.s3.model.*;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.obs.constants.ObsConstants;
import com.crunii.micro.service.obs.core.client.AbstractFileClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.multipart.MultipartFile;


import java.io.*;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.*;


/**
 * @author wangchen create 2023/9/5 16:44
 */
@Slf4j
public class S3FileClient extends AbstractFileClient<S3FileClientConfig> {

    private static final Integer READ_LIMIT = 50000000;

    private static final String USER_NAME = "gqtcqsw";

    private AmazonS3 amazonS3 ;

    public S3FileClient(String getObsName, S3FileClientConfig config) {
        super(getObsName, config);
    }

    @Override
    protected void doInit() {
        AWSCredentials awsCredentials = new BasicAWSCredentials(config.getAccessKey(),config.getAccessSecret());
        ClientConfiguration clientConfiguration = new ClientConfiguration();
        clientConfiguration.setProtocol(Protocol.HTTP);

        amazonS3 = AmazonS3ClientBuilder.standard()
                .withClientConfiguration(clientConfiguration)
                .withCredentials(new AWSStaticCredentialsProvider(awsCredentials))
                .withPathStyleAccessEnabled(true)
                .withEndpointConfiguration(new AwsClientBuilder.EndpointConfiguration(config.getEndpoint(),Regions.US_EAST_2.getName()))
                .build();
    }


    @Override
    public String uploadFile(String bucketName, InputStream inputStream, String fileKey) throws IOException {
        if(!headBucket(bucketName)){
            throw new BusinessException("不存在该桶+"+bucketName);
        }
        int available = inputStream.available();
        ObjectMetadata objectMetadata = new ObjectMetadata();
        objectMetadata.setContentLength(available);
        PutObjectRequest request = new PutObjectRequest(bucketName, fileKey, inputStream,objectMetadata);
        request.setCannedAcl(CannedAccessControlList.PublicRead);
        request.getRequestClientOptions().setReadLimit(READ_LIMIT);
        amazonS3.putObject(request);
        return getUrl(bucketName,fileKey);
    }

    @Override
    public String uploadMultipartFileByPart(String bucketName,MultipartFile file, String fileKey) {
        //声明线程池
        ExecutorService exec = new ThreadPoolExecutor(5, 5,
                0L, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<Runnable>());
        long size = file.getSize();
        int minPartSize = 100 * 1024 * 1024;
        // 得到总共的段数，和 分段后，每个段的开始上传的字节位置
        List<Long> positions = Collections.synchronizedList(new ArrayList<>());
        long filePosition = 0;
        while (filePosition < size) {
            positions.add(filePosition);
            filePosition += Math.min(minPartSize, (size - filePosition));
        }
        log.info("总大小：{}，分为{}段", size, positions.size());
        // 创建一个列表保存所有分传的 PartETag, 在分段完成后会用到
        List<PartETag> partETags = Collections.synchronizedList(new ArrayList<>());
        // 第一步，初始化，声明下面将有一个 Multipart Upload
        InitiateMultipartUploadRequest initRequest = new InitiateMultipartUploadRequest(bucketName, fileKey);
        initRequest.setCannedACL(CannedAccessControlList.PublicRead);
        initRequest.getRequestClientOptions().setReadLimit(READ_LIMIT);
        InitiateMultipartUploadResult initResponse = amazonS3.initiateMultipartUpload(initRequest);
        log.info("开始上传");
        long begin = System.currentTimeMillis();
        try {
            // MultipartFile 转 File
            File toFile = multipartFileToFile(file);
            for (int i = 0; i < positions.size(); i++) {
                int finalI = i;
                exec.execute(() -> {
                    long time1 = System.currentTimeMillis();
                    UploadPartRequest uploadRequest = new UploadPartRequest()
                            .withBucketName(bucketName)
                            .withKey(fileKey)
                            .withUploadId(initResponse.getUploadId())
                            .withPartNumber(finalI + 1)
                            .withFileOffset(positions.get(finalI))
                            .withFile(toFile)
                            .withPartSize(Math.min(minPartSize, (size - positions.get(finalI))));
                    // 第二步，上传分段，并把当前段的 PartETag 放到列表中
                    partETags.add(amazonS3.uploadPart(uploadRequest).getPartETag());
                    long time2 = System.currentTimeMillis();
                    log.info("第{}段上传耗时：{}", finalI + 1, (time2 - time1));
                });
            }
            //任务结束关闭线程池
            exec.shutdown();
            //判断线程池是否结束，不加会直接结束方法
            while (true) {
                if (exec.isTerminated()) {
                    break;
                }
            }

            // 第三步，完成上传，合并分段
            CompleteMultipartUploadRequest compRequest = new CompleteMultipartUploadRequest(bucketName, fileKey,
                    initResponse.getUploadId(), partETags);
            amazonS3.completeMultipartUpload(compRequest);
            //删除本地缓存文件
            toFile.delete();
        } catch (Exception e) {
            amazonS3.abortMultipartUpload(new AbortMultipartUploadRequest(bucketName, fileKey, initResponse.getUploadId()));
            log.error("Failed to upload, " + e.getMessage());
        }
        long end = System.currentTimeMillis();
        log.info("总上传耗时：{}", (end - begin));
        return getUrl(bucketName,fileKey);
    }

    /**
     * MultipartFile 转 File
     */
    public static File multipartFileToFile(MultipartFile file) throws Exception {
        File toFile = null;
        if (file.equals("") || file.getSize() <= 0) {
            file = null;
        } else {
            InputStream ins = null;
            ins = file.getInputStream();
            toFile = new File(file.getOriginalFilename());
            //获取流文件
            OutputStream os = new FileOutputStream(toFile);
            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead = ins.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            ins.close();
        }
        return toFile;
    }

    @Override
    public void delete(String bucket, String fileKey) {
        amazonS3.deleteObject(bucket,fileKey);
    }

    @Override
    public void createBucket(String bucket) {
        if(headBucket(bucket)){
            throw new BusinessException("桶已存在");
        }
        CreateBucketRequest req = new CreateBucketRequest(bucket);
        req.setCannedAcl(CannedAccessControlList.PublicRead);
        amazonS3.createBucket(req);
    }

    @Override
    public InputStream getObject(String bucketName, String fileKey) {
        if(!doesObjectExist(bucketName,fileKey)){
            throw new BusinessException("不存在该文件");
        }
        S3Object object = amazonS3.getObject(bucketName, fileKey);
        return object.getObjectContent();
    }

    @Override
    public boolean doesObjectExist(String bucketName, String fileKey) {
       return amazonS3.doesObjectExist(bucketName,fileKey);
    }

    @Override
    public void deleteBucket(String bucketName) {
        // 如桶不存在直接返回
        if(!headBucket(bucketName)){
           return;
        }
        amazonS3.deleteBucket(bucketName);
    }

    @Override
    public boolean headBucket(String bucketName) {
        return amazonS3.doesBucketExistV2(bucketName);
    }

    @Override
    public  String getUrl(String bucketName, String fileKey) {
        String env = System.getProperty("spring.profiles.active");
        if("h3cTest".equals(env)){
            return ObsConstants.S3Url +fileKey;
        }
        return ObsConstants.S3_TEST_Url +fileKey;
    }

}
