package com.crunii.micro.service.obs.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpException;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONObject;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqUploadVideo;
import com.crunii.micro.domain.repo.main.YcqUploadVideoRepo;
import com.crunii.micro.ds.uid.util.DateUtils;
import com.crunii.micro.service.obs.config.VideoConfigProperty;
import com.crunii.micro.service.obs.core.client.s3.S3FileClientConfig;
import com.crunii.micro.service.obs.dto.*;
import com.crunii.micro.service.obs.jdbc.VideoDao;
import com.crunii.micro.service.obs.service.VideoService;
import com.crunii.micro.service.obs.util.UploadFileUtil;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Map;

import static com.crunii.micro.service.obs.constants.ObsConstants.*;


/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/10/11 16:58
 */
@Service
@Slf4j
public class VideoServiceImpl implements VideoService {

    @Resource
    private VideoConfigProperty videoConfigProperty;

    @Resource
    private S3FileClientConfig s3FileClientConfig;

    @Resource
    private YcqUploadVideoRepo ycqUploadVideoRepo;

    @Resource
    private TsmPropertyCache tsmPropertyCache;

    @Resource
    private VideoDao videoDao;

    @Override
    public Result<UploadVideoVo> videoFileUpload(Worker worker, MultipartFile file,Integer serverType) {
        String code = isTypeExist(serverType);
        String fileName = file.getOriginalFilename();
        YcqUploadVideo video = new YcqUploadVideo();
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String objectName = videoConfigProperty.getSystemName() + "/" + code + "/" + date + "/originalVideo/";
        long size = file.getSize() / 1024;
        String uuidName;
        try {
            // 解析文件后缀
            int index = fileName.lastIndexOf(".");
            String suffix;
            if (index == -1 || (suffix = fileName.substring(index + 1)).isEmpty()) {
                return Result.ofFail("非视频文件，请重新上传");
            }
            //判断是否为视频文件(简单校验一下)
            if (!isVideoFile(suffix)) {
                return Result.ofFail("非视频文件，请重新上传");
            }
            // 获取文件信息
            uuidName = UUIDUtil.getUUID() + "." + suffix;
            String objectKey = objectName + uuidName;
            Map<String, Object> map = UploadFileUtil.signUrl(s3FileClientConfig, objectKey, 60 * 24 * 1);
            String actionUrl = map.get("fileUrl").toString();
            // 上传文件
            UploadFileUtil.uploadFile(actionUrl, file);
            // 保存数据库
            video.setServerType(serverType);
            video.setFileName(fileName);
            video.setFileKey(objectKey);
            video.setFileType(suffix.toLowerCase());
            video.setFileSize(String.valueOf(size));
            video.setCreateBy(worker.getLoginId());
            video.setFileDate(date);
            video.setCreateTime(new Date());
        } catch (Exception e) {
            log.info("上传文件失败，{}", e.getMessage());
            return Result.ofFail("文件上传失败");
        }
        // 上传完成后通知转码
        NoticeDto noticeDto = new NoticeDto();
        noticeDto.setFileName(uuidName);
        noticeDto.setUploadDate(date);
        noticeDto.setUserCode(code);
        UploadVideoVo vo = uploadFileNotification(worker, noticeDto);
        vo.setFileName(fileName);
        if (ObjectUtil.isEmpty(vo)) {
            throw new BusinessException("文件转码失败");
        }
        video.setFileId(vo.getFileId());
        // 替换成公网访问
        String pictureUrl = vo.getPictureUrl();
        String newPictureUrl = pictureUrl.replace(videoConfigProperty.getPictureUrlPrefix(), videoConfigProperty.getWebPreviewUrl());
        video.setPictureUrl(newPictureUrl);
        ycqUploadVideoRepo.saveAndFlush(video);
        vo.setPictureUrl(newPictureUrl);
        return Result.ofSuccess(vo);
    }

    @NotNull
    private String isTypeExist(Integer serverType) {
        String code = tsmPropertyCache.get("ycq-video-type", serverType + "");
        if(!StringUtils.hasText(code)){
            throw new BusinessException("传入编码无法在ycq-video-type静态值找到");
        }
        return code;
    }

    @Override
    public UploadVideoVo uploadFileNotification(Worker worker, NoticeDto noticeDto) {
        UploadVideoVo vo =new UploadVideoVo();
        String url = videoConfigProperty.getUploadFileNotificationUrl();
        NotificationDto dto = new NotificationDto();
        dto.setAppKey(videoConfigProperty.getAppKey());
        dto.setNonce(getNonce());
        dto.setCurTime(System.currentTimeMillis() + "");
        dto.setCheckNum(getCheckNum(dto.getNonce(), dto.getCurTime()));
        dto.setSystemName(videoConfigProperty.getSystemName());
        dto.setUserName(noticeDto.getUserCode());
        dto.setUploadDate(noticeDto.getUploadDate());
        dto.setFileName(noticeDto.getFileName());
        dto.setNotifyURL(videoConfigProperty.getNotifyUrl());
        String json = JsonUtil.getJsonString(dto);
        String result;
        try {
            result = HttpRequest.post(url)
                    .body(json)
                    .execute().body();
        } catch (HttpException e) {
            log.info("----------->{}", e.getMessage());
            throw new BusinessException("出错了，请联系管理员");
        }
        JSONObject jsonObject = new JSONObject(result);
        log.info("------------{}", jsonObject);
        String msg = (String) jsonObject.get("msg");
        if (!"OK".equals(msg)) {
            log.info("请求参数{}", json);
            throw new BusinessException(jsonObject.get("msg").toString());
        }
        vo.setFileId((String) jsonObject.get("fileID"));
        vo.setPictureUrl((String) jsonObject.get("pictureUrl"));
        return vo;
    }

    @Override
    public Result<String> getFileUrl(Worker worker, FileDto fileDto) {
        String typeExist = isTypeExist(fileDto.getServerType());
        YcqUploadVideo video = ycqUploadVideoRepo.findByFileIdAndServerTypeAndDelFlag(fileDto.getFileId(), fileDto.getServerType(),0);
        if (ObjectUtil.isEmpty(video)) {
            return Result.ofFail(FILE_STATUS_MGS_ONE);
        }
        // 获取视频转化状态
        if (video.getFileStatus() == STATUS_INVALID) {
            return Result.ofFail(FILE_STATUS_MGS_TWO);
        }
        String url = videoConfigProperty.getGetFileUrl();
        GetFileDto dto = new GetFileDto();
        dto.setAppKey(videoConfigProperty.getAppKey());
        dto.setNonce(getNonce());
        dto.setCurTime(System.currentTimeMillis() + "");
        dto.setCheckNum(getCheckNum(dto.getNonce(), dto.getCurTime()));
        dto.setSystemName(videoConfigProperty.getSystemName());
        dto.setUserName(typeExist);
        dto.setUploadDate(video.getFileDate());
        dto.setFileID(video.getFileId());
        dto.setFileType(fileDto.getFileType());
        String json = JsonUtil.getJsonString(dto);
        String result;
        try {
            result = HttpRequest.post(url)
                    .body(json)
                    .execute().body();
        } catch (HttpException e) {
            throw new BusinessException("出错了，请联系管理员");
        }
        JSONObject jsonObject = new JSONObject(result);
        String msg = (String) jsonObject.get("msg");
        if (!"OK".equals(msg)) {
            log.info("请求参数{},错误信息{}", json, jsonObject.get("msg").toString());
            throw new BusinessException("获取失败，请联系管理员");
        }
        // 替换成公网访问
        String oldFileUrl = (String) jsonObject.get("fileUrl");
        String fileUrl = oldFileUrl.replace(videoConfigProperty.getPictureUrlPrefix(), videoConfigProperty.getWebPreviewUrl());
        return Result.ofSuccess(fileUrl);
    }

    @Override
    public Result<VideoPlatVo> queryRealTimeVideoAddress(Worker worker, VideoReqDto req) {
        String typeExist = isTypeExist(req.getServerType());
        YcqUploadVideo video = ycqUploadVideoRepo.findByFileIdAndServerTypeAndDelFlag(req.getFileId(),req.getServerType() ,0);
        // 根据fileNumber、tableName获取信息表记录 如没有fileId 返回null
        if(!StringUtils.hasText(req.getFileId())){
            return new Result<>();
        }
        if (ObjectUtil.isEmpty(video)) {
            return Result.ofFail(FILE_STATUS_MGS_ONE);
        }
        // 获取视频转化状态
        if (video.getFileStatus() == STATUS_INVALID) {
            return Result.ofFail(FILE_STATUS_MGS_TWO);
        }
        String url = videoConfigProperty.getGetFileUrl();
        GetFileDto dto = new GetFileDto();
        dto.setAppKey(videoConfigProperty.getAppKey());
        dto.setNonce(getNonce());
        dto.setCurTime(System.currentTimeMillis() + "");
        dto.setCheckNum(getCheckNum(dto.getNonce(), dto.getCurTime()));
        dto.setSystemName(videoConfigProperty.getSystemName());
        dto.setUserName(typeExist);
        dto.setUploadDate(video.getFileDate());
        dto.setFileID(req.getFileId());
        dto.setFileType(VIDEO_1080);
        String json = JsonUtil.getJsonString(dto);
        String result;
        try {
            result = HttpRequest.post(url)
                    .body(json)
                    .execute().body();
        } catch (HttpException e) {
            throw new BusinessException("出错了，请联系管理员");
        }
        JSONObject jsonObject = new JSONObject(result);
        String msg = (String) jsonObject.get("msg");
        if (!"OK".equals(msg)) {
            log.info("请求参数{}", jsonObject.get("msg").toString());
            Result.ofFail("获取视频失败");
        }
        // 替换成公网访问
        String oldFileUrl = (String) jsonObject.get("fileUrl");
        String fileUrl = oldFileUrl.replace(videoConfigProperty.getPictureUrlPrefix(), videoConfigProperty.getWebPreviewUrl());
        VideoPlatVo videoPlatVo = new VideoPlatVo();
        videoPlatVo.setVideoPlayUrl(fileUrl);
        videoPlatVo.setFileName(video.getFileName());
        return Result.ofSuccess(videoPlatVo);
    }

    @Override
    public Result<String> updateFileStatus(VideoDto videoDto) {
        Result<String> result = new Result<>();
        videoDao.updateFileStatus(videoDto);
        result.setCode("0");
        result.setMsg("OK");
        return result;
    }

    @Override
    public Result<VideoPlatVo> downloadVideo(VideoReqDto dto) {
        String typeExist = isTypeExist(dto.getServerType());
        YcqUploadVideo video = ycqUploadVideoRepo.findByFileIdAndServerTypeAndDelFlag(dto.getFileId(), dto.getServerType(),0);
        if (ObjectUtil.isEmpty(video)) {
            return Result.ofFail("视频不存在");
        }
        Map<String, Object> returnMap = UploadFileUtil.downloadUrl(s3FileClientConfig, video.getFileKey(), 60 * 24 * 7);
        String fileUrl = (String) returnMap.get("fileUrl");
        if (StrUtil.isEmpty(fileUrl)) {
            return Result.ofFail("获取视频失败");
        }
        VideoPlatVo videoPlatVo = new VideoPlatVo();
        videoPlatVo.setVideoPlayUrl(fileUrl);
        videoPlatVo.setFileName(video.getFileName());
        return Result.ofSuccess(videoPlatVo);
    }

    private String getCheckNum(String nonce, String curTime) {
        return SecureUtil.sha1(nonce + videoConfigProperty.getSecretKey() + curTime);
    }

    public String getNonce() {
        int randomNumber = (int) (Math.random() * 1000000);
        return String.format("%06d", randomNumber);
    }

    public boolean isVideoFile(String suffix) {
        String extension = suffix.toLowerCase();
        return "mp4".equalsIgnoreCase(extension)
                || "avi".equalsIgnoreCase(extension)
                || "mov".equalsIgnoreCase(extension)
                || "wmv".equalsIgnoreCase(extension)
                || "flv".equalsIgnoreCase(extension)
                || "webm".equals(extension);
    }
}
