package com.crunii.micro.service.oneaffair.controller;

import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.feign.client.dto.OrgNode;
import com.crunii.micro.service.oneaffair.annotation.Desensitization;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.*;
import com.crunii.micro.service.oneaffair.manager.TeacherManager;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Slf4j
@Api(tags = "教师API接口")
@RestController
@RequestMapping("/yjs/teacher/auth")
public class TeacherController {

    @Resource
    private TeacherManager teacherManager;

    @ApiOperation(value = "分页查询教师信息")
    @RequestMapping(path = "/pageSearchTeacherInfo", method = RequestMethod.POST)
    @Desensitization
    @ResponseBody
    public Result<PageRsp<TeacherInfoVo>> pageSearchTeacherInfo(@RequestBody PageReq<TeacherReq> req) {
        log.info("TeacherController.pageSearchTeacherInfo.param：{}", req.toString());
        try {
            return Result.ofSuccess(teacherManager.pageSearchTeacherInfo(req));
        } catch (BusinessException e) {
            log.error("TeacherController.pageSearchTeacherInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherController.pageSearchTeacherInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新建教师信息")
    @RequestMapping(path = "/encryptionRequired/createTeacherInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createTeacherInfo(@Validated @EncryptRequestBody TeacherInfoCreateReq createReq) {
        log.info("TeacherController.createTeacherInfo.param：{}", createReq.toString());
        try {
            teacherManager.createTeacherInfo(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("TeacherController.createTeacherInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherController.createTeacherInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改教师信息")
    @RequestMapping(path = "/encryptionRequired/updateTeacherInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateTeacherInfo(@Validated @EncryptRequestBody TeacherInfoUpdateReq updateReq) {
        log.info("TeacherController.updateTeacherInfo.param：{}", updateReq.toString());
        try {
            teacherManager.updateStudentInfo(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("TeacherController.updateTeacherInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherController.updateTeacherInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除教师信息")
    @RequestMapping(path = "/deleteTeacherInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> deleteTeacherInfo(@Validated @RequestBody TeacherInfoDeleteReq deleteReq) {
        log.info("TeacherController.deleteTeacherInfo.param：{}", deleteReq.toString());
        try {
            teacherManager.deleteTeacherInfo(deleteReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("TeacherController.deleteTeacherInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherController.deleteTeacherInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "通过父组织Id获取子组织(数据权限过滤)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentOrgId", value = "父组织id", required = true)
    })
    @RequestMapping(path = "/getOrgChildrenByParentOrgId", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<OrgNode>> getOrgChildrenByParentOrgId(@RequestParam("parentOrgId") Long parentOrgId) {
        log.info("TeacherController.getOrgChildrenByParentOrgId.param：{}", parentOrgId);
        try {
            return Result.ofSuccess(teacherManager.getOrgChildrenByParentOrgId(parentOrgId));
        } catch (BusinessException e) {
            log.error("TeacherController.getOrgChildrenByParentOrgId.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherController.getOrgChildrenByParentOrgId.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "Excel模板导入批量新增教师信息")
    @RequestMapping(path = "/importTeacherInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> importTeacherInfo(HttpServletResponse response, @ApiParam(value = "导入文件", required = true) @RequestParam(value = "file", required = true) MultipartFile file) {
        log.info("TeacherController.importTeacherInfo.param：{}", file.getName());
        try {
            Boolean flag = teacherManager.importTeacherInfo(response, file);
            if (flag) {
                return Result.ofSuccess(flag);
            } else {
                return Result.ofFail("Excel模板导入批量新增教师信息失败");
            }
        } catch (BusinessException e) {
            log.error("TeacherController.importTeacherInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherController.importTeacherInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


}
