package com.crunii.micro.service.oneaffair.dto.excel.listener;

import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.excel.util.ListUtils;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.excel.StudentInfoThreeImportExcel;
import com.crunii.micro.service.oneaffair.dto.excel.StudentInfoTwoImportExcel;
import com.crunii.micro.service.oneaffair.enums.*;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import dm.jdbc.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

@Slf4j
public class StudentThreeImportExcelListener implements ReadListener<StudentInfoThreeImportExcel> {

    private String msg;

    public String getMsg() {
        return msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * 每隔100条存储数据库，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 100;

    private static final int MAX_COUNT = 500;

    private static final String REGEX = "[1][3456789][0-9]{9}";

    /**
     * 缓存的数据,List<AttdnOver>
     */
    private List<StudentInfoThreeImportExcel> cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
    private List<StudentInfoThreeImportExcel> excelData = ListUtils.newArrayList();

    public List<StudentInfoThreeImportExcel> getExcelData() {
        return excelData;
    }

    public StudentThreeImportExcelListener() {

    }

    @Override
    public void onException(Exception exception, AnalysisContext context) {
        log.error("学生数据上传文件解析失败，{}", exception.getMessage());
    }

    @Override
    public void invoke(StudentInfoThreeImportExcel excelData, AnalysisContext context) {
        log.info("学生数据上传文件解析到一条数据:{}", excelData);
        StringBuilder errorMessage = new StringBuilder();
        // 学生姓名 32位
        String studentName = excelData.getStudentName();
        if (StringUtil.isEmpty(studentName)) {
            errorMessage.append("【学生姓名】必填 ");
        } else {
            if (studentName.length() > 32) {
                errorMessage.append("【学生姓名】长度不能超过32位 ");
            }
            // 去空格
            excelData.setStudentName(studentName.replace(" ", ""));
        }
        // 学生性别：男/女 SexTypeEnum
        String sexStr = excelData.getSexStr();
        if (StringUtil.isEmpty(sexStr)) {
            errorMessage.append("【学生性别】必选 ");
        } else {
            SexTypeEnum sexTypeEnum = SexTypeEnum.getEnumByName(sexStr.trim());
            if (sexTypeEnum == null) {
                errorMessage.append("【学生性别】的值必须是下拉框选中的值 ");
            } else {
                excelData.setSexStr(sexTypeEnum.desc());
                excelData.setSex(sexTypeEnum.code());
            }
        }
        // 身份证号验证
        String identityCardNo = excelData.getIdentityCardNo();
        if (StringUtil.isEmpty(identityCardNo)) {
            errorMessage.append("【身份证号】必填 ");
        } else {
            identityCardNo = identityCardNo.replace(" ", "");
            // 身份证号的格式：身份证号码是由18位数字和最后一位可能是字母X组成的，其中前17位为地区、出生年月日和顺序号，最后一位为校验码
            if (!CommonUtil.isIDCard(identityCardNo)) {
                errorMessage.append("【身份证号】格式不正确 ");
            }
            excelData.setIdentityCardNo(identityCardNo);
        }
        // 手机号验证
        String mobile = excelData.getMobile();
        if (!StringUtil.isEmpty(mobile)) {
            mobile = mobile.replace(" ", "");
            // 手机号码的格式：第一位只能为1，第二位可以是3，4，5，7，8，第三位到第十一位可以为0-9中任意一个数字
            if (!mobile.matches(REGEX)) {
                errorMessage.append("【手机号】格式不正确 ");
            }
            excelData.setMobile(mobile);
        }
        // 政治面貌：群众、少先队员、共青团员、预备党员、共产党员 PoliticalStatusTypeEnum
        String politicalStatus = excelData.getPoliticalStatus();
        if (StringUtil.isEmpty(politicalStatus)) {
            errorMessage.append("【政治面貌】必选 ");
        } else {
            politicalStatus = politicalStatus.replace(" ", "");
            PoliticalStatusTypeEnum politicalStatusTypeEnum = PoliticalStatusTypeEnum.getEnumByName(politicalStatus);
            if (politicalStatusTypeEnum == null || !politicalStatusTypeEnum.code().equals(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code())) {
                errorMessage.append("【政治面貌】的值默认是共青团员，不可修改 ");
            } else {
                excelData.setPoliticalStatus(politicalStatusTypeEnum.desc());
                excelData.setPoliticalStatusType(politicalStatusTypeEnum.code());
            }
        }
        // 学校名称
        String schoolName = excelData.getCurrentSchoolName();
        if (StringUtil.isEmpty(schoolName)) {
            errorMessage.append("【学校名称】必填 ");
        } else {
            excelData.setCurrentSchoolName(schoolName.replace(" ", ""));
        }
        // 年级
        String gradeName = excelData.getCurrentGradeName();
        if (StringUtil.isEmpty(gradeName)) {
            errorMessage.append("【年级】必填 ");
        } else {
            excelData.setCurrentGradeName(gradeName.replace(" ", ""));
        }
        // 班级
        String className = excelData.getCurrentClassName();
        if (StringUtil.isEmpty(className)) {
            errorMessage.append("【班级】必填 ");
        } else {
            excelData.setCurrentClassName(className.replace(" ", ""));
        }
        // 学期：1-上学期；2-下学期
        String semesterName = excelData.getSemesterName();
        if (StringUtil.isEmpty(semesterName)) {
            errorMessage.append("【学期】必选 ");
        } else {
            semesterName = semesterName.replace(" ", "");
            SemesterEnum semesterEnum = SemesterEnum.getEnumByName(semesterName);
            if (semesterEnum == null) {
                errorMessage.append("【学期】的值必须是下拉框选中的值 ");
            } else {
                excelData.setSemesterName(semesterEnum.desc());
                excelData.setSemesterType(semesterEnum.code());
            }
        }
        // 班主任姓名
        String classDirectorName = excelData.getCurrentClassDirectorName();
        if (StringUtil.isEmpty(classDirectorName)) {
            errorMessage.append("【班主任姓名】必填 ");
        } else {
            excelData.setCurrentClassDirectorName(classDirectorName.replace(" ", ""));
        }
        // 辅导老师姓名
        String tutorTeacherName = excelData.getTutorTeacherName();
        if (StringUtil.isEmpty(tutorTeacherName)) {
            errorMessage.append("【辅导老师姓名】必填 ");
        } else {
            excelData.setTutorTeacherName(tutorTeacherName.replace(" ", ""));
        }
        // 入队时间
        String joinYoungPioneersTimeStr = excelData.getJoinYoungPioneersTimeStr();
        excelData.setIfJoinYouthPioneers(IfFlagEnum.TRUE.code());
        if(StringUtil.isEmpty(joinYoungPioneersTimeStr)){
            errorMessage.append("【入队时间】必填 ");
        }else{
            try {
                Date joinYoungPioneersTime = DateUtil.parse(joinYoungPioneersTimeStr);
                excelData.setJoinYoungPioneersTime(joinYoungPioneersTime);
            } catch (Exception e) {
                errorMessage.append("【入队时间】格式不正确，正确格式为YYYY-MM-dd ");
            }
        }
        // 入团时间
        String joinYouthLeagueTimeStr = excelData.getJoinYouthLeagueTimeStr();
        excelData.setIfJoinYouthLeague(IfFlagEnum.TRUE.code());
        if(StringUtil.isEmpty(joinYouthLeagueTimeStr)){
            errorMessage.append("【入团时间】必填 ");
        }else{
            try {
                Date joinYouthLeagueTime = DateUtil.parse(joinYouthLeagueTimeStr);
                excelData.setJoinYouthLeagueTime(joinYouthLeagueTime);
            } catch (Exception e) {
                errorMessage.append("【入团时间】格式不正确，正确格式为YYYY-MM-dd ");
            }
        }

        excelData.setCompleteStatus(TaskCompleteStatusEnum.UN_COMPLETE.code());
        // 是否检验通过
        if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
            excelData.setVerifyStatus(IfFlagEnum.FALSE.code());
            excelData.setErrorMessage(errorMessage.toString());
        } else {
            excelData.setVerifyStatus(IfFlagEnum.TRUE.code());
        }
        cachedDataList.add(excelData);
        if (cachedDataList.size() >= BATCH_COUNT) {
            saveData();
        }
    }

    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        log.info("学生数据上传文件所有数据解析完成！");
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
    }

    /**
     * 存储数据
     */
    private void saveData() {
        if (!CollectionUtils.isEmpty(cachedDataList)) {
            this.excelData.addAll(cachedDataList);
            // 存储完成清理 list
            cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
        }
        if (excelData.size() > MAX_COUNT) {
            throw new BusinessException("学生数据上传文件最大上传" + MAX_COUNT + "条数据");
        }
    }

}
