package com.crunii.micro.service.oneaffair.dto.excel.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.excel.util.ListUtils;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.excel.TeacherInfoImportExcel;
import com.crunii.micro.service.oneaffair.enums.*;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import dm.jdbc.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

import java.util.List;

@Slf4j
public class TeacherImportExcelListener implements ReadListener<TeacherInfoImportExcel> {

    private String msg;

    public String getMsg() {
        return msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * 每隔100条存储数据库，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 100;

    private static final int MAX_COUNT = 500;

    private static final String REGEX = "[1][3456789][0-9]{9}";

    /**
     * 缓存的数据,List<AttdnOver>
     */
    private List<TeacherInfoImportExcel> cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
    private List<TeacherInfoImportExcel> excelData = ListUtils.newArrayList();

    public List<TeacherInfoImportExcel> getExcelData() {
        return excelData;
    }

    public TeacherImportExcelListener() {

    }

    @Override
    public void onException(Exception exception, AnalysisContext context) {
        log.error("教师数据上传文件解析失败，{}", exception.getMessage());
    }

    @Override
    public void invoke(TeacherInfoImportExcel excelData, AnalysisContext context) {
        log.info("教师数据上传文件解析到一条数据:{}", excelData);
        StringBuilder errorMessage = new StringBuilder();
        // 教师姓名 32位
        String teacherName = excelData.getTeacherName();
        if (StringUtil.isEmpty(teacherName)) {
            errorMessage.append("【教师姓名】必填 ");
        } else {
            if (teacherName.length() > 32) {
                errorMessage.append("【教师姓名】长度不能超过32位 ");
            }
            // 去空格
            excelData.setTeacherName(teacherName.replace(" ", ""));
        }
        // 教师工号：32位
        String loginCode = excelData.getLoginCode();
        if (!StringUtil.isEmpty(loginCode)) {
            if (loginCode.length() > 32) {
                errorMessage.append("【教师工号】长度不能超过32位 ");
            } else {
                excelData.setLoginCode(loginCode);
            }
        }
        // 教师性别：男/女 SexTypeEnum
        String sexStr = excelData.getSexStr();
        if (StringUtil.isEmpty(sexStr)) {
            errorMessage.append("【教师性别】必选 ");
        } else {
            SexTypeEnum sexTypeEnum = SexTypeEnum.getEnumByName(sexStr.trim());
            if (sexTypeEnum == null) {
                errorMessage.append("【教师性别】的值必须是下拉框选中的值 ");
            } else {
                excelData.setSexStr(sexTypeEnum.desc());
                excelData.setSex(sexTypeEnum.code());
            }
        }
        // 手机号验证
        String mobile = excelData.getMobile();
        if (StringUtil.isEmpty(mobile)) {
            errorMessage.append("【手机号】必填 ");
        } else {
            mobile = mobile.replace(" ", "");
            // 手机号码的格式：第一位只能为1，第二位可以是3，4，5，7，8，第三位到第十一位可以为0-9中任意一个数字
            if (!mobile.matches(REGEX)) {
                errorMessage.append("【手机号】格式不正确 ");
            }
            excelData.setMobile(mobile);
        }
        // 身份证号验证
        String identityCardNo = excelData.getIdentityCardNo();
        if (StringUtil.isEmpty(identityCardNo)) {
            errorMessage.append("【身份证号】必填 ");
        } else {
            identityCardNo = identityCardNo.replace(" ", "");
            // 身份证号的格式：身份证号码是由18位数字和最后一位可能是字母X组成的，其中前17位为地区、出生年月日和顺序号，最后一位为校验码
            if(!CommonUtil.isIDCard(identityCardNo)){
                errorMessage.append("【身份证号】格式不正确 ");
            }
            excelData.setIdentityCardNo(identityCardNo);
        }
        // 教师职务：三级教师、二级教师、一级教师、高级教师、正高级教师 TeacherPositionTypeEnum
        String teacherPosition = excelData.getTeacherPosition();
        if (StringUtil.isEmpty(teacherPosition)) {
            errorMessage.append("【教师职务】必选 ");
        } else {
            teacherPosition = teacherPosition.replace(" ", "");
            TeacherPositionTypeEnum teacherPositionTypeEnum = TeacherPositionTypeEnum.getEnumByName(teacherPosition);
            if (teacherPositionTypeEnum == null) {
                errorMessage.append("【教师职务】的值必须是下拉框选中的值 ");
            } else {
                excelData.setTeacherPosition(teacherPositionTypeEnum.desc());
                excelData.setTeacherPositionType(teacherPositionTypeEnum.code());
            }
        }
        // 少先队职务：中队辅导员、大队辅导员、区县总辅导员、校外辅导员、市级总辅导员 YoungPioneerPositionTypeEnum
        String youngPioneerPosition = excelData.getYoungPioneerPosition();
        if (StringUtil.isEmpty(youngPioneerPosition)) {
            errorMessage.append("【少先队职务】必选 ");
        } else {
            youngPioneerPosition = youngPioneerPosition.replace(" ", "");
            YoungPioneerPositionTypeEnum youngPioneerPositionTypeEnum = YoungPioneerPositionTypeEnum.getEnumByName(youngPioneerPosition);
            if (youngPioneerPositionTypeEnum == null) {
                errorMessage.append("【少先队职务】的值必须是下拉框选中的值 ");
            } else {
                excelData.setYoungPioneerPosition(youngPioneerPositionTypeEnum.desc());
                excelData.setYoungPioneerPositionType(youngPioneerPositionTypeEnum.code());
            }
        }
        // 政治面貌：群众、少先队员、共青团员、预备党员、共产党员 PoliticalStatusTypeEnum
        String politicalStatus = excelData.getPoliticalStatus();
        if (StringUtil.isEmpty(politicalStatus)) {
            errorMessage.append("【政治面貌】必选 ");
        } else {
            politicalStatus = politicalStatus.replace(" ", "");
            PoliticalStatusTypeEnum politicalStatusTypeEnum = PoliticalStatusTypeEnum.getEnumByName(politicalStatus);
            if (politicalStatusTypeEnum == null) {
                errorMessage.append("【政治面貌】的值必须是下拉框选中的值 ");
            } else {
                excelData.setPoliticalStatus(politicalStatusTypeEnum.desc());
                excelData.setPoliticalStatusType(politicalStatusTypeEnum.code());
            }
        }
        // 所属学校
        String schoolName = excelData.getSchoolName();
        if (StringUtil.isEmpty(schoolName)) {
            errorMessage.append("【所属学校】必填 ");
        } else {
            excelData.setSchoolName(schoolName.replace(" ", ""));
        }
        // 负责年级/班级
        String gradeClassStr = excelData.getGradeClassStr();
        if (StringUtil.isEmpty(gradeClassStr)) {
            errorMessage.append("【负责年级/班级】必填 ");
        } else {
            gradeClassStr = gradeClassStr.replace(" ", "").replace("：", ":").replace("，", ",");
            excelData.setGradeClassStr(gradeClassStr);
        }
        excelData.setCompleteStatus(TaskCompleteStatusEnum.UN_COMPLETE.code());
        // 是否检验通过
        if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
            excelData.setVerifyStatus(IfFlagEnum.FALSE.code());
            excelData.setErrorMessage(errorMessage.toString());
        } else {
            excelData.setVerifyStatus(IfFlagEnum.TRUE.code());
        }
        cachedDataList.add(excelData);
        if (cachedDataList.size() >= BATCH_COUNT) {
            saveData();
        }
    }

    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        log.info("教师数据上传文件所有数据解析完成！");
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
    }

    /**
     * 存储数据
     */
    private void saveData() {
        if (!CollectionUtils.isEmpty(cachedDataList)) {
            this.excelData.addAll(cachedDataList);
            // 存储完成清理 list
            cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
        }
        if (excelData.size() > MAX_COUNT) {
            throw new BusinessException("教师数据上传文件最大上传" + MAX_COUNT + "条数据");
        }
    }

}
