package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsImportRecord;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordReq;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 批量导入记录表(YCQ_YJS_IMPORT_RECORD)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsImportRecordJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select id,file_name,total_size,success_size,error_size,complete_status,complete_time,operate_type,del_flag," +
            "created_time,created_id,created_name,last_updated_time,last_updated_name,last_updated_id from YCQ_YJS_IMPORT_RECORD where del_flag = 0 ";

    private static final String COUNT_SQL = "select count(1) from YCQ_YJS_IMPORT_RECORD where del_flag = 0 ";

    /**
     * 列表查询
     *
     * @param req
     * @return
     */
    public List<YcqYjsImportRecord> selectList(ImportRecordReq req) {
        log.info("YcqYjsImportRecordJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 文件名称-精准匹配
            if (StringUtils.isNotBlank(req.getFileName())) {
                sql.append(" and file_name = ? ");
                param.add(req.getFileName());
            }
            // 文件名称-模糊匹配
            if (StringUtils.isNotBlank(req.getFileNameLike())) {
                sql.append(" and file_name like ? ");
                param.add("%" + req.getFileNameLike() + "%");
            }
            // 完成状态：0-未开始；1-进行中；2-已完成
            if (req.getCompleteStatus() != null) {
                sql.append(" and complete_status = ? ");
                param.add(req.getCompleteStatus());
            }
            // 操作类型：1-教师导入；2-学生导入
            if (req.getOperateType() != null) {
                sql.append(" and operate_type = ? ");
                param.add(req.getOperateType());
            }
            // 创建人ID
            if (req.getCreatedId() != null) {
                sql.append(" and created_id = ? ");
                param.add(req.getCreatedId());
            }
            // 创建时间
            if (req.getStartCreatedTime() != null) {
                sql.append(" and created_time >= ? ");
                param.add(req.getStartCreatedTime());
            }
            if (req.getEndCreatedTime() != null) {
                sql.append(" and  ?  >= created_time ");
                param.add(req.getEndCreatedTime());
            }
        }
        sql.append(" order by created_time");
        return jt.queryForBeans(YcqYjsImportRecord.class, sql.toString(), param.toArray());
    }

    /**
     * 分页查询导入记录数据
     *
     * @param req
     * @return
     */
    public PageRsp<ImportRecordVo> pageSearch(PageReq<ImportRecordReq> req) {
        log.info("YcqYjsImportRecordJdbcDao.pageSearch.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        StringBuilder countSql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();

        if (req.getQuery() != null) {
            // 文件名称-精准匹配
            if (StringUtils.isNotBlank(req.getQuery().getFileName())) {
                sql.append(" and file_name = ? ");
                countSql.append(" and file_name = ? ");
                param.add(req.getQuery().getFileName());
            }
            // 文件名称-模糊匹配
            if (StringUtils.isNotBlank(req.getQuery().getFileNameLike())) {
                sql.append(" and file_name like ? ");
                countSql.append(" and file_name like ? ");
                param.add("%" + req.getQuery().getFileNameLike() + "%");
            }
            // 完成状态：0-未开始；1-进行中；2-已完成
            if (req.getQuery().getCompleteStatus() != null) {
                sql.append(" and complete_status = ? ");
                countSql.append(" and complete_status = ? ");
                param.add(req.getQuery().getCompleteStatus());
            }
            // 操作类型：1-教师导入；2-学生导入
            if (req.getQuery().getOperateType() != null) {
                sql.append(" and operate_type = ? ");
                countSql.append(" and operate_type = ? ");
                param.add(req.getQuery().getOperateType());
            }
            // 创建人ID
            if (req.getQuery().getCreatedId() != null) {
                sql.append(" and created_id = ? ");
                countSql.append(" and created_id = ? ");
                param.add(req.getQuery().getCreatedId());
            }
            // 创建时间
            if (req.getQuery().getStartCreatedTime() != null) {
                sql.append(" and created_time >= ? ");
                countSql.append(" and created_time >= ? ");
                param.add(req.getQuery().getStartCreatedTime());
            }
            if (req.getQuery().getEndCreatedTime() != null) {
                sql.append(" and  ?  >= created_time ");
                countSql.append(" and  ?  >= created_time ");
                param.add(req.getQuery().getEndCreatedTime());
            }

        }
        sql.append(" order by created_time desc");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<ImportRecordVo> result;
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(ImportRecordVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<ImportRecordVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

}
