package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentMedal;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 学生奖章表(YCQ_YJS_STUDENT_MEDAL)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentMedalJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select s.* from YCQ_YJS_STUDENT_MEDAL s " +
            " inner join YCQ_YJS_MEDAL m on s.medal_id = m.id and m.del_flag = 0 where s.del_flag = 0 ";

    private static final String COUNT_SQL = "select count(s.id) from YCQ_YJS_STUDENT_MEDAL s" +
            " inner join YCQ_YJS_MEDAL m on s.medal_id = m.id and m.del_flag = 0 where s.del_flag = 0 ";

    /**
     * 列表查询学生奖章数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentMedal> selectList(StudentMedalReq req) {
        log.info("YcqYjsStudentMedalJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and s.student_id = ? ");
                param.add(req.getStudentId());
            }
            // 奖章ID
            if (req.getMedalId() != null) {
                sql.append(" and s.medal_id = ? ");
                param.add(req.getMedalId());
            }
            // 是否点亮：0-否；1-是
            if (req.getIfLight() != null) {
                sql.append(" and s.if_light = ? ");
                param.add(req.getIfLight());
            }
        }
        sql.append(" order by s.medal_id ");
        return jt.queryForBeans(YcqYjsStudentMedal.class, sql.toString(), param.toArray());
    }

    /**
     * 统计学生奖章数量
     *
     * @param req
     * @return
     */
    public Integer count(StudentMedalReq req) {
        log.info("YcqYjsStudentMedalJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and s.student_id = ? ");
                param.add(req.getStudentId());
            }
            // 奖章ID
            if (req.getMedalId() != null) {
                sql.append(" and s.medal_id = ? ");
                param.add(req.getMedalId());
            }
            // 是否点亮：0-否；1-是
            if (req.getIfLight() != null) {
                sql.append(" and s.if_light = ? ");
                param.add(req.getIfLight());
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }

    /**
     * 根据奖章类型统计学生的奖章情况(基础章、星级章、特色章)
     * @param req
     * @return
     */
    public Integer countByMedalType(StudentMedalReq req){
        log.info("YcqYjsStudentMedalJdbcDao.countByMedalType.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(" select count(yysm.id) from ycq_yjs_student_medal yysm left join ycq_yjs_medal yym on yysm.medal_id = yym.id where yysm.del_flag = 0  ");
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and yysm.student_id = ? ");
                param.add(req.getStudentId());
            }
            // 奖章类型
            if (req.getMedalType() != null) {
                sql.append(" and yym.medal_type = ? ");
                param.add(req.getMedalType());
            }
            // 奖章标识
            if (req.getMedalMark() != null) {
                sql.append(" and yym.medal_mark = ? ");
                param.add(req.getMedalMark());
            }
            // 是否点亮：0-否；1-是
            if (req.getIfLight() != null) {
                sql.append(" and yysm.if_light = ? ");
                param.add(req.getIfLight());
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }

    public List<Map<String,Object>> queryByStudent(BigInteger studentId){
        log.info("YcqYjsStudentMedalJdbcDao.queryByStudent.param：{}", studentId);

        StringBuilder sql = new StringBuilder(" select count(yysm.id) as count, yym.medal_type  " +
                "from ycq_yjs_student_medal yysm " +
                "left join ycq_yjs_medal yym on yysm.medal_id = yym.id " +
                "where yysm.del_flag = 0 and yysm.if_light = 1 and yysm.student_id = ? group by yym.medal_type  ");
        return jt.queryForList(sql.toString(), studentId);
    }

    /**
     * 当前星级章
     * @param req
     * @return
     */
    public String medalStarNow(StudentMedalReq req){
        log.info("YcqYjsStudentMedalJdbcDao.medalStarNow.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(" select yym.medal_name from ycq_yjs_student_medal yysm ");
        sql.append("left join ycq_yjs_medal yym on yysm.medal_id = yym.id ");
        sql.append("left join ycq_yjs_student yys on yys.id = yysm.student_id and yym.owner_grade = yys.current_grade_name and yym.owner_class = yys.current_class_name and yym.owner_semester = yys.semester_name" );
        sql.append(" where yysm.del_flag = 0   ");
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and yysm.student_id = ? ");
                param.add(req.getStudentId());
            }
            // 奖章ID
            if (req.getMedalType() != null) {
                sql.append(" and yym.medal_type = ? ");
                param.add(req.getMedalType());
            }
            // 是否点亮：0-否；1-是
            if (req.getIfLight() != null) {
                sql.append(" and yysm.if_light = ? ");
                param.add(req.getIfLight());
            }
        }
        sql.append(" order by yysm.created_time desc limit 1  ");
        return jt.queryForRawObject(String.class, sql.toString(), param.toArray());
    }


}
