package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudent;
import com.crunii.micro.service.oneaffair.dto.vo.student.*;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.enums.IfFlagEnum;
import com.crunii.micro.service.oneaffair.enums.PoliticalStatusTypeEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentRepo;
import com.crunii.micro.service.oneaffair.service.StudentService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生表(YCQ_YJS_STUDENT)表服务实现类
 */
@Slf4j
@Service
public class StudentServiceImpl implements StudentService {

    @Resource
    private YcqYjsStudentJdbcDao studentJdbcDao;
    @Resource
    private YcqYjsStudentRepo studentRepo;

    /**
     * 分页查询学生数据
     *
     * @param req
     * @return
     */
    @Override
    public PageRsp<StudentInfoVo> pageSearchStudentInfo(PageReq<StudentReq> req) {
        log.info("StudentServiceImpl.pageSearchStudentInfo.param：{}", req.toString());
        return studentJdbcDao.pageSearchStudentInfo(req);
    }

    /**
     * 列表查询学生数据
     *
     * @param req
     * @return
     */
    @Override
    public List<StudentInfoVo> selectList(StudentReq req) {
        log.info("StudentServiceImpl.selectList.param：{}", req.toString());
        return studentJdbcDao.selectList(req);
    }

    /**
     * 列表查询学生数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudent> selectInfoList(StudentReq req) {
        log.info("StudentServiceImpl.selectInfoList.param：{}", req.toString());
        return studentJdbcDao.selectInfoList(req);
    }

    /**
     * 根据身份证查询学生数据
     *
     * @param identityCardNo 身份证
     * @return
     */
    @Override
    public List<YcqYjsStudent> selectListByIdentityCardNo(String identityCardNo) {
        log.info("StudentServiceImpl.selectListByIdentityCardNo.param：{}", identityCardNo);
        return studentJdbcDao.selectListByIdentityCardNo(identityCardNo);
    }

    /**
     * 根据手机号查询学生数据
     *
     * @param mobile 手机号
     * @return
     */
    @Override
    public List<YcqYjsStudent> selectListByMobile(String mobile) {
        log.info("StudentServiceImpl.selectListByMobile.param：{}", mobile);
        return studentJdbcDao.selectListByMobile(mobile);
    }

    /**
     * 根据ID列表查询学生数据
     *
     * @param idList 主键ID列表
     * @return
     */
    @Override
    public List<YcqYjsStudent> selectListByIds(List<BigInteger> idList) {
        log.info("StudentServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentRepo.findAllById(idList);
    }

    /**
     * 根据ID查询学生数据
     *
     * @param id 主键ID
     * @return
     */
    @Override
    public YcqYjsStudent selectOneById(BigInteger id) {
        log.info("StudentServiceImpl.selectOneById.param：{}", id);
        return studentRepo.getOne(id);
    }

    /**
     * 单个新增学生数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudent createStudentInfo(StudentCreateReq createReq) {
        log.info("StudentServiceImpl.createStudentInfo.param：{}", createReq.toString());
        return studentRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudent.class));
    }

    /**
     * 批量新增学生数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudent> insertBatch(List<StudentCreateReq> createReqList) {
        log.info("StudentServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudent.class));
    }

    /**
     * 单个修改学生数据
     *
     * @param updateReq
     */
    @Override
    public void updateStudentInfo(StudentUpdateReq updateReq) {
        log.info("StudentServiceImpl.updateStudentInfo.param：{}", updateReq.toString());
        YcqYjsStudent updateEntity = selectOneById(updateReq.getId());
        if (updateEntity != null) {
            BeanUtils.copyProperties(updateReq, updateEntity);
            studentRepo.saveAndFlush(updateEntity);
        }
    }

    /**
     * 学生入队操作
     *
     * @param updateReq
     */
    @Override
    public void enqueueStudent(StudentEnqueueReq updateReq) {
        log.info("StudentServiceImpl.enqueueStudent.param：{}", updateReq.toString());
        List<YcqYjsStudent> updateEntityList = selectListByIds(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(updateEntityList)) {
            updateEntityList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
            });
            studentRepo.saveAllAndFlush(updateEntityList);
        }
    }

    /**
     * 更新学生政治面貌
     *
     * @param updateReq
     */
    @Override
    public void updateStudentPolitical(StudentPoliticalStatusUpdateReq updateReq) {
        log.info("StudentServiceImpl.updateStudentPolitical.param：{}", updateReq.toString());
        List<YcqYjsStudent> updateEntityList = selectListByIds(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(updateEntityList)) {
            updateEntityList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
            });
            studentRepo.saveAllAndFlush(updateEntityList);
        }
    }

    /**
     * 导师评分
     *
     * @param updateReq
     */
    @Override
    public void teacherScore(TeacherScoreReq updateReq) {
        log.info("StudentServiceImpl.teacherScore.param：{}", updateReq.toString());
        List<YcqYjsStudent> updateEntityList = selectListByIds(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(updateEntityList)) {
            updateEntityList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
            });
            studentRepo.saveAllAndFlush(updateEntityList);
        }
    }

    /**
     * 推优入团
     *
     * @param updateReq
     */
    @Override
    public void promoteBestToLeague(PromoteBestToLeagueReq updateReq) {
        log.info("StudentServiceImpl.promoteBestToLeague.param：{}", updateReq.toString());
        List<YcqYjsStudent> updateEntityList = selectListByIds(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(updateEntityList)) {
            updateEntityList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
            });
            studentRepo.saveAllAndFlush(updateEntityList);
        }
    }

    /**
     * 学生退出少先队
     *
     * @param updateReq
     */
    @Override
    public void quitEnqueueByStudent(StudentBatchUpdateReq updateReq) {
        log.info("StudentServiceImpl.quitEnqueueByStudent.param：{}", updateReq.toString());
        List<YcqYjsStudent> updateEntityList = selectListByIds(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(updateEntityList)) {
            updateEntityList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
                item.setPoliticalStatusType(PoliticalStatusTypeEnum.THE_MASSES.code());
                item.setPoliticalStatus(PoliticalStatusTypeEnum.THE_MASSES.desc());
                item.setJoinYoungPioneersTime(null);
                item.setIfJoinYouthPioneers(IfFlagEnum.FALSE.code());
            });
            studentRepo.saveAllAndFlush(updateEntityList);
        }
    }

    /**
     * 学生退出共青团
     *
     * @param updateReq
     */
    @Override
    public void quitLeagueByStudent(StudentBatchUpdateReq updateReq) {
        log.info("StudentServiceImpl.quitLeagueByStudent.param：{}", updateReq.toString());
        List<YcqYjsStudent> updateEntityList = selectListByIds(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(updateEntityList)) {
            updateEntityList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
                item.setPoliticalStatusType(PoliticalStatusTypeEnum.YOUNG_PIONEERS.code());
                item.setPoliticalStatus(PoliticalStatusTypeEnum.YOUNG_PIONEERS.desc());
                item.setScoreTime(null);
                item.setTeacherScore(0);
                item.setIfJoinYouthLeague(IfFlagEnum.FALSE.code());
                item.setJoinYouthLeagueTime(null);
            });
            studentRepo.saveAllAndFlush(updateEntityList);
        }
    }

    /**
     * 批量删除学生数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentInfoDeleteReq deleteReq) {
        log.info("StudentServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudent> deleteEntityList = studentRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentRepo.saveAllAndFlush(deleteEntityList);
        }
    }


}