package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacher;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.*;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsTeacherJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsTeacherRepo;
import com.crunii.micro.service.oneaffair.service.TeacherService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 教师表(YCQ_YJS_TEACHER)表服务实现类
 */
@Slf4j
@Service
public class TeacherServiceImpl implements TeacherService {

    @Resource
    private YcqYjsTeacherJdbcDao teacherJdbcDao;
    @Resource
    private YcqYjsTeacherRepo teacherRepo;

    /**
     * 根据ID查询教师数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsTeacher selectOneById(BigInteger id) {
        log.info("TeacherServiceImpl.selectOneById.param：{}", id);
        return teacherRepo.getOne(id);
    }

    /**
     * 根据ID列表查询教师数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsTeacher> selectListByIds(List<BigInteger> idList) {
        log.info("TeacherServiceImpl.selectListByIds.param：{}", idList.toString());
        return teacherRepo.findAllById(idList);
    }

    /**
     * 分页查询教师数据
     *
     * @param req
     * @return
     */
    @Override
    public PageRsp<TeacherInfoVo> pageSearchTeacherInfo(PageReq<TeacherReq> req) {
        log.info("TeacherServiceImpl.pageSearchTeacherInfo.param：{}", req.toString());
        return teacherJdbcDao.pageSearchTeacherInfo(req);
    }

    /**
     * 列表查询教师数据
     *
     * @param req
     * @return
     */
    @Override
    public List<TeacherInfoVo> selectTeacherInfoList(TeacherReq req) {
        log.info("TeacherServiceImpl.selectTeacherInfoList.param：{}", req.toString());
        return teacherJdbcDao.selectTeacherInfoList(req);
    }

    /**
     * 根据身份证查询教师数据
     *
     * @param identityCardNo 身份证
     * @return
     */
    @Override
    public List<YcqYjsTeacher> selectListByIdentityCardNo(String identityCardNo) {
        log.info("TeacherServiceImpl.selectListByIdentityCardNo.param：{}", identityCardNo);
        return teacherJdbcDao.selectListByIdentityCardNo(identityCardNo);
    }

    /**
     * 根据手机号查询教师数据
     * @param mobile 手机号
     * @return
     */
    @Override
    public List<YcqYjsTeacher> selectListByMobile(String mobile) {
        log.info("TeacherServiceImpl.selectListByMobile.param：{}", mobile);
        return teacherJdbcDao.selectListByMobile(mobile);
    }

    /**
     * 单个新增教师数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsTeacher insertOne(TeacherInfoCreateReq createReq) {
        log.info("TeacherServiceImpl.insertOne.param：{}", createReq.toString());
        return teacherRepo.save(BeanCopyUtil.convert(createReq, YcqYjsTeacher.class));
    }

    /**
     * 批量新增教师数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsTeacher> insertBatch(List<TeacherInfoCreateReq> createReqList) {
        log.info("TeacherServiceImpl.insertBatch.param：{}", createReqList.toString());
        return teacherRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsTeacher.class));
    }

    /**
     * 单个修改教师数据
     *
     * @param updateReq
     */
    @Override
    public void updateOne(TeacherInfoUpdateReq updateReq) {
        log.info("TeacherServiceImpl.updateOne.param：{}", updateReq.toString());
        // 校验教师是否存在
        YcqYjsTeacher updateEntity = selectOneById(updateReq.getId());
        if (updateEntity != null) {
            BeanUtils.copyProperties(updateReq, updateEntity);
            teacherRepo.saveAndFlush(updateEntity);
        }
    }

    /**
     * 批量删除教师数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(TeacherInfoDeleteReq deleteReq) {
        log.info("TeacherServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsTeacher> deleteEntityList = teacherRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                item.setLastUpdatedId(deleteReq.getLastUpdatedId());
                item.setLastUpdatedName(deleteReq.getLastUpdatedName());
                item.setLastUpdatedTime(deleteReq.getLastUpdatedTime());
            });
            teacherRepo.saveAllAndFlush(deleteEntityList);
        }
    }

}
