package com.crunii.micro.service.pay.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.nacos.shaded.com.google.gson.Gson;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.pay.dto.Product;
import com.crunii.micro.service.pay.service.WxPayH5;
import com.crunii.micro.service.pay.util.WechatPayUtil;
import com.crunii.micro.service.pay.util.WxPayH5Url;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static com.crunii.micro.common.utils.IPUtil.getIPAddr;
import static com.crunii.micro.service.pay.util.WechatPayUtil.subZeroAndDot;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/8 11:11
 */
@Service
@Slf4j
public class WxPayH5Service implements WxPayH5 {
    /**
     * APP_ID（公众号/小程序等）
     */
    @Value("${wechat-pay.app-id}")
    private String appId;

    /**
     * 商户号 （微信商户平台可以获取）
     */
    @Value("${wechat-pay.merchant-id}")
    private String merchantId;

    @Value("${wechat-pay.notify-url}")
    private String notifyUrl;


    @Override
    public Result<String> h5Pay(HttpServletRequest request, Product product) throws Exception {
        // 获取ip
        if (ObjectUtil.isEmpty(product.getSpbillCreateIp())) {
            String ipAddress = getIPAddr(request);
            if (StrUtil.isBlank(ipAddress)) {
                product.setSpbillCreateIp("127.0.0.1");
            }
        }

        // 初始化请求
        CloseableHttpClient applyClient = WechatPayUtil.initClientBuilder().build();
        // 构造请求
        HttpPost httpPost = new HttpPost(WxPayH5Url.UNIFIED_ORDER_URL);
        // 请求头设置
        httpPost.addHeader("Accept", "application/json");
        httpPost.addHeader("Content-type", "application/json; charset=utf-8");
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        // 请求参数
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("appid", appId)
                .put("mchid", merchantId)
                .put("description", product.getDescription())
                .put("out_trade_no", product.getOutTradeNo())
                .put("notify_url", notifyUrl);
        rootNode.putObject("amount")
                .put("total", subZeroAndDot(product.getTotalFee()));
        rootNode.putObject("scene_info")
                .put("payer_client_ip", product.getSpbillCreateIp())
                .putObject("h5_info")
                .put("type", product.getPayWay());

        objectMapper.writeValue(bos, rootNode);

        httpPost.setEntity(new StringEntity(bos.toString("UTF-8"), "UTF-8"));
        // 发起请求
        CloseableHttpResponse response = applyClient.execute(httpPost);
        try {
            // 响应体
            String bodyAsString = EntityUtils.toString(response.getEntity());
            // 响应状态码
            int statusCode = response.getStatusLine().getStatusCode();
            // 处理成功
            if (statusCode == 200) {
                log.info("成功, 返回结果 = " + bodyAsString);
                //处理成功，无返回Body
            } else if (statusCode == 204) {
                log.info("成功");
            } else {
                log.info("查单接口调用,响应码 = " + statusCode + ",返回结果 = " + bodyAsString);
                throw new IOException("request failed");
            }
            return Result.ofSuccess(bodyAsString);

        } finally {
            response.close();
        }
    }

    @Override
    public Result<String> h5OutTradeNo(String outTradeNo) throws Exception {
        log.info("查单接口调用 ===> {}", outTradeNo);
        // 初始化请求
        CloseableHttpClient applyClient = WechatPayUtil.initClientBuilder().build();

        String url = String.format(WxPayH5Url.TRANSACTIONS_ID_URL, outTradeNo).concat("?mchid=").concat(merchantId);

        HttpGet httpGet = new HttpGet(url);
        httpGet.setHeader("Accept", "application/json");
        httpGet.setHeader("Content-type", "application/json; charset=utf-8");

        // 发起请求
        CloseableHttpResponse response = applyClient.execute(httpGet);

        try {
            // 响应体
            String bodyAsString = EntityUtils.toString(response.getEntity());
            // 响应状态码
            int statusCode = response.getStatusLine().getStatusCode();
            // 处理成功
            if (statusCode == 200) {
                log.info("成功, 返回结果 = " + bodyAsString);

                //处理成功，无返回Body
            } else if (statusCode == 204) {
                log.info("成功");
            } else {
                log.info("查单接口调用,响应码 = " + statusCode + ",返回结果 = " + bodyAsString);
                throw new IOException("request failed");
            }
            return Result.ofSuccess(bodyAsString);
        } finally {
            response.close();
        }
    }

    @Override
    public Result<String> h5CloseOutTradeNo(String outTradeNo) throws Exception {
        log.info("关闭订单接口调用 ===> {}", outTradeNo);
        // 初始化请求
        CloseableHttpClient applyClient = WechatPayUtil.initClientBuilder().build();

        String url = String.format(WxPayH5Url.CLOSE_ID_URL, outTradeNo);

        HttpPost httpPost = new HttpPost(url);

        //组装json请求体
        Gson gson = new Gson();
        Map<String, String> paramsMap = new HashMap<>();
        paramsMap.put("mchid", merchantId);
        String jsonParams = gson.toJson(paramsMap);
        log.info("请求参数 ===> {}", jsonParams);

        //将请求参数设置到请求对象中
        StringEntity entity = new StringEntity(jsonParams, "utf-8");
        entity.setContentType("application/json");
        httpPost.setEntity(entity);
        httpPost.setHeader("Accept", "application/json");
        httpPost.setHeader("Content-type", "application/json; charset=utf-8");

        // 发起请求
        CloseableHttpResponse response = applyClient.execute(httpPost);

        try {
            // 响应状态码
            int statusCode = response.getStatusLine().getStatusCode();
            // 处理成功
            if (statusCode == 200) {
                log.info("成功200");
                //处理成功，无返回Body
            } else if (statusCode == 204) {
                log.info("成功204");
            } else {
                log.info("订单关闭失败");
                throw new IOException("request failed");
            }
            return Result.ofSuccess("订单关闭成功");
        } finally {
            response.close();
        }
    }
}
