package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmBookList;
import com.crunii.micro.domain.entity.main.YcqQmBookListRel;
import com.crunii.micro.domain.repo.main.YcqQmBookListRelRepo;
import com.crunii.micro.domain.repo.main.YcqQmBookListRepo;
import com.crunii.micro.service.qm.api.domain.dto.Book.BookInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.bookList.*;
import com.crunii.micro.service.qm.jdbc.BookListDao;
import com.crunii.micro.service.qm.service.BookListService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/19 14:15
 */
@Service
@Slf4j
public class BookListServiceImpl implements BookListService {

    @Resource
    private BookListDao bookListDao;

    @Resource
    private YcqQmBookListRepo bookListRepo;

    @Resource
    private YcqQmBookListRelRepo relRepo;

    @Override
    @InjectProperty
    public PageRsp<BookListVo> bookListPage(PageReq<BookListDto> req) {
        PageRsp<BookListVo> bookListVo = bookListDao.pageList(req);
        List<BookListVo> result = bookListVo.getResult();
        if (CollectionUtil.isNotEmpty(result)) {
            // 查询关联书籍数量
            List<Long> collect = result.stream().map(BookListVo::getBookListId).collect(Collectors.toList());
            List<BookListCount> countList = bookListDao.queryBookListCount(collect);
            Map<Long, Integer> map = countList.stream().collect(Collectors.toMap(BookListCount::getBookListId, BookListCount::getAssociatedBook));
            for (BookListVo listVo : result) {
                Integer count = map.get(listVo.getBookListId());
                listVo.setAssociatedBook(ObjectUtil.isEmpty(count) ? 0 : count);
            }
        } else {
            result = new ArrayList<>();
        }
        bookListVo.setResult(result);
        return bookListVo;
    }

    @Override
    public Result<String> createBookList(Worker worker, BookListC bookListC) {
        YcqQmBookList ycqQmBookList = new YcqQmBookList();
        BeanUtils.copyProperties(bookListC, ycqQmBookList);
        ycqQmBookList.setBookListCode(getResNumber());
        ycqQmBookList.setCreatorId(worker.getLoginId());
        ycqQmBookList.setCreateTime(new Date());
        bookListRepo.saveAndFlush(ycqQmBookList);
        // 关联书籍
        if (CollectionUtil.isNotEmpty(bookListC.getBookIds())) {
            bookListC.getBookIds().forEach(x -> {
                YcqQmBookListRel rel = new YcqQmBookListRel();
                rel.setBookId(x);
                rel.setBookListId(ycqQmBookList.getBookListId());
                rel.setCreateTime(new Date());
                rel.setCreatorId(worker.getLoginId());
                relRepo.saveAndFlush(rel);
            });
        }
        return Result.ofSuccess("新增成功");
    }

    @Override
    public Result<String> updateBookList(Worker worker, BookListU bookListU) {
        YcqQmBookList bookList = bookListRepo.findByBookListIdAndDeleted(bookListU.getBookListId(), CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(bookList)) {
            return Result.ofFail("编辑失败");
        }
        BeanUtils.copyProperties(bookListU, bookList);
        bookList.setUpdateTime(new Date());
        bookList.setUpdaterId(worker.getLoginId());
        bookListRepo.saveAndFlush(bookList);

        Map<String, Object> props = new HashMap<String, Object>(6);
        props.put("bookListId", bookListU.getBookListId());
        List<YcqQmBookListRel> oldRelas = relRepo.findByProps(props);
        relRepo.deleteAll(oldRelas);
        relRepo.flush();

        // 关联书籍
        if (CollectionUtil.isNotEmpty(bookListU.getBookIds())) {
            bookListU.getBookIds().forEach(x -> {
                YcqQmBookListRel rel = new YcqQmBookListRel();
                rel.setBookId(x);
                rel.setBookListId(bookList.getBookListId());
                rel.setCreateTime(new Date());
                rel.setCreatorId(worker.getLoginId());
                rel.setUpdaterId(worker.getLoginId());
                rel.setUpdateTime(new Date());
                relRepo.saveAndFlush(rel);
            });
        }
        return Result.ofSuccess("编辑成功");
    }

    @Override
    @InjectProperty
    public Result<BookListResVo> viewBookList(Worker worker, Long bookListId) {
        BookListResVo vo = new BookListResVo();
        YcqQmBookList bookList = bookListRepo.findByBookListIdAndDeleted(bookListId, CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(bookList)) {
            throw new BusinessException("查看失败");
        }
        vo.setBookListName(bookList.getBookListName());
        vo.setBookListTheme(bookList.getBookListTheme());
        vo.setBookListDesc(bookList.getBookListDesc());
        // 查询关联的书籍
        vo.setBookInfos(bookListDao.queryBookInfo(bookListId));
        return Result.ofSuccess(vo);
    }

    @Override
    public Result<String> removeBook(Worker worker, Long relId) {
        YcqQmBookListRel rel = relRepo.findByRelIdAndDeleted(relId, CommonConstants.STATE_NO);
        rel.setUpdaterId(worker.getLoginId());
        rel.setUpdateTime(new Date());
        rel.setDeleted(CommonConstants.STATE_VALID);
        relRepo.saveAndFlush(rel);
        return Result.ofSuccess("操作成功");
    }

    @Override
    public Result<String> deleteBookList(Worker worker, List<Long> bookListIds) {
        if (CollectionUtil.isEmpty(bookListIds)) {
            return Result.ofFail("操作失败");
        }
        for (Long bookListId : bookListIds) {
            YcqQmBookList bookList = bookListRepo.findByBookListIdAndDeleted(bookListId, CommonConstants.STATE_NO);
            if (ObjectUtil.isEmpty(bookList)) {
                continue;
            }
            if (bookList.getBookListStatus() == CommonConstants.STATE_VALID) {
                throw new BusinessException("选中的数据中存在上架资源，请先下架，再删除");
            }
            bookList.setUpdateTime(new Date());
            bookList.setUpdaterId(worker.getLoginId());
            bookList.setDeleted(CommonConstants.STATE_VALID);
            bookListRepo.saveAndFlush(bookList);
            // 删除关联关系
            List<YcqQmBookListRel> rel = relRepo.findAllByBookListIdAndDeleted(bookListId, CommonConstants.STATE_NO);
            if (CollectionUtil.isNotEmpty(rel)) {
                rel.forEach(x -> {
                    x.setUpdaterId(worker.getLoginId());
                    x.setDeleted(CommonConstants.STATE_VALID);
                    x.setUpdateTime(new Date());
                });
                relRepo.saveAll(rel);
                relRepo.flush();
            }
        }
        return Result.ofSuccess("操作成功");
    }

    @Override
    public Result<String> updateStatus(Worker worker, Long bookListId, Integer bookListStatus) {
        YcqQmBookList bookList = bookListRepo.findByBookListIdAndDeleted(bookListId, CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(bookList)) {
            return Result.ofFail("操作失败");
        }
        bookList.setBookListStatus(bookListStatus);
        bookList.setUpdateTime(new Date());
        bookList.setUpdaterId(worker.getLoginId());
        bookListRepo.saveAndFlush(bookList);
        return Result.ofSuccess("操作成功");
    }

    @Override
    public Result<String> associatedBookList(Worker worker, AssociatedBookListDto dto) {
        Map<String, Object> props = new HashMap<String, Object>(6);
        props.put("bookListId", dto.getBookListId());
        List<YcqQmBookListRel> rels = relRepo.findByProps(props);
        relRepo.deleteAll(rels);
        relRepo.flush();
        // 关联书籍
        if (CollectionUtil.isNotEmpty(dto.getBookIds())) {
            dto.getBookIds().forEach(x -> {
                YcqQmBookListRel rel = new YcqQmBookListRel();
                rel.setBookId(x);
                rel.setBookListId(dto.getBookListId());
                rel.setCreateTime(new Date());
                rel.setCreatorId(worker.getLoginId());
                rel.setUpdaterId(worker.getLoginId());
                rel.setUpdateTime(new Date());
                relRepo.saveAndFlush(rel);
            });
        }
        return Result.ofSuccess("操作成功");
    }

    @Override
    public PageRsp<BookInfoVo> queryAssociateBook(PageReq<AppBookListInfoDto> req) {
        return bookListDao.queryBookInfos(req);
    }

    public static String getResNumber() {
        return "SD" + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + getCode();
    }

    public static String getCode() {
        Random random = new Random();
        StringBuilder code = new StringBuilder();
        for (int i = 0; i < 4; i++) {
            int rand = random.nextInt(10);
            code.append(rand);
        }
        return code.toString();
    }
}
