package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmBook;
import com.crunii.micro.domain.entity.main.YcqQmBookListRel;
import com.crunii.micro.domain.repo.main.YcqQmBookListRelRepo;
import com.crunii.micro.domain.repo.main.YcqQmBookRepo;
import com.crunii.micro.service.qm.api.domain.dto.Book.*;
import com.crunii.micro.service.qm.api.domain.dto.bookList.BookListInfo;
import com.crunii.micro.service.qm.jdbc.BookDao;
import com.crunii.micro.service.qm.service.BookService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;
import java.util.Random;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/18 19:00
 */
@Service
@Slf4j
public class BookServiceImpl implements BookService {

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private BookDao bookDao;

    @Resource
    private YcqQmBookRepo bookRepo;

    @Resource
    private YcqQmBookListRelRepo relRepo;

    @Override
    public PageRsp<BookVo> bookPage(PageReq<BookDto> req) {
        PageRsp<BookVo> bookVo = bookDao.pageList(req);
        List<BookVo> result = bookVo.getResult();
        result.forEach(x -> {
            x.setSerialNum(result.indexOf(x) + 1);
            if (ObjectUtil.isNotEmpty(x.getFileKey())) {
                x.setUrl(urlPrefix + x.getFileKey());
            }
        });
        bookVo.setResult(result);
        return bookVo;
    }

    @Override
    public Result<String> createBook(Worker worker, BookC bookC) {
        YcqQmBook ycqQmBook = new YcqQmBook();
        BeanUtils.copyProperties(bookC, ycqQmBook);
        ycqQmBook.setBookCode(getResNumber());
        ycqQmBook.setUpdateWorkerName(worker.getWorkerName());
        ycqQmBook.setCreatorId(worker.getLoginId());
        ycqQmBook.setCreateTime(new Date());
        bookRepo.saveAndFlush(ycqQmBook);
        return Result.ofSuccess("新增成功");
    }

    @Override
    public Result<String> updateBook(Worker worker, BookU bookU) {
        YcqQmBook book = bookRepo.findByBookIdAndDeleted(bookU.getBookId(), CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(book)) {
            return Result.ofFail("编辑失败");
        }
        BeanUtils.copyProperties(bookU, book);
        book.setUpdateTime(new Date());
        book.setUpdaterId(worker.getLoginId());
        bookRepo.saveAndFlush(book);
        return Result.ofSuccess("编辑成功");
    }

    @Override
    public Result<String> deleteBook(Worker worker, List<Long> bookId) {
        if (CollectionUtil.isEmpty(bookId)) {
            return Result.ofFail("操作失败");
        }
        for (Long id : bookId) {
            YcqQmBook book = bookRepo.findByBookIdAndDeleted(id, CommonConstants.STATE_NO);
            if (ObjectUtil.isEmpty(book)) {
                continue;
            }
            if (book.getBookStatus() == CommonConstants.STATE_VALID) {
                throw new BusinessException("选中的数据中存在上架资源，请先下架，再删除");
            }
            book.setUpdateTime(new Date());
            book.setUpdaterId(worker.getLoginId());
            book.setDeleted(CommonConstants.STATE_VALID);
            bookRepo.saveAndFlush(book);
            // 删除关联书单中的书籍
            List<YcqQmBookListRel> rel = relRepo.findAllByBookIdAndDeleted(id, CommonConstants.STATE_NO);
            if (CollectionUtil.isNotEmpty(rel)) {
                rel.forEach(x -> {
                    x.setUpdaterId(worker.getLoginId());
                    x.setDeleted(CommonConstants.STATE_VALID);
                    x.setUpdateTime(new Date());
                });
                relRepo.saveAll(rel);
                relRepo.flush();
            }
        }
        return Result.ofSuccess("操作成功");
    }

    @Override
    @InjectProperty
    public Result<BookResVo> viewBook(Worker worker, Long bookId) {
        BookResVo vo = new BookResVo();
        YcqQmBook book = bookRepo.findByBookIdAndDeleted(bookId, CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(book)) {
            throw new BusinessException("查看失败");
        }
        vo.setBookName(book.getBookName());
        vo.setBookAuthor(book.getBookAuthor());
        vo.setBookDesc(book.getBookDesc());
        // 查询关联书籍
        List<BookListInfo> list = bookDao.queryBookListInfo(bookId);
        vo.setBookListInfos(list);
        return Result.ofSuccess(vo);
    }

    @Override
    public Result<String> removeBookList(Worker worker, Long bookId, Long bookListId) {
        YcqQmBookListRel rel = relRepo.findByBookIdAndBookListId(bookId, bookListId);
        rel.setUpdaterId(worker.getLoginId());
        rel.setUpdateTime(new Date());
        rel.setDeleted(CommonConstants.STATE_VALID);
        relRepo.saveAndFlush(rel);
        return Result.ofSuccess("操作成功");
    }

    @Override
    public Result<String> updateStatus(Worker worker, Long bookId, Integer bookStatus) {
        YcqQmBook book = bookRepo.findByBookIdAndDeleted(bookId, CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(book)) {
            return Result.ofFail("操作失败");
        }
        book.setBookStatus(bookStatus);
        book.setUpdateTime(new Date());
        book.setUpdaterId(worker.getLoginId());
        bookRepo.saveAndFlush(book);
        return Result.ofSuccess("操作成功");
    }

    public static String getResNumber() {
        return "SJ" + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + getCode();
    }

    public static String getCode() {
        Random random = new Random();
        StringBuilder code = new StringBuilder();
        for (int i = 0; i < 4; i++) {
            int rand = random.nextInt(10);
            code.append(rand);
        }
        return code.toString();
    }
}
