package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.domain.entity.main.YcqQmCourseware;
import com.crunii.micro.domain.entity.main.YcqQmVideo;
import com.crunii.micro.domain.repo.main.YcqObsFileRepo;
import com.crunii.micro.domain.repo.main.YcqQmCoursewareRepo;
import com.crunii.micro.domain.repo.main.YcqQmVideoRepo;
import com.crunii.micro.service.qm.api.domain.dto.courseware.*;
import com.crunii.micro.service.qm.jdbc.CoursewareDao;
import com.crunii.micro.service.qm.service.CoursewareService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;
import java.util.Random;

import static com.crunii.micro.service.qm.api.constants.QmConstants.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/18 12:06
 */
@Service
@Slf4j
public class CoursewareServiceImpl implements CoursewareService {

    @Resource
    private CoursewareDao coursewareDao;

    @Resource
    private YcqQmCoursewareRepo repo;

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private YcqObsFileRepo fileRepo;

    @Resource
    private YcqQmVideoRepo videoRepo;

    @Override
    @InjectProperty
    public PageRsp<CoursewareVo> coursewarePage(PageReq<CoursewareDto> req) {
        PageRsp<CoursewareVo> vo = new PageRsp<>();
        List<CoursewareVo> result = coursewareDao.pageList(req).getResult();
        result.forEach(x -> {
            x.setSerialNum(result.indexOf(x) + 1);
            // PDF
            if (x.getCertificateType().equals(CERTIFICATE_TYPE_PDF)) {
                YcqObsFile file = fileRepo.findByIdAndState(Long.valueOf(x.getFileId()), STATUS);
                if (ObjectUtil.isNotEmpty(file)) {
                    x.setFileName(file.getFileName());
                    x.setUrl(urlPrefix + x.getFileKey());
                }
                // 视频
            } else {
                YcqQmVideo video = videoRepo.findByFileIdAndDeleted(x.getFileId(), DELETED_0);
                if (ObjectUtil.isNotEmpty(video)) {
                    x.setFileName(video.getFileName());
                    x.setUrl(video.getPictureUrl());
                }
            }
        });
        vo.setResult(result);
        return vo;
    }

    @Override
    public Result<String> createCourseware(Worker worker, CoursewareC coursewareC) {
        YcqQmCourseware qmCourseware = new YcqQmCourseware();
        // 判断类型和上传文件是否相同
        if (!coursewareC.getCertificateType().equals(videoOrPdfType(coursewareC.getFileId()))){
            return Result.ofFail("课件类型和上传文件类型不匹配");
        }
        BeanUtils.copyProperties(coursewareC, qmCourseware);
        qmCourseware.setResNumber(getResNumber());
        qmCourseware.setWorkerName(worker.getWorkerName());
        qmCourseware.setCreatorId(worker.getLoginId());
        qmCourseware.setCreateTime(new Date());
        repo.saveAndFlush(qmCourseware);
        return Result.ofSuccess("新增成功");
    }

    @Override
    public Result<String> updateCourseware(Worker worker, CoursewareU coursewareU) {
        YcqQmCourseware courseware = repo.findByCertificateIdAndDeleted(coursewareU.getCertificateId(), CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(courseware)) {
            return Result.ofFail("编辑失败");
        }
        // 判断类型和上传文件是否相同
        if (!coursewareU.getCertificateType().equals(videoOrPdfType(coursewareU.getFileId()))){
            return Result.ofFail("课件类型和上传文件类型不匹配");
        }
        BeanUtils.copyProperties(coursewareU, courseware);
        courseware.setUpdateTime(new Date());
        courseware.setUpdaterId(worker.getLoginId());
        repo.saveAndFlush(courseware);
        return Result.ofSuccess("编辑成功");
    }

    public Integer videoOrPdfType(String fileId) {
        YcqQmVideo video = videoRepo.findByFileIdAndDeleted(fileId, DELETED_0);
        if (ObjectUtil.isNotEmpty(video)){
            return CERTIFICATE_TYPE_VIDEO;
        }
        return CERTIFICATE_TYPE_PDF;
    }

    @Override
    public Result<String> deleteCourseware(Worker worker, List<Long> certificateId) {
        if (CollectionUtil.isEmpty(certificateId)) {
            return Result.ofFail("操作失败");
        }
        for (Long id : certificateId) {
            YcqQmCourseware courseware = repo.findByCertificateIdAndDeleted(id, CommonConstants.STATE_NO);
            if (ObjectUtil.isEmpty(courseware)) {
                continue;
            }
            if (courseware.getCertificateStatus() == CommonConstants.STATE_VALID) {
                throw new BusinessException("选中的数据中存在上架资源，请先下架，再删除");
            }
            courseware.setUpdateTime(new Date());
            courseware.setUpdaterId(worker.getLoginId());
            courseware.setDeleted(CommonConstants.STATE_VALID);
            repo.saveAndFlush(courseware);
        }
        return Result.ofSuccess("操作成功");
    }

    @Override
    @InjectProperty
    public Result<CoursewareResVo> viewCourseware(Worker worker, Long certificateId) {
        CoursewareResVo vo = new CoursewareResVo();
        CoursewareInfoVo infoVo = new CoursewareInfoVo();
        YcqQmCourseware courseware = repo.findByCertificateIdAndDeleted(certificateId, CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(courseware)) {
            throw new BusinessException("课件不存在");
        }
        infoVo.setCertificateName(courseware.getCertificateName());
        infoVo.setCertificateTopic(courseware.getCertificateTopic());
        infoVo.setCertificateIntro(courseware.getCertificateIntro());
        // 查询课件下面的附件
        CoursewareResInfoVo resInfoVo;
        //PDF
        if (courseware.getCertificateType().equals(CERTIFICATE_TYPE_PDF)) {
            resInfoVo = coursewareDao.getCoursewareResInfoVo(Long.valueOf(courseware.getFileId()));
            if (ObjectUtil.isNotEmpty(resInfoVo)) {
                // 资源链接
                resInfoVo.setResUrl(urlPrefix + courseware.getFileKey());
            }
        } else {
            // 视频
            resInfoVo = coursewareDao.getCoursewareVideoInfoVo(courseware.getFileId());
            if (ObjectUtil.isEmpty(resInfoVo)) {
                resInfoVo = new CoursewareResInfoVo();
            }
        }
        // 课件类型
        resInfoVo.setCertificateType(courseware.getCertificateType());
        vo.setCoursewareInfoVo(infoVo);
        vo.setCoursewareResInfoVo(resInfoVo);
        return Result.ofSuccess(vo);
    }

    @Override
    public Result<String> updateStatus(Worker worker, Long certificateId, Integer certificateStatus) {
        YcqQmCourseware courseware = repo.findByCertificateIdAndDeleted(certificateId, CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(courseware)) {
            return Result.ofFail("操作失败");
        }
        courseware.setCertificateStatus(certificateStatus);
        courseware.setUpdateTime(new Date());
        courseware.setUpdaterId(worker.getLoginId());
        repo.saveAndFlush(courseware);
        return Result.ofSuccess("操作成功");
    }

    public static String getResNumber() {
        return "KC" + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + getCode();
    }

    public static String getCode() {
        Random random = new Random();
        StringBuilder code = new StringBuilder();
        for (int i = 0; i < 4; i++) {
            int rand = random.nextInt(10);
            code.append(rand);
        }
        return code.toString();
    }
}
