package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.entity.main.YcqQmUserInfo;
import com.crunii.micro.domain.repo.main.YcqQmUserInfoRepo;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.homepage.vo.HomePageBookListVo;
import com.crunii.micro.service.qm.api.domain.dto.homepage.vo.HomePageBookVo;
import com.crunii.micro.service.qm.api.domain.dto.homepage.vo.HomeStuCountVo;
import com.crunii.micro.service.qm.jdbc.HomePageDao;
import com.crunii.micro.service.qm.service.HomePageService;
import com.crunii.micro.service.qm.util.ListUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author xiayinqiang
 * @date 2023/9/19 0019 11:08:08
 */
@Slf4j
@Service
public class HomePageServiceImpl implements HomePageService {

	@Autowired HomePageDao homePageDao;
	@Autowired YcqQmUserInfoRepo ycqQmUserInfoRepo;

	@Value("${upload.url}")
	private String urlPrefix;
	/**
	 * 查询首页展示书单书籍列表
	 *
	 * @return Result<List < HomePageBookListVo>>
	 */
	@Override
	@InjectProperty
	public Result<List<HomePageBookListVo>> selectHomeBookList() {
		Result<List<HomePageBookListVo>> result = new Result<>();
		//查询展示到首页的最新的四个书单
		List<HomePageBookListVo> bookListIdList = homePageDao.selectBookListByFour();
		if (!ListUtil.hasValue(bookListIdList)) {
			result.setData(new ArrayList<>());
			return result;
		}
		for (HomePageBookListVo homePageBookListVo : bookListIdList) {
			List<HomePageBookVo> ycqQmBookList = homePageDao.selectBookByBookListId(homePageBookListVo.getBookListId());
			ycqQmBookList.forEach(ycqQmBook->ycqQmBook.setFileKey(urlPrefix+ycqQmBook.getFileKey()));
			homePageBookListVo.setHomePageBookVos(ycqQmBookList);
		}
		result.setData(bookListIdList);
		return result;
	}

	/**
	 * 导师端查询首页展示学员各种数量
	 *
	 * @return Result<HomePageBookListVo>
	 * @param loginId 人员id
	 */
	@Override public Result<HomeStuCountVo> selectHomeStuCount(int loginId) {
		Result<HomeStuCountVo> result = new Result<>();
		HomeStuCountVo homeStuCountVo = new HomeStuCountVo();
		YcqQmUserInfo ycqQmUserInfo = ycqQmUserInfoRepo.findByWorkerIdAndDeleted(loginId, QmConstants.DELETED_0);
		if (ObjectUtil.isEmpty(ycqQmUserInfo)) {
			return Result.ofFail("未查询到该人员信息");
		}
		if(QmConstants.USER_IDENTITY_STUDENT==ycqQmUserInfo.getUserIdentity()){
			return Result.ofFail("该用户是学员");
		}
		//查询学员数量
		List<Map<String,Object>> countList = homePageDao.selectStuCountByCorrelationState(loginId);
		for(Map<String,Object> map:countList){
			if(map.get("CORRELATION_STATE").toString().equals("0")){//待接收
				homeStuCountVo.setWaitStuCount(Integer.valueOf(map.get("con").toString()));
			}
			if(map.get("CORRELATION_STATE").toString().equals("1")){//已接收
				homeStuCountVo.setAcceptedStuCount(Integer.valueOf(map.get("con").toString()));
			}
			if(map.get("CORRELATION_STATE").toString().equals("2")){//已评价
				int evaluatedStuCount = Integer.valueOf(map.get("con").toString());
				homeStuCountVo.setEvaluatedStuCount(evaluatedStuCount);
				//已接受数量中需加上已评价的数量
				homeStuCountVo.setAcceptedStuCount(homeStuCountVo.getAcceptedStuCount()+evaluatedStuCount);
			}
		}
		//查询导师选择学员数量
		int stuCount = homePageDao.selectStuCountByChooseState(loginId);
		homeStuCountVo.setSelectedStuCount(stuCount);
		result.setData(homeStuCountVo);
		return result;
	}

	/**
	 * 查询导师聘用时长或学员入学时间
	 *
	 * @param loginId 用户id
	 * @return Result<String>
	 */
	@Override public Result<String> selectTeacherExpandDate(int loginId) {
		Result<String> result = new Result<>();
		YcqQmUserInfo ycqQmUserInfo = ycqQmUserInfoRepo.findByWorkerIdAndDeleted(loginId, QmConstants.DELETED_0);
		if (ObjectUtil.isEmpty(ycqQmUserInfo)) {
			return Result.ofFail("未查询到该人员信息");
		}
		if (QmConstants.USER_IDENTITY_STUDENT == ycqQmUserInfo.getUserIdentity()) {
			//学员
			String date = DateUtil.format(ycqQmUserInfo.getCreateTime(), "yyyy-MM-dd");
			result.setData("入学时间：" + date);
		} else if (QmConstants.USER_IDENTITY_TEACHER == ycqQmUserInfo.getUserIdentity()) {
			//导师
			LocalDateTime currentDate = LocalDateTime.now();
			Date createTime = ycqQmUserInfo.getCreateTime();
			LocalDateTime localCreateTime = createTime.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
			LocalDateTime expireTime = localCreateTime.plusYears(1);//到期时间
			if (expireTime.isBefore(currentDate)) {
				//一年前
				result.setData("导师任期：已解聘");
			} else {
				long daysDiff = ChronoUnit.DAYS.between(currentDate, expireTime);
				if(daysDiff>0){
					result.setData("导师任期：剩"+daysDiff+"天");
				}else{
					result.setData("导师任期：不足一天");
				}
			}
		}
		return result;
	}

	public static void main(String[] args) {
		// 获取当前日期
		LocalDateTime currentDate = LocalDateTime.now();
		// 假设要比较的日期是2022年9月20日
		String s = "2022-09-21 21:00:00";
		Date parse = DateUtil.parse(s);
		LocalDateTime targetDate = parse.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
		LocalDateTime localDateTime = targetDate.plusYears(1);//到期时间

		if(localDateTime.isBefore(currentDate)){
			//一年前
			System.out.println("一年前");
		}else{
			System.out.println("不到一年");
			long daysDiff = ChronoUnit.DAYS.between(currentDate, localDateTime);
			System.out.println("指定日期与当前日期相差 " + daysDiff + " 天");
		}


	}

}
