package com.crunii.micro.service.qm.service.impl;
import java.util.Date;
import java.util.*;
import java.util.stream.Collectors;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.ExertDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.GroupCultivationDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.StuHonorDto;
import com.crunii.micro.service.qm.api.enums.StuAuditFlowEnum;
import com.crunii.micro.service.qm.api.enums.StudentAuditEnum;
import com.crunii.micro.service.qm.config.QmFlowConfig;
import com.crunii.micro.service.qm.jdbc.StudentAppDao;
import com.crunii.micro.service.qm.service.CommonService;
import com.crunii.micro.service.qm.service.StudentAppService;
import com.crunii.micro.service.qm.util.EnvUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;

import static com.crunii.micro.service.qm.api.constants.QmConstants.*;

/**
 * @author wangchen create 2023/9/20 10:26
 */
@Slf4j
@Service
public class StudentAppServiceImpl implements StudentAppService {

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private YcqQmSelfCultivationRepo ycqQmSelfCultivationRepo;

    @Resource
    private QmFlowConfig qmFlowConfig;

    @Resource
    private YcqQmFlowRecordRepo ycqQmFlowRecordRepo;

    @Resource
    private YcqQmExertRepo ycqQmExertRepo;

    @Resource
    private YcqQmExertLinkRepo ycqQmExertLinkRepo;

    @Resource
    private YcqQmTeacherStuRepo ycqQmTeacherStuRepo;

    @Resource
    private CommonService commonService;

    @Resource
    private YcqQmGroupRepo ycqQmGroupRepo;

    private final StudentAppDao studentAppDao;

    public StudentAppServiceImpl(StudentAppDao studentAppDao) {
        this.studentAppDao = studentAppDao;
    }

    @Override
    public Boolean verifyIsBindTeacher(int loginId) {
        return studentAppDao.findTeacherIsExist(loginId);
    }

    @Override
    @InjectProperty
    public PageRsp<TeacherAppBaseVo> pageTeacherBase(PageReq<TeacherAppBaseDto> pageReq, Worker worker) {
        // 查询当前学员是否被拒绝2次
        Integer count = studentAppDao.findStuRejectCount(worker.getLoginId());
        if(count>=REJECT_MAX){
            return new PageRsp<>();
        }
        // 查询学员已申请的导师id
        List<Integer> ids = studentAppDao.findAuditWorkerId(worker.getLoginId());
        if(CollectionUtils.isEmpty(ids)){
            ids = new ArrayList<>();
        }
        // 查询当前学员的levelCode
        List<String> levelCodes = studentAppDao.findLevelCodes(worker.getLoginId());
        PageRsp<TeacherAppBaseVo> voPageRsp = studentAppDao.pageTeacherBase(pageReq,ids,levelCodes);
        if(CollectionUtils.isEmpty(levelCodes)){
            voPageRsp.setResult(null);
            voPageRsp.setTotalNum(0);
            return voPageRsp;
        }
        List<TeacherAppBaseVo> result = voPageRsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return voPageRsp;
        }
        // 查询导师是否可申请
        Integer isApply = studentAppDao.findStuIsApply(worker.getLoginId());
        // 查询学员已拒绝的导师id
        List<Integer> rejectUserIds = studentAppDao.findRejectTeacherId(worker.getLoginId());
        if(CollectionUtils.isEmpty(rejectUserIds)){
            rejectUserIds = new ArrayList<>();
        }
        Iterator<TeacherAppBaseVo> iterator = result.iterator();
        List<Integer> collect = result.stream().map(TeacherAppBaseVo::getWorkerId).collect(Collectors.toList());
        Map<Integer,String> levelNames = getWorkerLevelName(collect);
        while (iterator.hasNext()){
            TeacherAppBaseVo next = iterator.next();
            List<String> teacherField = studentAppDao.findTeacherField(next.getWorkerId());
            next.setCorrelationStateTrans(QmConstants.MAY_APPLY);
            next.setField(String.join(";",teacherField));
            // 设置头像
            if(StringUtils.hasText(next.getHeadPortraitPicKey())){
                next.setHeadPortraitPicKey(urlPrefix+next.getHeadPortraitPicKey());
            }
            // 不可调换顺序
            if(ids.contains(next.getWorkerId())){
                next.setCorrelationStateTrans(QmConstants.IN_REVIEW);
            }
            if(QmConstants.TEACHER_MAX_STU <= next.getStuNumber()){
                next.setCorrelationStateTrans(QmConstants.FULL_STRENGTH);
            }
            if(rejectUserIds.contains(next.getWorkerId())){
                next.setCorrelationStateTrans(QmConstants.REJECT);
            }
            next.setLevelName(levelNames.get(next.getWorkerId()));
            next.setStuMaxNumber(QmConstants.TEACHER_MAX_STU);
            next.setBinding(isApply <= 0);
        }
        return voPageRsp;
    }

    /**
     * 获取导师worker的级别名称
     */
    private Map<Integer, String> getWorkerLevelName(List<Integer> collect) {
        List<WorkerIdLeveName> list = studentAppDao.getWorkerLevelName(collect);
        Map<Integer, String> collect1 = new HashMap<>(16);
        if(!CollectionUtils.isEmpty(list)){
            collect1 = list.stream().collect(Collectors.toMap(WorkerIdLeveName::getWorkerId, WorkerIdLeveName::getLevelName));
        }
        return collect1;
    }

    @Override
    @InjectProperty
    public List<TeacherAppBaseVo> findMyTeacher(Worker worker) {
        return commonService.findMyTeacher(worker.getLoginId());
    }

    @Override
    public String saveSelfCultivation(SelfCultivationDto selfCultivationDto, Worker worker) {
        YcqQmSelfCultivation one = new YcqQmSelfCultivation();
        Date now = new Date();
        if(Objects.nonNull(selfCultivationDto.getSelfCultivationId())){
            one = ycqQmSelfCultivationRepo.getOne(selfCultivationDto.getSelfCultivationId());
        }else {
            one.setCreatorId(worker.getLoginId());
            one.setCreateTime(now);
        }
        if(selfCultivationDto.getCultivationType() == 1){
            throw new BusinessException("学习强国不可添加！");
        }
        one.setCompleteTime(selfCultivationDto.getCreateTime());
        one.setCultivationType(selfCultivationDto.getCultivationType());
        one.setBookName(selfCultivationDto.getBookName());
        one.setUpdateTime(now);
        one.setUpdaterId(worker.getLoginId());
        one.setExperience(selfCultivationDto.getExperience());
        one.setDeleted(0);
        one.setWorkerId(worker.getLoginId());
        ycqQmSelfCultivationRepo.saveAndFlush(one);
        return "保存成功";
    }

    @Override
    public String saveExert(ExertCreateDto exertCreateDto, Worker worker) {
        // 修改
        if(Objects.nonNull(exertCreateDto.getExertId())){
            modifyEnterExertTwo(exertCreateDto,worker);
        }else {
            // 新增
            YcqQmExert ycqQmExert = new YcqQmExert();
            ycqQmExert.setWorkerId(worker.getLoginId());
            ycqQmExert.setExertType(exertCreateDto.getExertType());
            ycqQmExert.setExperience(exertCreateDto.getExperience());
            ycqQmExert.setCompleteTime(exertCreateDto.getCreateTime());
            ycqQmExert.setCreatorId(worker.getLoginId());
            ycqQmExert.setCreateTime(new Date());
            ycqQmExert.setCoverNumber(exertCreateDto.getCoverNumber());
            ycqQmExert.setDeleted(0);
            YcqQmExert ycqQmExert1 = ycqQmExertRepo.saveAndFlush(ycqQmExert);
            List<ExertFileTypeDto> fileList = exertCreateDto.getFileList();
            List<YcqQmExertLink> list = new ArrayList<>();
            for (ExertFileTypeDto exertFileTypeDto : fileList) {
                YcqQmExertLink ycqQmExertLink1 = new YcqQmExertLink();
                ycqQmExertLink1.setExertId(ycqQmExert1.getExertId());
                ycqQmExertLink1.setFileId(exertFileTypeDto.getFileId());
                ycqQmExertLink1.setPicUrl(exertFileTypeDto.getPicUrl());
                ycqQmExertLink1.setFileKey(exertFileTypeDto.getFileKey());
                ycqQmExertLink1.setCreatorId(worker.getLoginId());
                ycqQmExertLink1.setCreateTime(new Date());
                ycqQmExertLink1.setDeleted(0);
                list.add(ycqQmExertLink1);
            }
            if(!CollectionUtils.isEmpty(list)){
                ycqQmExertLinkRepo.saveAllAndFlush(list);
            }
        }
        return "操作成功";
    }

    @Override
    public String saveRecommend(RecommendCreateDto recommendCreateDto, Worker worker) {

        YcqQmGroup ycqQmGroup = new YcqQmGroup();
        if(Objects.nonNull(recommendCreateDto.getGroupId())){
            ycqQmGroup = ycqQmGroupRepo.getOne(recommendCreateDto.getGroupId());
        }else {
            ycqQmGroup.setCreatorId(worker.getLoginId());
            ycqQmGroup.setCreateTime(new Date());
        }
        ycqQmGroup.setCompleteTime(recommendCreateDto.getCreateTime());
        ycqQmGroup.setRecommendType(recommendCreateDto.getRecommendType());
        ycqQmGroup.setWorkerId(worker.getLoginId());
        ycqQmGroup.setRecommendReason(recommendCreateDto.getRecommendReason());
        ycqQmGroup.setUpdaterId(worker.getLoginId());
        ycqQmGroup.setUpdateTime(new Date());
        ycqQmGroup.setDeleted(0);

        ycqQmGroupRepo.saveAndFlush(ycqQmGroup);
        return "保存成功";
    }

    @Override
    public String getAdvantage(Worker worker) {
        return commonService.getAdvantage(worker.getLoginId());
    }

    @Override
    public SelfCultivationVo getSelfCultivation(TrackingCultivationDto trackingCultivationDto, Worker worker) {
        trackingCultivationDto.setWorkerId(worker.getLoginId());
        return commonService.getSelfCultivation(trackingCultivationDto,worker);
    }

    @Override
    @InjectProperty
    public PageRsp<GrowthVo> pageSelfGrowth(PageReq<TrackingCultivationDto> pageReq, Worker worker) {
        TrackingCultivationDto query = pageReq.getQuery();
        GroupCultivationDto groupCultivationDto = new GroupCultivationDto();
        groupCultivationDto.setWorkerId(worker.getLoginId());
        groupCultivationDto.setStartTime(query.getStartTime());
        groupCultivationDto.setEndTime(query.getEndTime());

        PageReq<GroupCultivationDto> req = new PageReq<>();
        req.setQuery(groupCultivationDto);
        req.setPageSize(pageReq.getPageSize());
        req.setPageNo(pageReq.getPageNo());
        req.setOrderProps(pageReq.getOrderProps());
        return commonService.pageGrowth(req);
    }

    @Override
    @InjectProperty
    public PageRsp<ExertVo> pageExert(PageReq<TrackingCultivationDto> pageReq, Worker worker) {
        TrackingCultivationDto query = pageReq.getQuery();
        ExertDto exertDto = new ExertDto();
        exertDto.setWorkerId(worker.getLoginId());
        exertDto.setStartTime(query.getStartTime());
        exertDto.setEndTime(query.getEndTime());

        PageReq<ExertDto> req = new PageReq<>();
        req.setQuery(exertDto);
        req.setPageSize(pageReq.getPageSize());
        req.setPageNo(pageReq.getPageNo());
        req.setOrderProps(pageReq.getOrderProps());
        return commonService.pageExert(req);
    }

    @Override
    public PageRsp<TeacherTraineeVo> studentAppService(PageReq<TrackingCultivationDto> pageReq, Worker worker) {
        pageReq.getQuery().setWorkerId(worker.getLoginId());
        return commonService.pageStuTraineeExpand(pageReq);
    }

    @Override
    @InjectProperty
    public List<VolunteerVo> getVolunteer(Worker worker) {
        StuHonorDto stuHonorDto = new StuHonorDto();
        stuHonorDto.setWorkerId(worker.getLoginId());
        return commonService.getVolunteer(stuHonorDto);
    }

    @Override
    public String delSelfCultivation(Long selfCultivationId,Worker worker) {
        YcqQmSelfCultivation one = ycqQmSelfCultivationRepo.getOne(selfCultivationId);
        if(Objects.nonNull(one)){
            if(!one.getWorkerId().equals(worker.getLoginId())){
                throw new BusinessException("非本人操作");
            }
            one.setDeleted(1);
            ycqQmSelfCultivationRepo.saveAndFlush(one);
        }
        return "删除成功";
    }

    @Override
    public String delExert(Long exertId, Worker worker) {
        YcqQmExert one = ycqQmExertRepo.getOne(exertId);
        if(Objects.nonNull(one)){
            if(!one.getWorkerId().equals(worker.getLoginId())){
                throw new BusinessException("非本人操作");
            }
            one.setDeleted(1);
            ycqQmExertRepo.saveAndFlush(one);
            // 查询发挥下所有的视频图片
            List<YcqQmExertLink> byExertIdAndDeleted = ycqQmExertLinkRepo.findByExertIdAndDeleted(one.getExertId(), 0);
            studentAppDao.delExertLink(one.getExertId());
            // 删除视频
            List<String> videoDtoList = byExertIdAndDeleted.stream().map(YcqQmExertLink::getFileId).filter(StringUtils::hasText).collect(Collectors.toList());
            if(!CollectionUtils.isEmpty(videoDtoList)){
                studentAppDao.updateVideo(videoDtoList);
            }
            // 删除图片
        }
        return "删除成功";
    }


    @Override
    public String delSelfGrowth(Long groupId, Worker worker) {
        YcqQmGroup one = ycqQmGroupRepo.getOne(groupId);
        if(Objects.nonNull(one)){
            if(!one.getWorkerId().equals(worker.getLoginId())){
                throw new BusinessException("非本人操作");
            }
            one.setDeleted(1);
            ycqQmGroupRepo.saveAndFlush(one);
        }
        return "删除成功";
    }

    @Override
    public HonorFieldVo getTeacherExpand(int loginId) {
        HonorFieldVo honorFieldVo = new HonorFieldVo();
        // 获取荣誉
        List<WorkerHonorVo> honorVos = studentAppDao.findTeacherHonor(loginId);
        honorFieldVo.setHonorVos(honorVos);
        // 获取擅长领域
        List<String> filed = studentAppDao.findTeacherField(loginId);
        String join = String.join(";", filed);
        honorFieldVo.setField(join);
        return honorFieldVo;
    }

    @Override
    public String applyTeacher(Integer teacherId, Worker stuId) {
        // 判断是否可申请导师
        Integer isApply = studentAppDao.findStuIsApply(stuId.getLoginId());
        if(isApply >0){
            throw new BusinessException("已申请过导师");
        }
        if(teacherId.equals(stuId.getLoginId())){
            throw new BusinessException("申请有误，请检查当前用户是否为导师");
        }
        YcqQmTeacherStu ycqQmTeacherStu = studentAppDao.findTeacherStu(teacherId,stuId);
        if(Objects.isNull(ycqQmTeacherStu)){
            ycqQmTeacherStu = new YcqQmTeacherStu();
            Long stuClassesId = studentAppDao.findStuClassesId(stuId.getLoginId());
            ycqQmTeacherStu.setStuClassesId(stuClassesId);
            ycqQmTeacherStu.setCreatorId(stuId.getLoginId());
            ycqQmTeacherStu.setCreateTime(new Date());
        }
        ycqQmTeacherStu.setTeachWorkerId(teacherId);
        ycqQmTeacherStu.setStuWorkerId(stuId.getLoginId());
        ycqQmTeacherStu.setCorrelationState(0);
        ycqQmTeacherStu.setChooseState(0);
        ycqQmTeacherStu.setBinding(1);
        ycqQmTeacherStu.setUpdaterId(stuId.getLoginId());
        ycqQmTeacherStu.setUpdateTime(new Date());
        ycqQmTeacherStu.setCorrelationState(0);
        ycqQmTeacherStu.setDeleted(0);
        YcqQmTeacherStu ycqQmTeacherStu1 = ycqQmTeacherStuRepo.saveAndFlush(ycqQmTeacherStu);
        if(EnvUtil.getEnvIsOnline()){
//            createFlow(stuId, ycqQmTeacherStu1.getTutorTraineeId(),teacherId);
        }
        return "申请成功";
    }

    /**
     * 创建工作流
     */
/*    private void createFlow(Worker stuId, Long tutorTraineeId,Integer teacherId) {
        log.info("开始执行申请流程---");
        FlowIdProperty stuAudit = qmFlowConfig.getStuAudit();
        FlowStartDTO flowStartDTO = new FlowStartDTO();
        flowStartDTO.setFlowDeployId(stuAudit.getDeployId());
        flowStartDTO.setFlowModuleId(stuAudit.getModuleId());
        flowStartDTO.setBizId(String.valueOf(tutorTraineeId));
        flowStartDTO.setFlowTitle(STU_AUDIT_TITLE);
        // 设置下一个流程有权限的人
        FlowTaskAssigennDTO flowTaskAssigennDTO = new FlowTaskAssigennDTO();
        List<IdNameDTO> idNameDTOS = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        idNameDTO.setId((long) stuId.getLoginId());
        idNameDTOS.add(idNameDTO);
        flowTaskAssigennDTO.setAssigennUser(idNameDTOS);
        flowStartDTO.setAddedTaskAssigenn(flowTaskAssigennDTO);
        // 设置启动人
        FlowStarterInfoDTO flowStarterInfoDTO = new FlowStarterInfoDTO();
        flowStarterInfoDTO.setUserId((long) stuId.getLoginId());
        flowStarterInfoDTO.setUserName(stuId.getWorkerName());
        String orgId = stuId.getOrgId();
        flowStarterInfoDTO.setOrgId(Objects.nonNull(stuId.getOrgId())?Long.valueOf(orgId):null);
        flowStarterInfoDTO.setOrgName(stuId.getOrgName());
        flowStartDTO.setStarter(flowStarterInfoDTO);
        CommonResult<CruniiStartProcessResult> result;
        try {
            result = flowInstanceApi.startProcess(flowStartDTO);
        } catch (Exception e) {
            log.info("远程调用流程执行失败@！参数-》{}", JsonUtil.getJsonString(flowStartDTO));
            throw new BusinessException("执行流程调用失败！");
        }
        if(!result.isSuccess()){
            throw new BusinessException("执行流程执行失败！");
        }
        CruniiStartProcessResult data = result.getData();
        String flowInstanceId = data.getFlowInstanceId();
        String nodeInstanceId = data.getActiveTaskInstance().getNodeInstanceId();
        log.info("执行完成开始流程 flowInstanceId->{},nodeInstanceId->{}",flowInstanceId,nodeInstanceId);
        // 导师审核
        FlowCommitDTO flowCommitDTO = new FlowCommitDTO();
        FlowCommitorInfoDTO flowCommitorInfoDTO = new FlowCommitorInfoDTO();
        flowCommitDTO.setFlowInstanceId(flowInstanceId);
        flowCommitDTO.setTaskInstanceId(nodeInstanceId);
        flowCommitDTO.setProcessResult(StuAuditFlowEnum.getAuditDesc(STU_AUDIT_TEACHER_AUDIT));
        // 设置下一个节点人
        FlowTaskAssigennDTO flowTaskAssigennDTO1 = new FlowTaskAssigennDTO();
        List<IdNameDTO> idNameDTOS1 = new ArrayList<>();
        IdNameDTO idNameDTO1 = new IdNameDTO();
        idNameDTO1.setId(Long.valueOf(teacherId));
        idNameDTOS1.add(idNameDTO1);
        flowTaskAssigennDTO1.setAssigennUser(idNameDTOS1);
        flowCommitDTO.setAddedTaskAssigenn(flowTaskAssigennDTO1);
        // 设置启动人
        flowCommitorInfoDTO.setUserId((long) stuId.getLoginId());
        flowCommitorInfoDTO.setUserName(stuId.getWorkerName());
        flowCommitorInfoDTO.setOrgId(Objects.nonNull(stuId.getOrgId())?Long.valueOf(orgId):null);
        flowCommitorInfoDTO.setOrgName(stuId.getOrgName());
        flowCommitDTO.setCommitor(flowCommitorInfoDTO);
        CommonResult<CruniiCommitTaskResult> resultCommonResult = flowInstanceApi.commitProcess(flowCommitDTO);
        log.info("执行完成提交流程 resultCommonResult=>{}，执行参数{}",JSONUtil.toJsonStr(resultCommonResult),JSONUtil.toJsonStr(flowCommitDTO));
        String nodeInstanceId1 = resultCommonResult.getData().getActiveTaskInstance().getNodeInstanceId();
        // 用于查询流程状态
        YcqQmFlowRecord ycqQmFlowRecord = ycqQmFlowRecordRepo.findByServiceId(tutorTraineeId);
        if(Objects.isNull(ycqQmFlowRecord)){
            ycqQmFlowRecord = new YcqQmFlowRecord();
        }
        ycqQmFlowRecord.setServiceId(tutorTraineeId);
        ycqQmFlowRecord.setFlowInstanceId(flowInstanceId);
        ycqQmFlowRecord.setCreatorId(stuId.getLoginId());
        ycqQmFlowRecord.setCreateTime(new Date());
        ycqQmFlowRecord.setNodeInstanceId(nodeInstanceId1);
        ycqQmFlowRecordRepo.saveAndFlush(ycqQmFlowRecord);
    }*/

    @Override
    public String modifyLearIntegral(LearIntegral learIntegral, Worker worker) {
        YcqQmSelfCultivation one = ycqQmSelfCultivationRepo.getOne(learIntegral.getSelfCultivationId());
        one.setIntegral(learIntegral.getIntegral());
        one.setUpdateTime(new Date());
        one.setUpdaterId(worker.getLoginId());
        ycqQmSelfCultivationRepo.saveAndFlush(one);
        return "修改成功";
    }

    @Override
    public List<GrowthPartyVo> findParty(Worker worker) {
        TrackingCultivationDto trackingCultivationDto = new TrackingCultivationDto();
        trackingCultivationDto.setWorkerId(worker.getLoginId());
        return commonService.findParty(trackingCultivationDto);
    }

    @Override
    public StudentAuditVo findStuAuditState(Worker worker) {
        // 查询是否被拒绝2次
        Integer stuRejectCount = studentAppDao.findStuRejectCount(worker.getLoginId());
        if(stuRejectCount>=REJECT_MAX){
            StudentAuditVo studentAuditVo = new StudentAuditVo();
            StudentAuditEnum rejectTwo = StudentAuditEnum.getRejectTwo();
            studentAuditVo.setAuditState(rejectTwo.getAuditState());
            studentAuditVo.setStateName(rejectTwo.getAuditTrans());
            return studentAuditVo;
        }
        StudentAuditVo studentAuditVo = studentAppDao.findStuAuditState(worker.getLoginId());
        if (Objects.isNull(studentAuditVo)){
            studentAuditVo = studentAppDao.findIsBinging(worker.getLoginId());
            if(Objects.isNull(studentAuditVo)){
                studentAuditVo = new StudentAuditVo();
            }else {
                studentAuditVo.setAuditState(StudentAuditEnum.BINDING.getAuditState());
            }
        }
        // 判断当前用户申请的导师是否满员 满员则删除申请记录
        if(USER_CORRELATION_STATE_0.equals(studentAuditVo.getAuditState())){
            Integer number = studentAppDao.findTeacherStuCount(studentAuditVo.getWorkerId());
            // 如果已绑定5个学员，删除纪律，返回已满员提醒用户
            if(number >= TEACHER_MAX_STU){
                Long tutorTraineeId = studentAuditVo.getTutorTraineeId();
                studentAppDao.delAuditRecord(tutorTraineeId);
                StudentAuditEnum auditEnum = StudentAuditEnum.getFullTeacher();
                studentAuditVo.setAuditState(auditEnum.getAuditState());
                studentAuditVo.setStateName(auditEnum.getAuditTrans());
            }
        }
        // 如是已移除状态，查询更新人
        if(StudentAuditEnum.BINDING.getAuditState().equals(studentAuditVo.getAuditState())){
            String workerName = studentAppDao.findUpdateWorkerName(worker.getLoginId());
            studentAuditVo.setWorkerName(workerName);
        }
        StudentAuditEnum auditTrans = StudentAuditEnum.getAuditTrans(studentAuditVo.getAuditState());
        studentAuditVo.setAuditState(auditTrans.getAuditState());
        studentAuditVo.setStateName(auditTrans.getAuditTrans());
        return studentAuditVo;
    }

    /**
     * 修改作用发挥
     * @param exertCreateDto 参数
     * @param worker worker
     */
    private void modifyEnterExertTwo(ExertCreateDto exertCreateDto, Worker worker) {
        // 查询作用发挥以及作用发挥文件信息
        YcqQmExert one = ycqQmExertRepo.getOne(exertCreateDto.getExertId());
        List<YcqQmExertLink> byExertIdAndDeleted = ycqQmExertLinkRepo.findByExertIdAndDeleted(one.getExertId(), 0);
        one.setCompleteTime(exertCreateDto.getCreateTime());
        one.setCoverNumber(exertCreateDto.getCoverNumber());
        one.setExertType(exertCreateDto.getExertType());
        one.setExperience(exertCreateDto.getExperience());
        ycqQmExertRepo.saveAndFlush(one);
        // 更新文件信息
        List<ExertFileTypeDto> fileList = exertCreateDto.getFileList();
        List<YcqQmExertLink> list = new ArrayList<>();
        // 需删除的视频/图片 url

        // 视频比对
        Set<String> videoDelIds = byExertIdAndDeleted.stream().map(YcqQmExertLink::getFileId).filter(StringUtils::hasText).collect(Collectors.toSet());
        List<ExertFileTypeDto> videoDtoList = fileList.stream().filter(x -> StringUtils.hasText(x.getFileId())).collect(Collectors.toList());
        for (ExertFileTypeDto exertFileTypeDto : videoDtoList) {
            if(videoDelIds.contains(exertFileTypeDto.getFileId())){
                videoDelIds.removeIf(x->x.equals(exertFileTypeDto.getFileId()));
            }
        }
        // 图片对比
        Set<String> fileKeys = byExertIdAndDeleted.stream().map(YcqQmExertLink::getFileKey).filter(StringUtils::hasText).collect(Collectors.toSet());
        List<ExertFileTypeDto> picDtoList = fileList.stream().filter(x -> StringUtils.hasText(x.getFileKey())).collect(Collectors.toList());
        for (ExertFileTypeDto exertFileTypeDto : picDtoList) {
            if(fileKeys.contains(exertFileTypeDto.getFileKey())){
                fileKeys.removeIf(x->x.equals(exertFileTypeDto.getFileKey()));
            }
        }

        studentAppDao.delExertLink(one.getExertId());
        for (ExertFileTypeDto exertFileTypeDto : fileList) {
            YcqQmExertLink ycqQmExertLink1 = new YcqQmExertLink();
            ycqQmExertLink1.setExertId(one.getExertId());
            ycqQmExertLink1.setFileId(exertFileTypeDto.getFileId());
            ycqQmExertLink1.setPicUrl(exertFileTypeDto.getPicUrl());
            ycqQmExertLink1.setFileKey(exertFileTypeDto.getFileKey());
            ycqQmExertLink1.setCreatorId(worker.getLoginId());
            ycqQmExertLink1.setCreateTime(new Date());
            ycqQmExertLink1.setDeleted(0);
            list.add(ycqQmExertLink1);
        }
        if(!CollectionUtils.isEmpty(list)){
          ycqQmExertLinkRepo.saveAllAndFlush(list);
        }
        // video 逻辑删除 图片未删 pic
        if(!CollectionUtils.isEmpty(videoDelIds)){
            studentAppDao.updateVideo(videoDelIds);
        }
    }
}