package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.document.YcqEventReport;
import com.crunii.micro.domain.entity.main.document.YcqEventReportAnnex;
import com.crunii.micro.domain.entity.main.document.YcqEventReportCover;
import com.crunii.micro.domain.entity.main.document.YcqPromiseYouth;
import com.crunii.micro.domain.repo.main.announcement.YcqEventReportAnnexRepo;
import com.crunii.micro.domain.repo.main.announcement.YcqEventReportCoverRepo;
import com.crunii.micro.domain.repo.main.announcement.YcqEventReportRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.document.*;
import com.crunii.micro.service.qzh.jdbc.YcqEventReportDao;
import com.crunii.micro.service.qzh.service.YcqEventReportService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.WORKER_NOT_EXISTS;

/**
 * @program: microservice
 * @description: 活动报道impl
 * @author: xiang xin
 * @create: 2024-01-15 15:44
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqEventReportServiceImpl implements YcqEventReportService {
    private final YcqEventReportRepo ycqEventReportRepo;
    //附件
    private final YcqEventReportAnnexRepo ycqEventReportAnnexRepo;
    //图片
    private final YcqEventReportCoverRepo ycqEventReportCoverRepo;
    @Autowired
    private YcqEventReportDao ycqEventReportDao;

    /**
     * @Description: 创建
     * @Param: [dto]
     * @return: com.crunii.micro.common.dto.Result<java.lang.Void>
     * @Author: xiang xin
     * @Date: 2024/1/15
     */
    @Override
    public Result<Void> createEventReport(CreateEventReportDTO dto) {
        if (null == dto.getCoverImageFile()) {
            throw new BusinessException("封面图不能为空");
        }
        if (null == dto.getAnnexDtoList()) {
            throw new BusinessException("报道附件不能为空");
        }
        Worker worker = getWorker();
        if (null == worker) {
            throw new BusinessException("该账号异常,请联系系统管理员!");
        }
        Snowflake snowflake = new Snowflake();
        //丰富数据
        YcqEventReport ycqEventReport = new YcqEventReport();
        ycqEventReport.setId(snowflake.nextId());
        ycqEventReport.setActivityId(dto.getActivityId());
        //活动报道正文
        ycqEventReport.setReportText(dto.getReportText());
        //活动报道状态 默认未发布
        ycqEventReport.setReportType(1);
        //活动报道标题
        if (dto.getReportTitle().length() <= 50) {
            ycqEventReport.setReportTitle(dto.getReportTitle());
        } else {
            return Result.ofFail("活动标题不能超过50个字符!");
        }
        //发布人
        ycqEventReport.setCreatedBy(Long.valueOf(worker.getLoginId()));
        //发布时间YCQ_QZH_DOCUMENT
        ycqEventReport.setCreatedTime(new Date());
        ycqEventReportRepo.save(ycqEventReport);
        //存储附件和图片
        if (null != dto.getAnnexDtoList() && dto.getAnnexDtoList().size() > 0) {
            saveAnnex(dto.getAnnexDtoList(), ycqEventReport.getId());
        }
        if (null != dto.getCoverImageFile() && dto.getCoverImageFile().size() > 0) {
            saveCover(dto.getCoverImageFile(), ycqEventReport.getId());
        }
        return new Result<>(true);
    }

    /**
     * @Description: 分页
     * @Param: [dto]
     * @return: com.crunii.micro.common.dto.Result<com.crunii.micro.common.dto.PageRsp < com.crunii.micro.service.qzh.api.domain.dto.document.PageEventReportRespDTO>>
     * @Author: xiang xin
     * @Date: 2024/1/15
     */
    @Override
    public Result<PageRsp<PageEventReportRespDTO>> pageEventReport(PageReq<PageEventReportReqDTO> dto) {
        Worker worker = getWorker();
        PageRsp<PageEventReportRespDTO> dtoPageRsp = ycqEventReportDao.pageEventReport(dto, worker);
        for (PageEventReportRespDTO pageEventReportRespDTO : dtoPageRsp.getResult()) {
//            //封面
//            List<YcqEventReportCover> covers = ycqEventReportCoverRepo.findOneByArticleId(pageEventReportRespDTO.getId(), 1);
//            //附件
//            List<YcqEventReportAnnex> annexes = ycqEventReportAnnexRepo.findOneByArticleId(pageEventReportRespDTO.getId(), 1);
//            pageEventReportRespDTO.setCoverImageFile(covers);
//            pageEventReportRespDTO.setAnnexDtoList(annexes);
            //按钮展示
            if (pageEventReportRespDTO.getCreatedId().equals(Long.valueOf(worker.getLoginId()))) {
                pageEventReportRespDTO.setIsShow(1);
            } else {
                pageEventReportRespDTO.setIsShow(0);
            }
        }
        return new Result<>(Boolean.TRUE, dtoPageRsp);
    }

    /**
     * @Description: 修改
     * @Param: [dto]
     * @return: com.crunii.micro.common.dto.Result
     * @Author: xiang xin
     * @Date: 2024/1/15
     */
    @Override
    public Result updateEventReport(CreateEventReportDTO dto) {
        if (null == dto.getCoverImageFile()) {
            throw new BusinessException("封面图不能为空");
        }
        if (null == dto.getAnnexDtoList()) {
            throw new BusinessException("报道附件不能为空");
        }
        Worker worker = getWorker();
        if (null == worker) {
            throw new BusinessException("该账号异常,请联系系统管理员!");
        }
        Optional<YcqEventReport> byId = ycqEventReportRepo.findById(dto.getId());
        if (byId.isPresent()) {
            YcqEventReport ycqEventReport = byId.get();
            if (!ycqEventReport.getCreatedBy().equals(Long.valueOf(worker.getLoginId()))) {
                return Result.ofFail("该账号没有处理该数据的权限,请核对!");
            }
            ycqEventReport.setActivityId(dto.getActivityId());
            //活动报道正文
            ycqEventReport.setReportText(dto.getReportText());
            //活动报道状态
            ycqEventReport.setReportType(ycqEventReport.getReportType());
            //活动报道标题
            if (dto.getReportTitle().length() <= 50) {
                ycqEventReport.setReportTitle(dto.getReportTitle());
            } else {
                return Result.ofFail("活动标题不能超过50个字符!");
            }
            //发布人
            ycqEventReport.setUpdatedBy(Long.valueOf(worker.getLoginId()));
            //发布时间
            ycqEventReport.setUpdatedTime(new Date());
            ycqEventReportRepo.save(ycqEventReport);
            //先删除之前提交的图片和附件
            ycqEventReportDao.delectAll(ycqEventReport.getId());
            //存储附件和图片
            if (null != dto.getAnnexDtoList() && dto.getAnnexDtoList().size() > 0) {
                saveAnnex(dto.getAnnexDtoList(), ycqEventReport.getId());
            }
            if (null != dto.getCoverImageFile() && dto.getCoverImageFile().size() > 0) {
                saveCover(dto.getCoverImageFile(), ycqEventReport.getId());
            }
        }
        return new Result<>(true);
    }

    /**
     * @Description: 查看详细
     * @Param: [dto]
     * @return: com.crunii.micro.common.dto.Result<com.crunii.micro.service.qzh.api.domain.dto.document.PageEventReportRespDTO>
     * @Author: xiang xin
     * @Date: 2024/1/15
     */
    @Override
    public Result<PageEventReportRespDTO> doEventReport(YcqEventReportDTO dto) {
        Worker worker = getWorker();
        PageEventReportRespDTO dtoPageRsp = ycqEventReportDao.doEventReport(dto, worker);
        //封面
        List<YcqEventReportCover> covers = ycqEventReportCoverRepo.findOneByArticleId(dtoPageRsp.getId(), 1);
        //附件
        List<YcqEventReportAnnex> annexes = ycqEventReportAnnexRepo.findOneByArticleId(dtoPageRsp.getId(), 1);
        List<CoverImageDto> coverImageFile = new ArrayList<>();
        List<AnnexDto> annexDtoList = new ArrayList<>();
        if (covers.size() > 0) {
            CoverImageDto coverImageDto = new CoverImageDto();
            YcqEventReportCover ycqEventReportCover = covers.get(0);
            BeanUtils.copyProperties(ycqEventReportCover, coverImageDto);
            coverImageFile.add(coverImageDto);
        }
        if (annexes.size() > 0) {
            AnnexDto annexDto = new AnnexDto();
            YcqEventReportAnnex ycqEventReportAnnex = annexes.get(0);
            BeanUtils.copyProperties(ycqEventReportAnnex, annexDto);
            annexDtoList.add(annexDto);
        }
        dtoPageRsp.setCoverImageFile(coverImageFile);
        dtoPageRsp.setAnnexDtoList(annexDtoList);
        return new Result<>(Boolean.TRUE, dtoPageRsp);
    }

    /**
     * @Description: 删除
     * @Param: [dto]
     * @return: com.crunii.micro.common.dto.Result
     * @Author: xiang xin
     * @Date: 2024/1/15
     */
    @Override
    public Result deleteYcqQzhActivity(YcqEventReportDTO dto) {
        Worker worker = getWorker();
        Optional<YcqEventReport> byId = ycqEventReportRepo.findById(dto.getId());
        if (byId.isPresent()) {
            YcqEventReport ycqEventReport = byId.get();
            if (!ycqEventReport.getCreatedBy().equals(Long.valueOf(worker.getLoginId()))) {
                return Result.ofFail("该账号没有处理该数据的权限,请核对!");
            }
            ycqEventReport.setDelFlag(1);
            ycqEventReportRepo.save(ycqEventReport);
        }
        return new Result<>(true);
    }

    /**
     * @Description: app端分页查询 （已发布）
     * @Param: [dto]
     * @return: com.crunii.micro.common.dto.Result<com.crunii.micro.common.dto.PageRsp < com.crunii.micro.service.qzh.api.domain.dto.document.PageEventReportRespDTO>>
     * @Author: xiang xin
     * @Date: 2024/1/15
     */
    @Override
    public Result<PageRsp<PageEventReportRespDTO>> pageEventReportApp(PageReq<PageEventReportReqDTO> dto) {
        Worker worker = getWorker();
        PageRsp<PageEventReportRespDTO> dtoPageRsp = ycqEventReportDao.pageEventReportApp(dto, worker);
        for (PageEventReportRespDTO pageEventReportRespDTO : dtoPageRsp.getResult()) {
            //封面
            List<YcqEventReportCover> covers = ycqEventReportCoverRepo.findOneByArticleId(pageEventReportRespDTO.getId(), 1);
            List<CoverImageDto> coverImageFile = new ArrayList<>();
            if (covers.size() > 0) {
                CoverImageDto coverImageDto = new CoverImageDto();
                YcqEventReportCover ycqEventReportCover = covers.get(0);
                BeanUtils.copyProperties(ycqEventReportCover, coverImageDto);
                coverImageFile.add(coverImageDto);
            }
            pageEventReportRespDTO.setCoverImageFile(coverImageFile);
        }
        return new Result<>(Boolean.TRUE, dtoPageRsp);
    }

    /**
     * @Description: 1发布 2下架 /1未发布、2已发布、3已下架
     * @Param: [dto]
     * @return: com.crunii.micro.common.dto.Result
     * @Author: xiang xin
     * @Date: 2024/1/16
     */
    @Override
    public Result updateType(YcqEventReportDTO dto) {
        Worker worker = getWorker();
        Optional<YcqEventReport> byId = ycqEventReportRepo.findById(dto.getId());
        if (byId.isPresent()) {
            YcqEventReport ycqEventReport = byId.get();
            if (!ycqEventReport.getCreatedBy().equals(Long.valueOf(worker.getLoginId()))) {
                return Result.ofFail("该账号没有处理该数据的权限,请核对!");
            }
            //发布
            if (null != dto.getType() && 1 == dto.getType()) {
                ycqEventReport.setReportType(2);
            }
            //下架
            if (null != dto.getType() && 2 == dto.getType()) {
                ycqEventReport.setReportType(3);
            }
            ycqEventReportRepo.save(ycqEventReport);
        }
        return new Result(true);
    }

    private void saveAnnex(List<AnnexDto> subsidiaryFile, Long id) {
        //保存附件
        subsidiaryFile.forEach(s -> {
            YcqEventReportAnnex ycqQzhAnnex = doCrateConverter(s);
            ycqQzhAnnex.setArticleId(id);
            ycqQzhAnnex.setType(1);
            ycqEventReportAnnexRepo.save(ycqQzhAnnex);
        });
    }

    private void saveCover(List<CoverImageDto> coverImageFile, Long id) {
        //保存封面
        coverImageFile.forEach(s -> {
            YcqEventReportCover ycqQzhCover = doCrateConverter(s);
            ycqQzhCover.setArticleId(id);
            ycqQzhCover.setType(1);
            ycqEventReportCoverRepo.save(ycqQzhCover);
        });
    }

    //图片
    public static YcqEventReportAnnex doCrateConverter(AnnexDto annexDto) {
        if (annexDto == null) {
            return null;
        }
        YcqEventReportAnnex ycqQzhAnnex = new YcqEventReportAnnex();
        ycqQzhAnnex.setFileId(annexDto.getFileId());
        ycqQzhAnnex.setFileName(annexDto.getFileName());
        ycqQzhAnnex.setFileKey(annexDto.getFileKey());
        ycqQzhAnnex.setFileType(annexDto.getFileType());
        if (annexDto.getFileSize() != null) {
            ycqQzhAnnex.setFileSize(annexDto.getFileSize());
        }
        ycqQzhAnnex.setFileUrl(annexDto.getFileUrl());

        return ycqQzhAnnex;
    }

    //附件
    public static YcqEventReportCover doCrateConverter(CoverImageDto coverDto) {
        if (coverDto == null) {
            return null;
        }
        YcqEventReportCover ycqQzhCover = new YcqEventReportCover();
        ycqQzhCover.setFileId(coverDto.getFileId());
        ycqQzhCover.setFileName(coverDto.getFileName());
        ycqQzhCover.setFileKey(coverDto.getFileKey());
        ycqQzhCover.setFileType(coverDto.getFileType());
        if (coverDto.getFileSize() != null) {
            ycqQzhCover.setFileSize(Integer.parseInt(coverDto.getFileSize()));
        }
        ycqQzhCover.setFileUrl(coverDto.getFileUrl());
        ycqQzhCover.setPicHeight(coverDto.getPicHeight());
        ycqQzhCover.setPicWidth(coverDto.getPicWidth());

        return ycqQzhCover;
    }

    /**
     * 获取登录用户
     *
     * @return
     */
    private Worker getWorker() {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }

    /**
     * @Description: 1月16号加入公用青春之约认证判断
     * @Param: true 已认证 false 去认证
     * @return:
     * @Author: xiang xin
     * @Date: 2024/1/15
     */
    @Override
    public Boolean promiseYouth() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (null != worker) {
            List<YcqPromiseYouth> youths = ycqEventReportDao.promiseYouth(Long.valueOf(worker.getLoginId()));
            if (youths.size() > 0) {
                return true;
            }
        }
        return false;
    }
}
