package com.crunii.micro.service.record.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.obs.ObsServiceCodeDto;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.entity.main.YcqEventsTask;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.domain.repo.main.YcqEventsTaskRepo;
import com.crunii.micro.domain.repo.main.YcqObsFileRepo;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.service.record.constants.RecordConstants;
import com.crunii.micro.service.record.dto.*;
import com.crunii.micro.service.record.jdbc.RecordEventsDao;
import com.crunii.micro.service.record.service.RecordEventsService;
import jodd.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RMapCache;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;



import javax.annotation.Resource;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @author: LG
 * @date: 2023/6/25 17:17
 */
@Service
@Slf4j
@RefreshScope
public class RecordEventsServiceImpl implements RecordEventsService {

    @Resource
    private TsmWorkerRepo tsmWorkerRepo;

    @Resource
    private YcqEventsTaskRepo ycqEventsTaskRepo;

    @Resource
    private ObsService obsService;

    @Resource
    private RecordEventsDao recordEventsDao;

    @Value("${upload.url}")
    private String urlPrefix;

    @Value("${obs.s3.innerURL}")
    private String urlPrefixExcel;

    @Value("${record.daysCompletion}")
    private Integer daysCompletion;

    @Value("${record.promptStartDate}")
    private String promptStartDate;

    @Value("${record.presentationDeadline}")
    private String presentationDeadline;

    @Resource
    private TsmPropertyCache tsmPropertyCache;

    @Resource
    private RedissonClient redisson;

    @Resource
    private YcqObsFileRepo obsFileRepo;

    @Resource
    private TsmOrgRepo tsmOrgRepo;

    /**
     * 打卡天数最小天数单位
     */
    public static final double MIX_DAY = 0.5;

    /**
     * 交流谈心任务类别
     */
    public static final String INTERFLOW_TYPE = "2";

    @Override
    public Result<Object> updatePersonalInformation(Worker worker, ModifyPersonalInformationDto modifyPersonalInformationDto) {
        String storeCaptcha = removeCaptchaKeyId(modifyPersonalInformationDto.getCaptchaKey());
        if (StringUtils.hasText(storeCaptcha) && storeCaptcha.equalsIgnoreCase(modifyPersonalInformationDto.getCaptcha())) {
        } else {
            throw new BusinessException("验证码错误");
        }
        //拿到当前用户，修改信息
        TsmWorker user = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(),CommonConstants.STATE_VALID);
        user.setWorkerName(modifyPersonalInformationDto.getUserName());
        user.setRegistrationSystem(Integer.valueOf(modifyPersonalInformationDto.getRegistrationSystem()));
        if(StringUtils.hasText(modifyPersonalInformationDto.getLeagueCode())){
            user.setOrgId(tsmOrgRepo.findIdByCode(modifyPersonalInformationDto.getLeagueCode()).toString());
        }
        user.setIdentityCategory(Integer.valueOf(modifyPersonalInformationDto.getIdentityCategory()));
        if (StringUtil.isNotBlank(modifyPersonalInformationDto.getPosition())){
            user.setPosition(modifyPersonalInformationDto.getPosition());
        }
        if (StringUtil.isNotBlank(modifyPersonalInformationDto.getHeadPortraitPicKey())) {
            if(modifyPersonalInformationDto.getHeadPortraitPicKey().startsWith(urlPrefix)){
                String replace = modifyPersonalInformationDto.getHeadPortraitPicKey().replace(urlPrefix,"");
                user.setHeadPortraitPicKey(replace);
            }else{
                user.setHeadPortraitPicKey(modifyPersonalInformationDto.getHeadPortraitPicKey());
            }
        }else {
            user.setHeadPortraitPicKey(null);
        }
        tsmWorkerRepo.save(user);
        return Result.ofSuccess("修改成功");
    }

    @Override
    public Result<Object> saveEventTask(Worker worker, TaskPunchDto taskPunchDto) {
        if (!rangeInDefined(Double.parseDouble(taskPunchDto.getContactDays()),MIX_DAY,daysCompletion)){
            throw new   BusinessException("请输入有效联系天数");
        }
        /*String env = System.getProperty("spring.profiles.active");
        if("dev".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }*/

        YcqEventsTask ycqEventsTask = new YcqEventsTask();
        if (StringUtil.isNotBlank(taskPunchDto.getEventsId())){
            ycqEventsTask.setEventsId(Long.valueOf(taskPunchDto.getEventsId()));
        }
        //存纪实任务表
        ycqEventsTask.setTaskTime(taskPunchDto.getTaskTime());
        ycqEventsTask.setLongitude(taskPunchDto.getLongitude());
        ycqEventsTask.setLatitude(taskPunchDto.getLatitude());
        ycqEventsTask.setTransactionDescription(taskPunchDto.getTransactionDescription());
        ycqEventsTask.setTaskType(Integer.valueOf(taskPunchDto.getTaskType()));
        ycqEventsTask.setCoverPeople(Integer.valueOf(taskPunchDto.getCoverPeople()));
        ycqEventsTask.setContactDays(round(Double.parseDouble(taskPunchDto.getContactDays()),0.5));
        ycqEventsTask.setCreateTime(new Date());
        ycqEventsTask.setUploadWorkerId((long) worker.getLoginId());
        ycqEventsTask.setState(CommonConstants.STATE_VALID);
        ycqEventsTask.setPlaceName(taskPunchDto.getPlaceName());
        if(INTERFLOW_TYPE.equals(taskPunchDto.getTaskType())){
            if (StringUtil.isBlank(taskPunchDto.getCoverGroupPeople())){
                throw new BusinessException("请选覆盖团干部人次");
            }else {
                ycqEventsTask.setCoverGroupPeople(Integer.valueOf(taskPunchDto.getCoverGroupPeople()));
            }
        }
        YcqEventsTask save = ycqEventsTaskRepo.save(ycqEventsTask);

        if (StringUtil.isNotBlank(taskPunchDto.getEventsId())){
            //修改图片- set0之前存在 现在不存在的图片
            List<YcqObsFile> obsFiles = obsFileRepo.findByServiceCodeAndServiceIdAndState(
                    CommonConstants.OBS_FILE_RECORD_CODE, Long.valueOf(taskPunchDto.getEventsId()), CommonConstants.STATE_VALID);
            System.out.println(obsFiles.size());
            if(!obsFiles.isEmpty()){
                List<YcqObsFile> collect = obsFiles.stream().filter(o -> !taskPunchDto.getPicIds().contains(String.valueOf(o.getId()))).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(collect)){
                    collect.forEach(item -> item.setState(CommonConstants.STATE_INVALID));
                    obsFileRepo.saveAllAndFlush(collect);
                }

            }
        }

        //存图片表
        if (CollectionUtil.isNotEmpty(taskPunchDto.getPicIds())) {

            List<ObsServiceCodeDto> listPic = new ArrayList<>();
            for (String picId : taskPunchDto.getPicIds()) {
                ObsServiceCodeDto obsServiceCodeDto = new ObsServiceCodeDto();
                obsServiceCodeDto.setFileId(Long.valueOf(picId));
                obsServiceCodeDto.setServiceCode(CommonConstants.OBS_FILE_RECORD_CODE);
                obsServiceCodeDto.setServiceId(save.getEventsId());
                listPic.add(obsServiceCodeDto);
            }
            obsService.updateObsServiceCode(listPic);

        }
        return Result.ofSuccess("提交成功");
    }

    @Override
    @InjectProperty
    public PageRsp<TaskLookOverVo> taskLookOver(Worker worker, PageReq<Object> req){
        PageRsp<TaskLookOverVo> taskLookOverVoPageRsp = recordEventsDao.taskLookOver(worker, req);
        if (CollectionUtil.isEmpty(taskLookOverVoPageRsp.getResult())){
            return taskLookOverVoPageRsp;
        }
        // 封装图片url
        return setPicUrls(taskLookOverVoPageRsp,false);
    }

    private PageRsp<TaskLookOverVo> setPicUrls(PageRsp<TaskLookOverVo> taskLookOverVoPageRsp,boolean b) {
        for (TaskLookOverVo taskLookOverVo : taskLookOverVoPageRsp.getResult()) {
            if ( b){
                //交流谈心类型的覆盖人次相加
                /*if (2 == taskLookOverVo.getTaskType()){
                    taskLookOverVo.setCoverPeople(taskLookOverVo.getCoverPeople()+(taskLookOverVo.getCoverGroupPeople()==null?0:taskLookOverVo.getCoverGroupPeople()));
                }*/
            }
            beanFilInfo(taskLookOverVo);
        }
        return taskLookOverVoPageRsp;
    }

    private List<TaskLookOverVo> setPicUrls(List<TaskLookOverVo> taskLookOverVoPageRsp) {
        for (TaskLookOverVo taskLookOverVo : taskLookOverVoPageRsp) {
            List<ObsFileVo> obsFileVos = recordEventsDao.queryObsPic(Long.valueOf(taskLookOverVo.getEventsId()));
            if (CollectionUtil.isNotEmpty(obsFileVos)){
                for (ObsFileVo s : obsFileVos) {
                    String s1 = urlPrefixExcel + s.getFileKey();
                    s.setFileKey(s1);
                }
            }
            taskLookOverVo.setFileInfo(obsFileVos);
        }
        return taskLookOverVoPageRsp;
    }

    private void beanFilInfo(TaskLookOverVo taskLookOverVo) {
        List<ObsFileVo> obsFileVos = recordEventsDao.queryObsPic(Long.valueOf(taskLookOverVo.getEventsId()));
        if (CollectionUtil.isNotEmpty(obsFileVos)){
            for (ObsFileVo s : obsFileVos) {
                String s1 = urlPrefix + s.getFileKey();
                s.setFileKey(s1);
            }
        }
        taskLookOverVo.setFileInfo(obsFileVos);
    }

    @Override
    public DoneConditionVo doneCondition(Worker worker) {
        /*String env = System.getProperty("spring.profiles.active");
        if("dev".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }*/
        String year = String.valueOf(Calendar.getInstance().get(Calendar.YEAR));
        LocalDate starDay = LocalDate.parse(year+"-"+promptStartDate);
        LocalDate endDay = LocalDate.parse(year+"-"+presentationDeadline);
        LocalDate now = LocalDate.now();
        int DONE_DAY = daysCompletion;
        List<TaskLookOverVo> taskLookOverVos = recordEventsDao.taskLookOver(worker);
        double contactDays = 0;
        double daysOutstanding ;
        for (TaskLookOverVo taskLookOverVo : taskLookOverVos) {
            // 算已完成和未完成天数
            Double contactDay = taskLookOverVo.getContactDays();
            contactDays+=contactDay;
        }
        DoneConditionVo doneConditionVo = new DoneConditionVo();
        doneConditionVo.setDaysCompleted(contactDays);
        if (DONE_DAY>contactDays){
            daysOutstanding = DONE_DAY-contactDays;
        }else {
            daysOutstanding = 0;
        }
        doneConditionVo.setDaysOutstanding(daysOutstanding);
        doneConditionVo.setDaysTarget(DONE_DAY);
        if (daysOutstanding==0){
            doneConditionVo.setWhetherDone(true);
        }else {
            doneConditionVo.setWhetherDone(false);
        }
        //在提示范围日期为true，不在就为false
        boolean b = (now.isAfter(starDay) || now.isEqual(starDay)) && (now.isBefore(endDay) || now.isEqual(endDay));
        if (b){
            doneConditionVo.setWhetherHintDay(true);
        }else {
            doneConditionVo.setWhetherHintDay(false);
        }

        return doneConditionVo;
    }

    @Override
    @InjectProperty
    public PageRsp<TaskLookOverVo> statisticalList(PageReq<StatisticalListDto> req) {
        PageRsp<TaskLookOverVo> taskLookOverVoPageRsp = recordEventsDao.taskLookOver( req);
        if (CollectionUtil.isEmpty(taskLookOverVoPageRsp.getResult())){
            return taskLookOverVoPageRsp;
        }
        //封装图片url
        setPicUrls(taskLookOverVoPageRsp,true);
        //设置序号
        AtomicInteger index = new AtomicInteger(0);
        taskLookOverVoPageRsp.getResult().forEach(ta -> {
            ta.setSequenceNumber(index.incrementAndGet());
        });

        return taskLookOverVoPageRsp;
    }

    @Override
    public Result informationModificationCheck(Worker worker) {
        Result<Object> objectResult = new Result<>();
        //校验应填信息是否为空
        TsmWorker user = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(),CommonConstants.STATE_VALID);
        String workerName = user.getWorkerName();
        Integer registrationSystem = user.getRegistrationSystem();
        String position = user.getPosition();
        Integer identityCategory = user.getIdentityCategory();
        String orgId = user.getOrgId();
        if (StringUtil.isBlank(orgId) ||StringUtil.isBlank(workerName) || StringUtil.isBlank(position) ||  registrationSystem == null || identityCategory == null){
            objectResult.setData("0");
        }else {
            objectResult.setData("1");
        }
        return objectResult;
    }


    private List<TaskLookOverVo> queryExcelData(List<String> eventsIds){
        List<TaskLookOverVo> taskLookOverVos = recordEventsDao.taskLookOver(eventsIds);
        //封装图片url
        setPicUrls(taskLookOverVos);
        return taskLookOverVos;
    }

    @Override
    @InjectProperty
    public List<TaskLookOverExcel> excelOutPut(List<String> eventsIds ) throws MalformedURLException {
        List<TaskLookOverVo> taskLookOverVos = queryExcelData(eventsIds);
        List<TaskLookOverExcel> list = new ArrayList<>();
        for (TaskLookOverVo taskLookOverVo : taskLookOverVos) {
            TaskLookOverExcel taskLookOverExcel = new TaskLookOverExcel();
            BeanUtil.copyProperties(taskLookOverVo,taskLookOverExcel);
            if (2 == taskLookOverExcel.getTaskType()){
                taskLookOverExcel.setCoverPeopleYoung(taskLookOverExcel.getCoverPeople());
                taskLookOverExcel.setCoverPeople(null);
                }
            List<String> collect = taskLookOverVo.getFileInfo().stream().map(ObsFileVo::getFileKey).collect(Collectors.toList());
            List<URL> urls = new ArrayList<>();
            for (String s : collect) {
                URL url = new URL(s);
                urls.add(url);
            }
            taskLookOverExcel.setPicUrl(urls);
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
            taskLookOverExcel.setTaskTime(simpleDateFormat.format(taskLookOverVo.getTaskTime()));
            taskLookOverExcel.setContactDaysText(taskLookOverVo.getContactDays() + "天");

            list.add(taskLookOverExcel);
        }

        return list;
    }

    @Override
    public List<LeagueCodeVo>

    queryLeagueCode() {
        return recordEventsDao.queryLeagueCode();
    }

    @Override
    @InjectProperty
    public EchoPersonalVo echoPersonalInformation(Worker worker) {
        EchoPersonalVo echoPersonalVo = recordEventsDao.queryEchoPersonalInformation(worker.getLoginId());
        if(null != echoPersonalVo.getOrgId()){
            echoPersonalVo.setOrgId(tsmOrgRepo.findCodeById(echoPersonalVo.getOrgId()));
        }
        if (null != echoPersonalVo) {
            if (null != echoPersonalVo.getHeadPortraitPicKey()) {
                echoPersonalVo.setHeadPortraitPicKey(urlPrefix + echoPersonalVo.getHeadPortraitPicKey());
            }
            if (null != echoPersonalVo.getRegistrationSystem()) {
                if (echoPersonalVo.getRegistrationSystem() == CommonConstants.STATE_VALID) {
                    echoPersonalVo.setIdentityCategoryText(tsmPropertyCache.get("eventsTaskCountyIdentityType", String.valueOf(echoPersonalVo.getIdentityCategory())));
                }
                if (echoPersonalVo.getRegistrationSystem() == CommonConstants.MENU_TYPE_BUTTON) {
                    echoPersonalVo.setIdentityCategoryText(tsmPropertyCache.get("eventsTaskCityIdentityCategory", String.valueOf(echoPersonalVo.getIdentityCategory())));
                }
            }
        }

        return echoPersonalVo;
    }

    public String removeCaptchaKeyId(String key) {
        RMapCache<String, String> captchaMap = redisson.getMapCache(RedisKeys.CAPTCHA_MAP_KEY);
        String code = captchaMap.remove(key);
        return code;
    }

    public static boolean rangeInDefined(double current, double min, double max) {
        return Math.max(min, current) == Math.min(current, max);
    }

    static double round( double num, double multipleOf) {
        if (num%multipleOf == 0){
            return num;
        }
        return Math.floor(num / multipleOf) * multipleOf;

    }

}

