package com.crunii.micro.service.report.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.report.dto.cockpit.map.grid.CockpitGridMapDataReqVO;
import com.crunii.micro.service.report.dto.cockpit.map.grid.CockpitGridMapDataRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.grid.CockpitGridMapYouthNumRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.grid.GridMapDataRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionAreaSumRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionDataReqVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionDataRespVO;
import com.crunii.micro.service.report.jdbc.CockpitMapDAO;
import com.crunii.micro.service.report.service.CockpitMapService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

import static com.crunii.micro.service.report.constants.CockpitMapErrorCodeConstants.QUERY_PARAM_MISS;

@Service
public class CockpitMapServiceImpl implements CockpitMapService {

    @Resource
    private CockpitMapDAO cockpitMapDAO;

    /**
     * 查询驾驶舱栅格图列表
     *
     * @param reqVO 请求参数
     * @return 驾驶舱栅格图列表
     */
    @Override
    public Result<List<GridMapDataRespVO>> listCockpitGridMapData(CockpitGridMapDataReqVO reqVO) {
        if (reqVO.getLeftDownLng() == null && reqVO.getLeftDownLat() == null) {
            reqVO.setLeftDownLng(CQ_LEFT_DOWN_LNG);
            reqVO.setLeftDownLat(CQ_LEFT_DOWN_LAT);
        }
        if (reqVO.getRightUpLng() == null && reqVO.getRightUpLat() == null) {
            reqVO.setRightUpLng(CQ_RIGHT_UP_LNG);
            reqVO.setRightUpLat(CQ_RIGHT_UP_LAT);
        }
        if (ObjectUtil.hasNull(reqVO.getLeftDownLng(), reqVO.getLeftDownLat(),
                reqVO.getRightUpLng(), reqVO.getRightUpLat(), reqVO.getMapGranularity())) {
            return Result.ofFail(QUERY_PARAM_MISS);
        }
        List<CockpitGridMapDataRespVO> cockpitGridMapDataRespVO = cockpitMapDAO.listCockpitGridMapData(reqVO);
        List<GridMapDataRespVO> respVO = cockpitGridMapDataRespVO.stream().map(o -> {
            GridMapDataRespVO respVOItem = new GridMapDataRespVO(o.getId(), o.getCenterLng(), o.getCenterLat(), o.getYouthNum());
            BigDecimal[][] coordinates = new BigDecimal[][]{
                    {o.getRightDownLng(), o.getRightDownLat()},
                    {o.getRightDownLng(), o.getLeftUpLat()},
                    {o.getLeftUpLng(), o.getLeftUpLat()},
                    {o.getLeftUpLng(), o.getRightDownLat()},
                    {o.getRightDownLng(), o.getRightDownLat()}
            };
            respVOItem.setCoordinates(coordinates);
            if (MAIN_AREA_NAME_SET.contains(o.getAreaName())) {
                respVOItem.setGradientFlag(1);
            } else if (OTHER_AREA_NAME_SET.contains(o.getAreaName())) {
                respVOItem.setGradientFlag(2);
            } else {
                respVOItem.setGradientFlag(0);
            }
            return respVOItem;
        }).collect(Collectors.toList());
        return Result.ofSuccess(respVO);
    }

    /**
     * 查询驾驶舱栅格图-人数极值
     *
     * @param reqVO 请求参数
     * @return 人数极值
     */
    @Override
    public Result<CockpitGridMapYouthNumRespVO> getCockpitGridMapYouthNum(CockpitGridMapDataReqVO reqVO) {
        return Result.ofSuccess(cockpitMapDAO.getCockpitGridMapYouthNum(reqVO));
    }

    /**
     * 查询驾驶舱阵地区域地图列表
     *
     * @param reqVO 请求参数
     * @return 驾驶舱阵地区域地图列表
     */
    @Override
    public Result<List<CockpitPositionAreaSumRespVO>> listCockpitPositionDataAreaSum(CockpitPositionDataReqVO reqVO) {
        if (reqVO.getTypes() == null) {
            return Result.ofFail(QUERY_PARAM_MISS);
        }
        List<CockpitPositionAreaSumRespVO> result = cockpitMapDAO.listCockpitPositionDataAreaSum(reqVO);
        CockpitPositionAreaSumRespVO citywide = new CockpitPositionAreaSumRespVO();
        citywide.setId(0L);
        citywide.setPositionType(String.join("、", reqVO.getTypes()));
        citywide.setAreaName("全市");
        citywide.setPositionNum(result.stream().mapToLong(CockpitPositionAreaSumRespVO::getPositionNum).sum());
        result.add(0, citywide);
        return Result.ofSuccess(result);
    }

    /**
     * 查询驾驶舱阵地地图列表
     *
     * @param reqVO 请求参数
     * @return 驾驶舱阵地地图列表
     */
    @Override
    public Result<List<CockpitPositionDataRespVO>> listCockpitPositionMapData(CockpitPositionDataReqVO reqVO) {

        if (reqVO.getLeftDownLng() == null && reqVO.getLeftDownLat() == null) {
            reqVO.setLeftDownLng(CQ_LEFT_DOWN_LNG);
            reqVO.setLeftDownLat(CQ_LEFT_DOWN_LAT);
        }
        if (reqVO.getRightUpLng() == null && reqVO.getRightUpLat() == null) {
            reqVO.setRightUpLng(CQ_RIGHT_UP_LNG);
            reqVO.setRightUpLat(CQ_RIGHT_UP_LAT);
        }
        if (ObjectUtil.hasNull(reqVO.getLeftDownLng(), reqVO.getLeftDownLat(),
                reqVO.getRightUpLng(), reqVO.getRightUpLat(), reqVO.getTypes())) {
            return Result.ofFail(QUERY_PARAM_MISS);
        }
        return Result.ofSuccess(cockpitMapDAO.listCockpitPositionMapData(reqVO));
    }

    /**
     * 阵地地图数据同步
     * 使用map_position_youth_cq表中的数据更新map_area_flag表
     *
     * @return 更新行数
     */
    @Override
    public Result<Integer> positionToAreaFlag() {
        return Result.ofSuccess(cockpitMapDAO.positionToAreaFlag());
    }

}
