package com.crunii.micro.service.sms.service.impl;

import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.ObjectUtil;
import com.aliyun.dysmsapi20170525.Client;
import com.aliyun.dysmsapi20170525.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teaopenapi.models.Config;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.domain.entity.main.YcqSmsInfo;
import com.crunii.micro.domain.repo.main.YcqSmsInfoRepo;
import com.crunii.micro.service.sms.constants.SmsConstants;
import com.crunii.micro.service.sms.jdbc.TestJdbcDao;
import com.crunii.micro.service.sms.service.SmsService;
import com.crunii.micro.service.sms.util.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.regex.Pattern;

import static com.aliyun.teautil.Common.assertAsString;
import static com.crunii.micro.service.sms.constants.SmsConstants.SMS_SUCCESS;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/28 17:32
 */
@Service
@Slf4j
public class SmsServiceImpl implements SmsService {

    @Value("${sms.accessKeyId}")
    private String accessKeyId;

    @Value("${sms.accessKeySecret}")
    private String accessKeySecret;

    @Value("${sms.signName}")
    private String signName;

    @Value("${sms.templateCode}")
    private String templateCode;

    @Value("${sms.validityTime}")
    private Integer smsValidityTime;

    @Value("${sms.url}")
    private String endPoint;

    @Value("${sms.protocol}")
    private String protocol;

    private int defaultPageSize = 50;

    @Autowired
    private RedisUtils redisUtils;

    @Resource
    private YcqSmsInfoRepo ycqSmsInfoRepo;

    @Resource
    private TestJdbcDao testJdbcDao;

    public void updateTestFile() {
        try {
            testJdbcDao.updateTestFile();
        } catch (Exception e) {

        }
    }

    /**
     * 使用AK&SK初始化账号Client
     */
    public Client createClient(String accessKeyId, String accessKeySecret) throws Exception {
        Config config = new Config().setAccessKeyId(accessKeyId).setAccessKeySecret(accessKeySecret);
        config.endpoint = "dysmsapi.aliyuncs.com";
        String env = System.getProperty("spring.profiles.active");
        if (env.equals("h3cTest")) {
            // 访问的域名
            config.endpoint = endPoint;
            config.setProtocol(protocol);
            config.setSignatureAlgorithm("v2");
        }
        return new Client(config);
    }

    public String smsCode(String phone) throws Exception {
        Map<String, String> jsonParam = new HashMap<>(3);
        String verifyCode = getVerifyCode();
        jsonParam.put("code", verifyCode);
        Client client = createClient(accessKeyId, accessKeySecret);
        SendSmsRequest sendSmsRequest = new SendSmsRequest()
                // 手机号
                .setPhoneNumbers(phone)
                // 短信模板CODE
                .setTemplateCode(templateCode)
                // 短信模板变量对应的实际值
                .setTemplateParam(JsonUtil.getJsonString(jsonParam))
                // 短信签名名称
                .setSignName(signName);
        try {
            SendSmsResponse response = client.sendSms(sendSmsRequest);
            SendSmsResponseBody body = response.getBody();
            if (ObjectUtil.isNotEmpty(body) && SMS_SUCCESS.equals(body.getCode())) {
                YcqSmsInfo info = new YcqSmsInfo();
                info.setPhone(DesensitizedUtil.mobilePhone(phone));
                info.setContent(templateCode);
                info.setSendTime(new Date());
                info.setCode(verifyCode);
                info.setBizId(body.getBizId());
                info.setMessage(body.getMessage());
                info.setRequestId(body.getRequestId());
                ycqSmsInfoRepo.saveAndFlush(info);
                return verifyCode;
            } else {
                log.info("短信发送失败,{}", body.getMessage());
            }
        } catch (Exception error) {
            TeaException error1 = new TeaException(error.getMessage(), error);
            log.info("短信发送异常,{},io->{},协议->{}", assertAsString(error1.message), endPoint, protocol);
        }
        return null;
    }


    @Override
    public Result<String> sendSmsCode(String phone) throws Exception {
        Result<String> result = new Result<>();

        if (ObjectUtil.isEmpty(phone)) {
            return Result.ofFail("手机号码不能为空");
        }

        if (!Pattern.matches(SmsConstants.REGULAR, phone)) {
            return Result.ofFail("手机号码输入有误，请重新输入");
        }

        String codeKey = phone + "YANG-SMS-CODE";
        String messageCode = (String) redisUtils.getValue(codeKey);

        //验证码未过期,且重新获取验证码
        if (ObjectUtil.isNotEmpty(messageCode)) {
            //获取key生存时间
            long expireTime = redisUtils.expire(codeKey);
            long lastTime = (60L * smsValidityTime) - expireTime;
            if (lastTime > 60 && expireTime > 0) {
                //删除未失效的key
                redisUtils.del(codeKey);
            }
            //一分钟之内不得多次获取验证码
            if (lastTime < 60L) {
                return Result.ofFail("操作过于频繁，请一分钟之后再次点击发送");
            }
        }
        // 请求发送短信
        String smsCode = smsCode(phone);
        if (ObjectUtil.isNotEmpty(smsCode)) {
            redisUtils.set(codeKey, smsCode, 5);
            result.setData(smsCode);
            result.setMsg("验证码下发成功!");
            return result;
        }
        return Result.ofFail("验证码发送失败");
    }

    /**
     * 随机获取6位短信数字验证码
     *
     * @return 验证码
     */
    public static String getVerifyCode() {
        Random random = new Random();
        StringBuilder code = new StringBuilder();
        for (int i = 0; i < 6; i++) {
            int rand = random.nextInt(10);
            code.append(rand);
        }
        return code.toString();
    }

    public void querySmsTemplateList() throws Exception {
        Client client = createClient(accessKeyId, accessKeySecret);
        int pageIndex = 1;
        boolean hasMoreData = true;

        while (hasMoreData) {
            QuerySmsTemplateListRequest req = new QuerySmsTemplateListRequest()
                    .setPageIndex(pageIndex)
                    .setPageSize(defaultPageSize);
            QuerySmsTemplateListResponse resp = client.querySmsTemplateList(req);

            Long totalCount1 = resp.getBody().getTotalCount();
            List<QuerySmsTemplateListResponseBody.QuerySmsTemplateListResponseBodySmsTemplateList> smsTemplateList = resp.getBody().getSmsTemplateList();


            if (totalCount1 > (pageIndex * defaultPageSize)) {
                pageIndex++;
            } else {
                hasMoreData = false;
            }
        }

    }

    /**
     * @MethodName sendMessage
     * @Description 通手机号发送短信
     * @param: phone
     * @return: com.crunii.micro.common.dto.Result<java.lang.String>
     * @Author Jiang
     * @Date 2023/10/26 18:45
     */
    @Override
    public Result<String> sendMessage(String phone, String content) {
        return new Result<>("验证码发送成功", Boolean.TRUE);
    }

    /**
     * @MethodName sendMessage
     * @Description 通手机号+短信模板+填充数据发送阿里云短信
     * @param: phone
     * @return: com.crunii.micro.common.dto.Result<java.lang.String>
     * @Author Jiang
     * @Date 2023/10/26 18:45
     */
    public Result<String> sendSms(String phone, String templateCode, String fillData)  {
        try {
            Client client = createClient(accessKeyId, accessKeySecret);

            SendSmsRequest sendSmsRequest = new SendSmsRequest()
                    .setPhoneNumbers(phone)
                    .setTemplateCode(templateCode)
                    .setTemplateParam(fillData)
                    .setSignName(signName);
            SendSmsResponse response = client.sendSms(sendSmsRequest);
            SendSmsResponseBody body = response.getBody();
            if (ObjectUtil.isNotEmpty(body) && SMS_SUCCESS.equals(body.getCode())) {
                YcqSmsInfo info = new YcqSmsInfo();
                info.setPhone(DesensitizedUtil.mobilePhone(phone));
                info.setContent(templateCode);
                info.setSendTime(new Date());

                info.setBizId(body.getBizId());
                info.setMessage(body.getMessage());
                info.setRequestId(body.getRequestId());
                ycqSmsInfoRepo.saveAndFlush(info);
                return Result.ofSuccessMsg("短信发送成功");
            } else {
                log.info("短信发送失败,{}", body.getMessage());
                return Result.ofFail("短信发送失败");
            }
        } catch (Exception error) {
            TeaException error1 = new TeaException(error.getMessage(), error);
            log.info("短信发送异常,{},io->{},协议->{}", assertAsString(error1.message), endPoint, protocol);
            return Result.ofFail("短信发送异常");
        }
    }
}
