/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.sso.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.common.utils.RSAUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.LoginDto;
import com.crunii.micro.feign.client.dto.LoginPhoneDto;
import com.crunii.micro.feign.client.dto.SmsCodeSend;
import com.crunii.micro.service.sso.SSOConstant;
import com.crunii.micro.service.sso.config.SSOCacheProperty;
import com.crunii.micro.service.sso.config.Sm2SecretKeyProperties;
import com.crunii.micro.service.sso.config.Sm2SecretProperty;
import com.crunii.micro.service.sso.dto.*;
import com.crunii.micro.service.sso.service.ParamCheckService;
import com.crunii.micro.service.sso.service.SSOCacheService;
import com.crunii.micro.service.sso.service.SSOService;
import com.crunii.micro.service.sso.utils.Sm2Util;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RMapCache;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author shenhs create 2023/6/2 18:38
 */
@Slf4j
@Service
public class SSOServiceImpl implements SSOService {

    @Autowired
    private Sm2SecretKeyProperties properties;

    @Autowired
    private SSOCacheProperty ssoCacheProperty;

    @Autowired
    private ParamCheckService paramCheckService;

    @Autowired
    private SSOCacheService cacheService;

    @Autowired
    private RedissonClient redisson;

    @Autowired
    private SysService sysService;

    /**
     * // 最大密码登录错误次数，超过后单位时间内不能登录
     */
    //@Value("${login.pwd.max.error.times:5}")
    public int loginPwdMaxErrorTimes = 5;

    /**
     * // 单位小时 登录密码错误后多少时间不能从新登录
     */
    //@Value("${login.pwd.max.error.period:5}")
    public int loginPwdMaxErrorPeriod = 1;

    @Override
    public Result<Object> generateAuthCode(Worker worker, String sysCode) {
        // 参数校验
        Result<Object> result = paramCheckService.checkAuthCodeParam(sysCode);
        if (result != null) {
            return result;
        }
        // 组装authCode对象，获取authCode
        AuthCode authCodeObj = new AuthCode(sysCode, System.currentTimeMillis());
        String authCode = getAuthCode(authCodeObj);

        // 将登录信息写入缓存
        cacheService.put(authCode, worker);
        // 加号全部替换为%20
        String replaceAuthCode = authCode.replace(SSOConstant.ADD_STR, SSOConstant.ENC_STR);
        // 拼接拓展参数
        String extendParam = properties.get(sysCode).getExtendParam();
        authCode = replaceAuthCode + extendParam;
        log.info("generateAuthCode {} {} ", sysCode, authCode);
        return Result.ofSuccess(authCode);
    }

    @Override
    public Result<Object> checkAuthCode(CheckAuthCode checkAuthCode) {
        log.info("checkAuthCode {} {} {}", checkAuthCode.getSysCode(), checkAuthCode.getTransactionId(), checkAuthCode.getAuthCode());
        Result<Object> result = paramCheckService.checkAuthCodeParam(checkAuthCode.getSysCode());
        if (result != null) {
            return result;
        }
        String token = cacheService.getToken(checkAuthCode.getAuthCode());
        if (StrUtil.isBlank(token)) {
            return paramCheckService.getResult(SSOConstant.ERROR_AUTH_CODE);
        }
        TokenInfo tokenInfo = new TokenInfo(token, ssoCacheProperty.getTokenTimeout());
        String jsonString = JsonUtil.getJsonString(tokenInfo);
        Sm2SecretProperty sm2SecretProperty = properties.get(checkAuthCode.getSysCode());
        String tokenEncStr = Sm2Util.encrypt(sm2SecretProperty.getPublicKey(),sm2SecretProperty.isHex(), jsonString);
        return Result.ofSuccess(tokenEncStr);
    }

    @Override
    public Result<Object> getUserInfo(UserInfoReq req) {
        log.info("getUserInfo {} {}", req.getTransactionId(), req.getEncode());
        String encode = req.getEncode();
        String transactionId = req.getTransactionId();
        String sysCode = transactionId.substring(0, transactionId.length() - 20);
        Sm2SecretProperty sm2SecretProperty = properties.get(sysCode);
        String checkTokenJsonStr = "";
        try {
            checkTokenJsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(),sm2SecretProperty.isHex(), encode);
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        CheckToken checkToken = JsonUtil.jsonToObject(checkTokenJsonStr, CheckToken.class);
        log.info("getUserInfo {} {} ", req.getTransactionId(), JsonUtil.getJsonString(checkToken));
        String token = checkToken.getToken();
        SSOWorkerInfo workerInfo = cacheService.getWorkerInfo(token);
        if (workerInfo == null) {
            log.error("token 无效或过期 {}", token);
            return paramCheckService.getResult(SSOConstant.ERROR_TOKEN);
        }
        workerInfo.setOrgId("100233");
        workerInfo.setOrgName("维护青少年权益部");
        String encrypt = Sm2Util.encrypt(sm2SecretProperty.getPublicKey(),sm2SecretProperty.isHex(), JsonUtil.getJsonString(workerInfo));
        return Result.ofSuccess(encrypt);
    }

    @Override
    public Result<Object> doLogin(SSOLoginDto loginDto) {
        log.info("doLogin {} {} {}", loginDto.getSysCode(), loginDto.getTransactionId(), loginDto.getSysCode());
        Sm2SecretProperty sm2SecretProperty = properties.get(loginDto.getSysCode());
        RMapCache<Integer, Integer> loginErrorMap = redisson.getMapCache(RedisKeys.LOGIN_PWD_ERROR_KEY);

        Result<Object> result = paramCheckService.checkAuthCodeParam(loginDto.getSysCode());
        if (result != null) {
            return result;
        }
        String encode = loginDto.getEncode();
        //String transactionId = loginDto.getTransactionId();

        String loginJsonStr = "";
        try {
            loginJsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(),sm2SecretProperty.isHex(), encode);
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        Login login = JsonUtil.jsonToObject(loginJsonStr, Login.class);
        log.info("login {} {} ", loginDto.getTransactionId(), JsonUtil.getJsonString(loginJsonStr));
        if (login.getAuthMode() == null || ObjectUtil.isEmpty(login.getAuthKey())) {
            log.info("AuthMode或AuthKey为空");
            throw new BusinessException("登录失败");
        }
        Worker worker = null;
        if (login.getAuthMode() == 0) {
            // 密码登录
            String mobile = login.getPhoneNumber();
            LoginDto sysLoginDto = new LoginDto();
            sysLoginDto.setScope(CommonConstants.SCOPE_SSO);
            sysLoginDto.setUsername(mobile);
            sysLoginDto.setPassword(RSAUtil.encryptHex(login.getAuthKey()));

            Result<Worker> loginRes = sysService.login(sysLoginDto);
            if(loginRes.isSuccess()){
                worker = loginRes.getData();
            }else{
                return new Result<Object>(false, Result.DEFAULT_ERROR_CODE ,loginRes.getMsg());
            }
        } else if (login.getAuthMode() == 1) {
            //TODO  短信验证码登录
            // 调用接口 获得worker
            Worker smsCodeWorker = new Worker();
            LoginPhoneDto dto = new LoginPhoneDto();
            dto.setSmsCode(login.getAuthKey());
            dto.setMobile(login.getPhoneNumber());
            dto.setScope(CommonConstants.SCOPE_SSO);
            Result<Worker> loginRes = sysService.smsCodeLogin(dto);
            if(loginRes.isSuccess()){
                worker = loginRes.getData();
            }else{
                return new Result<Object>(false, Result.DEFAULT_ERROR_CODE ,loginRes.getMsg());
            }
        } else {
            log.error("未知登录方式,{}", login.getAuthMode());
            return Result.ofFail("登录失败");
        }
        if(worker!=null){
            // 随便获取一个字符串生成token
            String encrypt = Sm2Util.encrypt(sm2SecretProperty.getPublicKey(), sm2SecretProperty.isHex(), System.currentTimeMillis() + "");
            String token = cacheService.put(encrypt, worker);
            TokenInfo tokenInfo = new TokenInfo(token,ssoCacheProperty.getTokenTimeout());
            String jsonString = JsonUtil.getJsonString(tokenInfo);
            String tokenEncStr = Sm2Util.encrypt(sm2SecretProperty.getPublicKey(),sm2SecretProperty.isHex(), jsonString);
            return Result.ofSuccess(tokenEncStr);
        }

        return Result.ofFail("登录失败");
    }

    @Override public Result<String> sendSmsCode(BaseReq baseReq) {
        Sm2SecretProperty sm2SecretProperty = properties.get(baseReq.getSysCode());
        SmsCodeSend encodeObj = baseReq.getEncodeObj(sm2SecretProperty, SmsCodeSend.class);
        Result<String> phoneCode = sysService.getPhoneCode(encodeObj);
        return phoneCode;
    }

    private String getAuthCode(AuthCode authCode) {
        Sm2SecretProperty sm2SecretProperty = properties.get(authCode.getSysCode());
        Map<String, Object> map = new HashMap<>();
        map.put("sysCode",authCode.getSysCode());
        map.put("timestamp",System.currentTimeMillis());
        return Sm2Util.encrypt(sm2SecretProperty.getPublicKey(),sm2SecretProperty.isHex(), JsonUtil.getJsonString(map));
    }

    /**
     * 抛出异常curr状态
     *
     * @param curr curr
     */
    private void throwNewException(TsmWorker curr) {
        if (Objects.isNull(curr)) {
            throw new BusinessException("手机号失效");
        }
        if (curr.getState() == 0) {
            throw new BusinessException("手机号无效");
        }
        if (curr.getState() == 2) {
            throw new BusinessException("手机号已被锁定");
        }
    }
}
