package com.crunii.micro.service.studycenter.aop;

import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.studycenter.annotation.Desensitization;
import com.crunii.micro.service.studycenter.annotation.FiledDesensitization;
import com.crunii.micro.service.studycenter.enums.DesensitizationTypeEnum;
import com.crunii.micro.service.studycenter.enums.MaskingTypeEnum;
import com.crunii.micro.service.studycenter.util.SensitiveInfoUtils;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.util.ReflectionUtils;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Component
@Aspect
@Slf4j
/***
 * @author yingtiansheng
 * @date 2023/10/13 16:05
 * @description: 脱敏切面
 */
public class DesensitizationAspect {

    @Before("@annotation(com.crunii.micro.service.studycenter.annotation.Desensitization)")
    public void doAnnotationBefore(JoinPoint joinPoint)  {
        try {
            MethodSignature ms = (MethodSignature) joinPoint.getSignature();
            Desensitization value = ms.getMethod().getAnnotation(Desensitization.class);
            if (value.type().equals(DesensitizationTypeEnum.EXPORT)) {
                log.info(">>>>>>>>>>>>>>>>>导出方法脱敏");
                List<?> list = (List<?>) joinPoint.getArgs()[5];
                setDesensitizationListData(list);
            }
        } catch (Exception e) {
            log.error("", e);
        }
    }

    @Around("@annotation(com.crunii.micro.service.studycenter.annotation.Desensitization)")
    public Result<?> doAnnotationAround(ProceedingJoinPoint pjp)  {
        try {
            MethodSignature ms = (MethodSignature) pjp.getSignature();
            Desensitization value = ms.getMethod().getAnnotation(Desensitization.class);
            Result apiResult = (Result) pjp.proceed();
            if (value.type().equals(DesensitizationTypeEnum.QUERY)) {
                if (Objects.isNull(apiResult)||Objects.isNull(apiResult.getData())) {
                    return null;
                }
                Object proceed = apiResult.getData();
                String className = proceed.getClass().getName();
                log.info(">>>>>>>>>>>>>>>>>脱敏方法: {}执行前, 参数ClassName: {}",  pjp.getSignature().getDeclaringTypeName() + "." + pjp.getSignature().getName(), className);
                // 分页方法 PageRsp
                if ("com.crunii.micro.common.dto.PageRsp".equals(className)) {
                    PageRsp<?> pageInfo = (PageRsp<?>) proceed;
                    setDesensitizationListData(pageInfo.getResult());
                    return apiResult;
                }
                if ("java.util.ArrayList".equals(className)) {
                    setDesensitizationListData((List<?>) proceed);
                    return apiResult;
                }
                // 如果返回的是单个实体
                setDesensitizationData(proceed, proceed.getClass());
                apiResult.setData(proceed);
            }
            return apiResult;
        } catch (Throwable e) {
            log.error("", e);
            return Result.ofFail("系统异常");
        }
    }

    /***
     * @param list
     * @return void
     * @author guhongtao
     * @date 2022/1/12 11:08
     * @description: 脱敏集合处理
     */
    private void setDesensitizationListData(List<?> list) {
        try {
            for (Object object : list) {
                Class<?> aClass = object.getClass();
                setDesensitizationData(object, aClass);
            }
        } catch (IllegalAccessException e) {
            log.error("", e);
        }
    }

    /***
     * @param object 脱敏对象
     * @param aClass
     * @return void
     * @author guhongtao
     * @date 2022/1/12 11:06
     * @description: 反射根据字段重新设置脱敏值
     */
    private void setDesensitizationData(Object object, Class<?> aClass) throws IllegalAccessException {
        Field[] declaredFields = getAllFields(aClass);
        for (Field field : declaredFields) {
            if (field.isAnnotationPresent(FiledDesensitization.class)) {
                // setAccessible这行代码把对象上的私有字段为可以设置
                field.setAccessible(true);
                ReflectionUtils.makeAccessible(field);
                // 如果属性值为null
                if (Objects.isNull(field.get(object))) {
                    break;
                }
                String value = field.get(object).toString();
                FiledDesensitization annotation = field.getAnnotation(FiledDesensitization.class);
                MaskingTypeEnum type = annotation.type();
                switch (type) {
                    case ID_CARD:
                        field.set(object, SensitiveInfoUtils.idCardNum(value));
                        break;
                    case NAME:
                        field.set(object, SensitiveInfoUtils.chineseName(value));
                        break;
                    case PHONE:
                        field.set(object, SensitiveInfoUtils.mobilePhone(value));
                        break;
                    case EMAIL:
                        field.set(object, SensitiveInfoUtils.email(value));
                        break;
                }
            }
        }
    }

    public static Field[] getAllFields(Class<?> clazz) {
        List<Field> fields = new ArrayList<>();

        // 获取当前类的所有字段
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field field : declaredFields) {
            field.setAccessible(true);
            fields.add(field);
        }

        // 递归获取父类的所有字段
        Class<?> superClass = clazz.getSuperclass();
        if (superClass != null) {
            Field[] superFields = getAllFields(superClass);
            for (Field field : superFields) {
                field.setAccessible(true);
                fields.add(field);
            }
        }

        return fields.toArray(new Field[0]);
    }
}
